/* DO NOT EDIT THIS FILE
 * Automatically generated by toolchain/trunk/proc-defs/sh/create-arch-headers.sh
 * DO NOT EDIT THIS FILE
 */

#ifndef __MACH_DEF_BLACKFIN__
#define __MACH_DEF_BLACKFIN__

#ifdef __ADSPBF504__
# include "mach-bf506/BF504_def.h"
# include "mach-bf506/anomaly.h"
# include "mach-bf506/def_local.h"
#endif
#ifdef __ADSPBF506__
# include "mach-bf506/BF506_def.h"
# include "mach-bf506/anomaly.h"
# include "mach-bf506/def_local.h"
#endif
#ifdef __ADSPBF512__
# include "mach-bf518/BF512_def.h"
# include "mach-bf518/anomaly.h"
# include "mach-bf518/def_local.h"
#endif
#ifdef __ADSPBF514__
# include "mach-bf518/BF514_def.h"
# include "mach-bf518/anomaly.h"
# include "mach-bf518/def_local.h"
#endif
#ifdef __ADSPBF516__
# include "mach-bf518/BF516_def.h"
# include "mach-bf518/anomaly.h"
# include "mach-bf518/def_local.h"
#endif
#ifdef __ADSPBF518__
# include "mach-bf518/BF518_def.h"
# include "mach-bf518/anomaly.h"
# include "mach-bf518/def_local.h"
#endif
#ifdef __ADSPBF522__
# include "mach-bf527/BF522_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF523__
# include "mach-bf527/BF523_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF524__
# include "mach-bf527/BF524_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF525__
# include "mach-bf527/BF525_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF526__
# include "mach-bf527/BF526_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF527__
# include "mach-bf527/BF527_def.h"
# include "mach-bf527/anomaly.h"
# include "mach-bf527/def_local.h"
#endif
#ifdef __ADSPBF531__
# include "mach-bf533/BF531_def.h"
# include "mach-bf533/anomaly.h"
# include "mach-bf533/def_local.h"
#endif
#ifdef __ADSPBF532__
# include "mach-bf533/BF532_def.h"
# include "mach-bf533/anomaly.h"
# include "mach-bf533/def_local.h"
#endif
#ifdef __ADSPBF533__
# include "mach-bf533/BF533_def.h"
# include "mach-bf533/anomaly.h"
# include "mach-bf533/def_local.h"
#endif
#ifdef __ADSPBF534__
# include "mach-bf537/BF534_def.h"
# include "mach-bf537/anomaly.h"
# include "mach-bf537/def_local.h"
#endif
#ifdef __ADSPBF536__
# include "mach-bf537/BF536_def.h"
# include "mach-bf537/anomaly.h"
# include "mach-bf537/def_local.h"
#endif
#ifdef __ADSPBF537__
# include "mach-bf537/BF537_def.h"
# include "mach-bf537/anomaly.h"
# include "mach-bf537/def_local.h"
#endif
#ifdef __ADSPBF538__
# include "mach-bf538/BF538_def.h"
# include "mach-bf538/anomaly.h"
# include "mach-bf538/def_local.h"
#endif
#ifdef __ADSPBF539__
# include "mach-bf538/BF539_def.h"
# include "mach-bf538/anomaly.h"
# include "mach-bf538/def_local.h"
#endif
#ifdef __ADSPBF542__
# include "mach-bf548/BF542_def.h"
# include "mach-bf548/anomaly.h"
# include "mach-bf548/def_local.h"
#endif
#ifdef __ADSPBF544__
# include "mach-bf548/BF544_def.h"
# include "mach-bf548/anomaly.h"
# include "mach-bf548/def_local.h"
#endif
#ifdef __ADSPBF547__
# include "mach-bf548/BF547_def.h"
# include "mach-bf548/anomaly.h"
# include "mach-bf548/def_local.h"
#endif
#ifdef __ADSPBF548__
# include "mach-bf548/BF548_def.h"
# include "mach-bf548/anomaly.h"
# include "mach-bf548/def_local.h"
#endif
#ifdef __ADSPBF549__
# include "mach-bf548/BF549_def.h"
# include "mach-bf548/anomaly.h"
# include "mach-bf548/def_local.h"
#endif
#ifdef __ADSPBF561__
# include "mach-bf561/BF561_def.h"
# include "mach-bf561/anomaly.h"
# include "mach-bf561/def_local.h"
#endif
#ifdef __ADSPBF609__
# include "mach-bf609/BF609_def.h"
# include "mach-bf609/anomaly.h"
# include "mach-bf609/def_local.h"
#endif

#endif /* __MACH_DEF_BLACKFIN__ */
