/* DO NOT EDIT THIS FILE
 * Automatically generated by generate-def-headers.xsl
 * DO NOT EDIT THIS FILE
 */

#ifndef __BFIN_DEF_ADSP_BF544_proc__
#define __BFIN_DEF_ADSP_BF544_proc__

#include "../mach-common/ADSP-EDN-core_def.h"

#include "ADSP-EDN-BF544-extended_def.h"

#define CHIPID                         0xFFC00014
#define SWRST                          0xFFC00100 /* Software Reset Register */
#define SYSCR                          0xFFC00104 /* System Configuration register */

#endif /* __BFIN_DEF_ADSP_BF544_proc__ */
