/*
 * U-boot - deferred register layout
 *
 * Copyright 2004-2009 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 */

#ifndef _BLACKFIN_DEFER_H
#define _BLACKFIN_DEFER_H

#define deferred_regs_DCPLB_FAULT_ADDR	0
#define deferred_regs_ICPLB_FAULT_ADDR	1
#define deferred_regs_retx		2
#define deferred_regs_SEQSTAT		3
#define deferred_regs_SYSCFG		4
#define deferred_regs_IMASK		5
#define deferred_regs_last		6

#endif /* _BLACKFIN_DEFER_H */
