/*
 * Copyright (C) 2013-2014 Synopsys, Inc. All rights reserved.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __ASM_ARC_SECTIONS_H
#define __ASM_ARC_SECTIONS_H

#include <asm-generic/sections.h>

extern ulong __text_end;
extern ulong __ivt_start;
extern ulong __ivt_end;
extern ulong __image_copy_start;

#endif /* __ASM_ARC_SECTIONS_H */
