/*
 * gpio_cfi_flash.c - GPIO-assisted Flash Chip Support
 *
 * Copyright (c) 2009 Analog Devices Inc.
 *
 * Licensed under the GPL-2 or later.
 */

void *gpio_cfi_flash_swizzle(void *vaddr);
void gpio_cfi_flash_init(void);
