/*
 * Copyright (C) 2011-2015 Masahiro Yamada <yamada.masahiro@socionext.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#include <mach/init.h>

int ph1_sld3_pll_init(const struct uniphier_board_data *bd)
{
	/* add pll init code here */
	return 0;
}
