/*
 * U-boot - setup.h
 *
 * Copyright (c) 2005-2007 Analog Devices Inc.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _SHARED_RESOURCES_H_
#define _SHARED_RESOURCES_H_

void swap_to(int device_id);

#define FLASH	 0
#define ETHERNET 1

#endif /* _SHARED_RESOURCES_H_ */
