/*
 * Copyright (C) 2013, Intel Corporation
 * Copyright (C) 2014, Bin Meng <bmeng.cn@gmail.com>
 *
 * SPDX-License-Identifier:	Intel
 */

#ifndef __FSP_PLATFORM_H__
#define __FSP_PLATFORM_H__

struct fspinit_rtbuf {
	struct common_buf	common;	/* FSP common runtime data structure */
};

#endif
