/*
 * Copyright (C) 2008 Analog Devices Inc.
 * Licensed under the GPL-2 or later.
 */


#ifndef _MACH_GPIO_H_
#define _MACH_GPIO_H_

#define MAX_BLACKFIN_GPIOS 48

#define GPIO_PF0	0
#define GPIO_PF1	1
#define GPIO_PF2	2
#define GPIO_PF3	3
#define GPIO_PF4	4
#define GPIO_PF5	5
#define GPIO_PF6	6
#define GPIO_PF7	7
#define GPIO_PF8	8
#define GPIO_PF9	9
#define GPIO_PF10	10
#define GPIO_PF11	11
#define GPIO_PF12	12
#define GPIO_PF13	13
#define GPIO_PF14	14
#define GPIO_PF15	15
#define GPIO_PG0	16
#define GPIO_PG1	17
#define GPIO_PG2	18
#define GPIO_PG3	19
#define GPIO_PG4	20
#define GPIO_PG5	21
#define GPIO_PG6	22
#define GPIO_PG7	23
#define GPIO_PG8	24
#define GPIO_PG9	25
#define GPIO_PG10	26
#define GPIO_PG11	27
#define GPIO_PG12	28
#define GPIO_PG13	29
#define GPIO_PG14	30
#define GPIO_PG15	31
#define GPIO_PH0	32
#define GPIO_PH1	33
#define GPIO_PH2	34
#define GPIO_PH3	35
#define GPIO_PH4	36
#define GPIO_PH5	37
#define GPIO_PH6	38
#define GPIO_PH7	39
#define GPIO_PH8	40
#define GPIO_PH9	41
#define GPIO_PH10	42
#define GPIO_PH11	43
#define GPIO_PH12	44
#define GPIO_PH13	45
#define GPIO_PH14	46
#define GPIO_PH15	47

#define PORT_F GPIO_PF0
#define PORT_G GPIO_PG0
#define PORT_H GPIO_PH0

#endif /* _MACH_GPIO_H_ */
