/*
 * (C) Copyright 2008
 *  Ricado Ribalda-Universidad Autonoma de Madrid-ricardo.ribalda@uam.es
 *  This work has been supported by: QTechnology  http://qtec.com/
 * SPDX-License-Identifier:	GPL-2.0+
*/

#ifndef __CONFIG_GEN_H
#define __CONFIG_GEN_H

/*CPU*/
#define CONFIG_440		1
#define CONFIG_XILINX_440	1

#include <configs/xilinx-ppc.h>

#endif						/* __CONFIG_H */
