/*
 * Port E Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_E__
#define __BFIN_PERIPHERAL_PORT_E__

#define PE0		(1 << 0)
#define PE1		(1 << 1)
#define PE2		(1 << 2)
#define PE3		(1 << 3)
#define PE4		(1 << 4)
#define PE5		(1 << 5)
#define PE6		(1 << 6)
#define PE7		(1 << 7)
#define PE8		(1 << 8)
#define PE9		(1 << 9)
#define PE10		(1 << 10)
#define PE11		(1 << 11)
#define PE12		(1 << 12)
#define PE13		(1 << 13)
#define PE14		(1 << 14)
#define PE15		(1 << 15)

#endif
