/*
 * Port A Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT_A__
#define __BFIN_PERIPHERAL_PORT_A__

#define PA0		(1 << 0)
#define PA1		(1 << 1)
#define PA2		(1 << 2)
#define PA3		(1 << 3)
#define PA4		(1 << 4)
#define PA5		(1 << 5)
#define PA6		(1 << 6)
#define PA7		(1 << 7)
#define PA8		(1 << 8)
#define PA9		(1 << 9)
#define PA10		(1 << 10)
#define PA11		(1 << 11)
#define PA12		(1 << 12)
#define PA13		(1 << 13)
#define PA14		(1 << 14)
#define PA15		(1 << 15)

#endif
