/*
 * Copyright (C) 2015 Synopsys, Inc. All rights reserved.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _BOARD_SYNOPSYS_AXS10X_H
#define _BOARD_SYNOPSYS_AXS10X_H

enum {
	AXS_MB_V2,
	AXS_MB_V3
};

#endif /* _BOARD_SYNOPSYS_AXS10X_H */

