/*
 * Port Masks
 */

#ifndef __BFIN_PERIPHERAL_PORT__
#define __BFIN_PERIPHERAL_PORT__

#include "../mach-common/bits/ports-f.h"

/* The non-standard PF16+ */
#define PF16		(1 << 0)
#define PF17		(1 << 1)
#define PF18		(1 << 2)
#define PF19		(1 << 3)
#define PF20		(1 << 4)
#define PF21		(1 << 5)
#define PF22		(1 << 6)
#define PF23		(1 << 7)
#define PF24		(1 << 8)
#define PF25		(1 << 9)
#define PF26		(1 << 10)
#define PF27		(1 << 11)
#define PF28		(1 << 12)
#define PF29		(1 << 13)
#define PF30		(1 << 14)
#define PF31		(1 << 15)
#define PF32		(1 << 0)
#define PF33		(1 << 1)
#define PF34		(1 << 2)
#define PF35		(1 << 3)
#define PF36		(1 << 4)
#define PF37		(1 << 5)
#define PF38		(1 << 6)
#define PF39		(1 << 7)
#define PF40		(1 << 8)
#define PF41		(1 << 9)
#define PF42		(1 << 10)
#define PF43		(1 << 11)
#define PF44		(1 << 12)
#define PF45		(1 << 13)
#define PF46		(1 << 14)
#define PF47		(1 << 15)

#endif
