/*
 *
 * (C) Copyright 2008
 * Ricado Ribalda-Universidad Autonoma de Madrid-ricardo.ribalda@uam.es
 * This work has been supported by: QTechnology http://qtec.com/
 *
 * (C) Copyright 2008
 * Georg Schardt <schardt@team-ctech.de>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __CONFIG_H
#define __CONFIG_H

/* cpu parameter */
#define CONFIG_405		1
#define CONFIG_XILINX_405	1

#include <configs/xilinx-ppc.h>

#endif
