/*
 * (C) Copyright 2012
 * Texas Instruments, <www.ti.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */
#ifndef	_ASM_SPL_H_
#define	_ASM_SPL_H_

#define BOOT_DEVICE_NOR		1

/* Linker symbols */
extern char __bss_start[], __bss_end[];

#endif
