/*
 * (C) Copyright 2010-2015
 * NVIDIA Corporation <www.nvidia.com>
 *
 * SPDX-License-Identifier:     GPL-2.0+
 */

#ifndef _TEGRA210_PMU_H_
#define _TEGRA210_PMU_H_

/* Set core and CPU voltages to nominal levels */
int pmu_set_nominal(void);

#endif	/* _TEGRA210_PMU_H_ */
