/*
 * (C) Copyright 2014
 * Dirk Eibach,  Guntermann & Drunck GmbH, dirk.eibach@gdsys.cc
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _IOEP_FPGA_H_
#define _IOEP_FPGA_H_

void ioep_fpga_print_info(unsigned int fpga);
bool ioep_fpga_has_osd(unsigned int fpga);

#endif
