/* DO NOT EDIT THIS FILE
 * Automatically generated by generate-cdef-headers.xsl
 * DO NOT EDIT THIS FILE
 */

#ifndef __BFIN_CDEF_ADSP_BF506_proc__
#define __BFIN_CDEF_ADSP_BF506_proc__

#include "BF504_cdef.h"

#endif /* __BFIN_CDEF_ADSP_BF506_proc__ */
