/*
 * (C) Copyright 2012 Samsung Electronics
 * Donghwa Lee <dh09.lee@samsung.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _LIBTIZEN_H_
#define _LIBTIZEN_H_

#define HD_RESOLUTION	0

void get_tizen_logo_info(vidinfo_t *vid);

#endif	/* _LIBTIZEN_H_ */
