/*
 * (C) Copyright 2012,2015 Stephen Warren
 *
 * SPDX-License-Identifier:	GPL-2.0
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#define CONFIG_SKIP_LOWLEVEL_INIT
#define CONFIG_BCM2836
#define CONFIG_SYS_CACHELINE_SIZE		64
#define CONFIG_SYS_DCACHE_OFF

#include "rpi-common.h"

#endif
