/*
 * (C) Copyright 2011
 * Graeme Russ, <graeme.russ@gmail.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _INIT_HELPERS_H_
#define _INIT_HELPERS_H_

int init_cache_f_r(void);
int init_bd_struct_r(void);
int init_func_spi(void);

#endif	/* !_INIT_HELPERS_H_ */
