/*
 * Copyright (C) 2006 Atmel Corporation
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */
#ifndef __ASM_AVR32_STRING_H
#define __ASM_AVR32_STRING_H

#define __HAVE_ARCH_MEMSET
extern void *memset(void *s, int c, __kernel_size_t n);

#endif /* __ASM_AVR32_STRING_H */
