/*
 * (C) Copyright 2011 Linaro
 * Ryan Harkin, <ryan.harkin@linaro.org>
 *
 * Configuration for Versatile Express. Parts were derived from other ARM
 *   configurations.
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __VEXPRESS_CA5X2_h
#define __VEXPRESS_CA5X2_h

#define CONFIG_VEXPRESS_EXTENDED_MEMORY_MAP
#define CONFIG_BOOTP_VCI_STRING     "U-boot.armv7.vexpress_ca5x2"
#include "vexpress_common.h"

#endif /* __VEXPRESS_CA5X2_h */
