/*
 * (C) Copyright 2014
 * Dirk Eibach,  Guntermann & Drunck GmbH, dirk.eibach@gdsys.cc
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _CH7301_H_
#define _CH7301_H_

int ch7301_probe(unsigned screen, bool power);

#endif
