/*
 * Copyright (C) 2013 Imagination Technologies
 * Author: Paul Burton <paul.burton@imgtec.com>
 *
 * Setup code for the FDC37M817 super I/O controller
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef __BOARD_MALTA_SUPERIO_H__
#define __BOARD_MALTA_SUPERIO_H__

extern void malta_superio_init(void *io_base);

#endif /* __BOARD_MALTA_SUPERIO_H__ */
