/*
 * Copyright (C) 2014, Bin Meng <bmeng.cn@gmail.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _X86_ARCH_GPIO_H_
#define _X86_ARCH_GPIO_H_

/* Where in config space is the register that points to the GPIO registers? */
#define PCI_CFG_GPIOBASE 0x48

#endif /* _X86_ARCH_GPIO_H_ */
