/*
 * (C) Copyright 2013 - 2014 Xilinx, Inc
 *
 * Michal Simek <michal.simek@xilinx.com>
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _ASM_MICROBLAZE_SPL_H_
#define _ASM_MICROBLAZE_SPL_H_

#define BOOT_DEVICE_RAM		1
#define BOOT_DEVICE_NOR		2
#define BOOT_DEVICE_SPI		3

#endif
