/*
 * (C) Copyright 2010
 * Dirk Eibach,  Guntermann & Drunck GmbH, eibach@gdsys.de
 *
 * SPDX-License-Identifier:	GPL-2.0+
 */

#ifndef _OSD_H_
#define _OSD_H_

int ch7301_probe(unsigned screen, bool power);
int osd_probe(unsigned screen);

#endif
