/*
 * This was automagically generated from arch/arm/tools/mach-types!
 * Do NOT edit
 */

#ifndef __ASM_ARM_MACH_TYPE_H
#define __ASM_ARM_MACH_TYPE_H

#ifndef __ASSEMBLY__
/* The type of machine we're running on */
extern unsigned int __machine_arch_type;
#endif

/* see arch/arm/kernel/arch.c for a description of these */
#define MACH_TYPE_EBSA110              0
#define MACH_TYPE_RISCPC               1
#define MACH_TYPE_EBSA285              4
#define MACH_TYPE_NETWINDER            5
#define MACH_TYPE_CATS                 6
#define MACH_TYPE_SHARK                15
#define MACH_TYPE_BRUTUS               16
#define MACH_TYPE_PERSONAL_SERVER      17
#define MACH_TYPE_L7200                19
#define MACH_TYPE_PLEB                 20
#define MACH_TYPE_INTEGRATOR           21
#define MACH_TYPE_H3600                22
#define MACH_TYPE_P720T                24
#define MACH_TYPE_ASSABET              25
#define MACH_TYPE_LART                 27
#define MACH_TYPE_GRAPHICSCLIENT       29
#define MACH_TYPE_XP860                30
#define MACH_TYPE_CERF                 31
#define MACH_TYPE_NANOENGINE           32
#define MACH_TYPE_JORNADA720           48
#define MACH_TYPE_EDB7211              50
#define MACH_TYPE_PFS168               52
#define MACH_TYPE_FLEXANET             54
#define MACH_TYPE_SIMPAD               87
#define MACH_TYPE_LUBBOCK              89
#define MACH_TYPE_CLEP7212             91
#define MACH_TYPE_SHANNON              97
#define MACH_TYPE_CONSUS               105
#define MACH_TYPE_AAED2000             106
#define MACH_TYPE_CDB89712             107
#define MACH_TYPE_GRAPHICSMASTER       108
#define MACH_TYPE_ADSBITSY             109
#define MACH_TYPE_PXA_IDP              110
#define MACH_TYPE_PT_SYSTEM3           112
#define MACH_TYPE_AUTCPU12             118
#define MACH_TYPE_H3100                136
#define MACH_TYPE_COLLIE               146
#define MACH_TYPE_BADGE4               148
#define MACH_TYPE_FORTUNET             152
#define MACH_TYPE_MX1ADS               160
#define MACH_TYPE_H7201                161
#define MACH_TYPE_H7202                162
#define MACH_TYPE_IQ80321              169
#define MACH_TYPE_KS8695               180
#define MACH_TYPE_SMDK2410             193
#define MACH_TYPE_CEIVA                200
#define MACH_TYPE_VOICEBLUE            218
#define MACH_TYPE_H5400                220
#define MACH_TYPE_OMAP_INNOVATOR       234
#define MACH_TYPE_IXDP2400             242
#define MACH_TYPE_IXDP2800             243
#define MACH_TYPE_IXDP425              245
#define MACH_TYPE_HACKKIT              254
#define MACH_TYPE_IXCDP1100            260
#define MACH_TYPE_AT91RM9200DK         262
#define MACH_TYPE_CINTEGRATOR          275
#define MACH_TYPE_VIPER                283
#define MACH_TYPE_ADI_COYOTE           290
#define MACH_TYPE_IXDP2401             299
#define MACH_TYPE_IXDP2801             300
#define MACH_TYPE_IQ31244              327
#define MACH_TYPE_BAST                 331
#define MACH_TYPE_H1940                347
#define MACH_TYPE_ENP2611              356
#define MACH_TYPE_S3C2440              362
#define MACH_TYPE_GUMSTIX              373
#define MACH_TYPE_OMAP_H2              382
#define MACH_TYPE_E740                 384
#define MACH_TYPE_IQ80331              385
#define MACH_TYPE_VERSATILE_PB         387
#define MACH_TYPE_KEV7A400             388
#define MACH_TYPE_LPD7A400             389
#define MACH_TYPE_LPD7A404             390
#define MACH_TYPE_CSB337               399
#define MACH_TYPE_MAINSTONE            406
#define MACH_TYPE_XCEP                 413
#define MACH_TYPE_ARCOM_VULCAN         414
#define MACH_TYPE_NOMADIK              420
#define MACH_TYPE_CORGI                423
#define MACH_TYPE_POODLE               424
#define MACH_TYPE_ARMCORE              438
#define MACH_TYPE_MX31ADS              447
#define MACH_TYPE_HIMALAYA             448
#define MACH_TYPE_EDB9312              451
#define MACH_TYPE_OMAP_GENERIC         452
#define MACH_TYPE_EDB9301              462
#define MACH_TYPE_EDB9315              463
#define MACH_TYPE_VR1000               475
#define MACH_TYPE_OMAP_PERSEUS2        491
#define MACH_TYPE_E800                 496
#define MACH_TYPE_E750                 497
#define MACH_TYPE_SCB9328              508
#define MACH_TYPE_OMAP_H3              509
#define MACH_TYPE_OMAP_H4              510
#define MACH_TYPE_OMAP_OSK             515
#define MACH_TYPE_TOSA                 520
#define MACH_TYPE_AVILA                526
#define MACH_TYPE_EDB9302              538
#define MACH_TYPE_HUSKY                543
#define MACH_TYPE_SHEPHERD             545
#define MACH_TYPE_H4700                562
#define MACH_TYPE_RX3715               592
#define MACH_TYPE_NSLU2                597
#define MACH_TYPE_E400                 598
#define MACH_TYPE_IXDPG425             604
#define MACH_TYPE_VERSATILE_AB         606
#define MACH_TYPE_EDB9307              607
#define MACH_TYPE_KB9200               612
#define MACH_TYPE_SX1                  613
#define MACH_TYPE_IXDP465              618
#define MACH_TYPE_IXDP2351             619
#define MACH_TYPE_IQ80332              629
#define MACH_TYPE_GTWX5715             641
#define MACH_TYPE_CSB637               648
#define MACH_TYPE_N30                  656
#define MACH_TYPE_NEC_MP900            659
#define MACH_TYPE_KAFA                 662
#define MACH_TYPE_TS72XX               673
#define MACH_TYPE_OTOM                 680
#define MACH_TYPE_NEXCODER_2440        681
#define MACH_TYPE_ECO920               702
#define MACH_TYPE_ROADRUNNER           704
#define MACH_TYPE_AT91RM9200EK         705
#define MACH_TYPE_SPITZ                713
#define MACH_TYPE_ADSSPHERE            723
#define MACH_TYPE_COLIBRI              729
#define MACH_TYPE_GATEWAY7001          731
#define MACH_TYPE_PCM027               732
#define MACH_TYPE_ANUBIS               734
#define MACH_TYPE_AKITA                744
#define MACH_TYPE_E330                 753
#define MACH_TYPE_NOKIA770             755
#define MACH_TYPE_CARMEVA              769
#define MACH_TYPE_EDB9315A             772
#define MACH_TYPE_STARGATE2            774
#define MACH_TYPE_INTELMOTE2           775
#define MACH_TYPE_TRIZEPS4             776
#define MACH_TYPE_PNX4008              782
#define MACH_TYPE_IQ81340SC            799
#define MACH_TYPE_IQ81340MC            801
#define MACH_TYPE_MICRO9               811
#define MACH_TYPE_MICRO9L              812
#define MACH_TYPE_OMAP_PALMTE          817
#define MACH_TYPE_REALVIEW_EB          827
#define MACH_TYPE_BORZOI               831
#define MACH_TYPE_PALMLD               835
#define MACH_TYPE_IXDP28X5             838
#define MACH_TYPE_OMAP_PALMTT          839
#define MACH_TYPE_ARCOM_ZEUS           841
#define MACH_TYPE_OSIRIS               842
#define MACH_TYPE_PALMTE2              844
#define MACH_TYPE_MX27ADS              846
#define MACH_TYPE_AT91SAM9261EK        848
#define MACH_TYPE_LOFT                 849
#define MACH_TYPE_MX21ADS              851
#define MACH_TYPE_AMS_DELTA            862
#define MACH_TYPE_NAS100D              865
#define MACH_TYPE_MAGICIAN             875
#define MACH_TYPE_NXDKN                880
#define MACH_TYPE_PALMTX               885
#define MACH_TYPE_S3C2413              887
#define MACH_TYPE_WG302V2              890
#define MACH_TYPE_OMAP_2430SDP         900
#define MACH_TYPE_DAVINCI_EVM          901
#define MACH_TYPE_PALMZ72              904
#define MACH_TYPE_NXDB500              905
#define MACH_TYPE_PALMT5               917
#define MACH_TYPE_PALMTC               918
#define MACH_TYPE_OMAP_APOLLON         919
#define MACH_TYPE_ATEB9200             923
#define MACH_TYPE_N35                  927
#define MACH_TYPE_LOGICPD_PXA270       930
#define MACH_TYPE_NXEB500HMI           941
#define MACH_TYPE_ESPRESSO             949
#define MACH_TYPE_RX1950               952
#define MACH_TYPE_GESBC9312            958
#define MACH_TYPE_PICOTUX2XX           963
#define MACH_TYPE_DSMG600              964
#define MACH_TYPE_OMAP_FSAMPLE         970
#define MACH_TYPE_SNAPPER_CL15         986
#define MACH_TYPE_OMAP_PALMZ71         993
#define MACH_TYPE_SMDK2412             1009
#define MACH_TYPE_SMDK2413             1022
#define MACH_TYPE_AML_M5900            1024
#define MACH_TYPE_ECBAT91              1072
#define MACH_TYPE_ONEARM               1075
#define MACH_TYPE_SMDK2443             1084
#define MACH_TYPE_FSG                  1091
#define MACH_TYPE_AT91SAM9260EK        1099
#define MACH_TYPE_GLANTANK             1100
#define MACH_TYPE_N2100                1101
#define MACH_TYPE_QT2410               1108
#define MACH_TYPE_KIXRP435             1109
#define MACH_TYPE_CC9P9360DEV          1114
#define MACH_TYPE_EDB9302A             1127
#define MACH_TYPE_EDB9307A             1128
#define MACH_TYPE_OMAP_3430SDP         1138
#define MACH_TYPE_VSTMS                1140
#define MACH_TYPE_MICRO9M              1169
#define MACH_TYPE_BUG                  1179
#define MACH_TYPE_AT91SAM9263EK        1202
#define MACH_TYPE_EM7210               1212
#define MACH_TYPE_VPAC270              1227
#define MACH_TYPE_TREO680              1230
#define MACH_TYPE_ZYLONITE             1233
#define MACH_TYPE_MX31LITE             1236
#define MACH_TYPE_MIOA701              1257
#define MACH_TYPE_ARMADILLO5X0         1260
#define MACH_TYPE_CC9P9360JS           1264
#define MACH_TYPE_NOKIA_N800           1271
#define MACH_TYPE_EP80219              1281
#define MACH_TYPE_GORAMO_MLR           1292
#define MACH_TYPE_EM_X270              1297
#define MACH_TYPE_NEO1973_GTA02        1304
#define MACH_TYPE_AT91SAM9RLEK         1326
#define MACH_TYPE_COLIBRI320           1340
#define MACH_TYPE_CAM60                1351
#define MACH_TYPE_AT91EB01             1354
#define MACH_TYPE_DB88F5281            1358
#define MACH_TYPE_CSB726               1359
#define MACH_TYPE_DAVINCI_DM6467_EVM   1380
#define MACH_TYPE_DAVINCI_DM355_EVM    1381
#define MACH_TYPE_LITTLETON            1388
#define MACH_TYPE_REALVIEW_PB11MP      1407
#define MACH_TYPE_MX27_3DS             1430
#define MACH_TYPE_HALIBUT              1439
#define MACH_TYPE_TROUT                1440
#define MACH_TYPE_TCT_HAMMER           1460
#define MACH_TYPE_HERALD               1461
#define MACH_TYPE_SIM_ONE              1476
#define MACH_TYPE_JIVE                 1490
#define MACH_TYPE_SAM9_L9260           1501
#define MACH_TYPE_REALVIEW_PB1176      1504
#define MACH_TYPE_YL9200               1507
#define MACH_TYPE_RD88F5182            1508
#define MACH_TYPE_KUROBOX_PRO          1509
#define MACH_TYPE_MX31_3DS             1511
#define MACH_TYPE_QONG                 1524
#define MACH_TYPE_OMAP2EVM             1534
#define MACH_TYPE_OMAP3EVM             1535
#define MACH_TYPE_DNS323               1542
#define MACH_TYPE_OMAP3_BEAGLE         1546
#define MACH_TYPE_NOKIA_N810           1548
#define MACH_TYPE_PCM038               1551
#define MACH_TYPE_TS209                1565
#define MACH_TYPE_AT91CAP9ADK          1566
#define MACH_TYPE_MX31MOBOARD          1574
#define MACH_TYPE_TERASTATION_PRO2     1584
#define MACH_TYPE_LINKSTATION_PRO      1585
#define MACH_TYPE_E350                 1596
#define MACH_TYPE_TS409                1601
#define MACH_TYPE_CM_X300              1616
#define MACH_TYPE_AT91SAM9G20EK        1624
#define MACH_TYPE_SMDK6410             1626
#define MACH_TYPE_U300                 1627
#define MACH_TYPE_WRT350N_V2           1633
#define MACH_TYPE_OMAP_LDP             1639
#define MACH_TYPE_MX35_3DS             1645
#define MACH_TYPE_NEUROS_OSD2          1647
#define MACH_TYPE_TRIZEPS4WL           1649
#define MACH_TYPE_TS78XX               1652
#define MACH_TYPE_SFFSDR               1657
#define MACH_TYPE_SMARTWEB             1668
#define MACH_TYPE_PCM037               1673
#define MACH_TYPE_DB88F6281_BP         1680
#define MACH_TYPE_RD88F6192_NAS        1681
#define MACH_TYPE_RD88F6281            1682
#define MACH_TYPE_DB78X00_BP           1683
#define MACH_TYPE_SMDK2416             1685
#define MACH_TYPE_WBD111               1690
#define MACH_TYPE_MV2120               1693
#define MACH_TYPE_MX51_3DS             1696
#define MACH_TYPE_IMX27LITE            1701
#define MACH_TYPE_USB_A9260            1709
#define MACH_TYPE_USB_A9263            1710
#define MACH_TYPE_QIL_A9260            1711
#define MACH_TYPE_KZM_ARM11_01         1722
#define MACH_TYPE_NOKIA_N810_WIMAX     1727
#define MACH_TYPE_SAPPHIRE             1729
#define MACH_TYPE_STMP37XX             1732
#define MACH_TYPE_STMP378X             1733
#define MACH_TYPE_EZX_A780             1740
#define MACH_TYPE_EZX_E680             1741
#define MACH_TYPE_EZX_A1200            1742
#define MACH_TYPE_EZX_E6               1743
#define MACH_TYPE_EZX_E2               1744
#define MACH_TYPE_EZX_A910             1745
#define MACH_TYPE_EDMINI_V2            1756
#define MACH_TYPE_ZIPIT2               1757
#define MACH_TYPE_OMAP3_PANDORA        1761
#define MACH_TYPE_MSS2                 1766
#define MACH_TYPE_LB88RC8480           1769
#define MACH_TYPE_MX25_3DS             1771
#define MACH_TYPE_OMAP3530_LV_SOM      1773
#define MACH_TYPE_DAVINCI_DA830_EVM    1781
#define MACH_TYPE_AT572D940HFEB        1783
#define MACH_TYPE_DOVE_DB              1788
#define MACH_TYPE_OVERO                1798
#define MACH_TYPE_AT2440EVB            1799
#define MACH_TYPE_NEOCORE926           1800
#define MACH_TYPE_WNR854T              1801
#define MACH_TYPE_RD88F5181L_GE        1812
#define MACH_TYPE_RD88F5181L_FXO       1818
#define MACH_TYPE_STAMP9G20            1824
#define MACH_TYPE_SMDKC100             1826
#define MACH_TYPE_TAVOREVB             1827
#define MACH_TYPE_SAAR                 1828
#define MACH_TYPE_AT91SAM9M10G45EK     1830
#define MACH_TYPE_MXLADS               1851
#define MACH_TYPE_LINKSTATION_MINI     1858
#define MACH_TYPE_AFEB9260             1859
#define MACH_TYPE_IMX27IPCAM           1871
#define MACH_TYPE_RD88F6183AP_GE       1894
#define MACH_TYPE_REALVIEW_PBA8        1897
#define MACH_TYPE_REALVIEW_PBX         1901
#define MACH_TYPE_MICRO9S              1902
#define MACH_TYPE_RUT100               1908
#define MACH_TYPE_G3EVM                1919
#define MACH_TYPE_W90P910EVB           1921
#define MACH_TYPE_W90P950EVB           1923
#define MACH_TYPE_W90N960EVB           1924
#define MACH_TYPE_MV88F6281GTW_GE      1932
#define MACH_TYPE_NCP                  1933
#define MACH_TYPE_DAVINCI_DM365_EVM    1939
#define MACH_TYPE_CENTRO               1944
#define MACH_TYPE_NOKIA_RX51           1955
#define MACH_TYPE_OMAP_ZOOM2           1967
#define MACH_TYPE_CPUAT9260            1973
#define MACH_TYPE_EUKREA_CPUIMX27      1975
#define MACH_TYPE_ACS5K                1982
#define MACH_TYPE_SNAPPER_9260         1987
#define MACH_TYPE_DSM320               1988
#define MACH_TYPE_EXEDA                1994
#define MACH_TYPE_MINI2440             1999
#define MACH_TYPE_COLIBRI300           2000
#define MACH_TYPE_LINKSTATION_LS_HGL   2005
#define MACH_TYPE_CPUAT9G20            2031
#define MACH_TYPE_SMDK6440             2032
#define MACH_TYPE_NAS4220B             2038
#define MACH_TYPE_ZYLONITE2            2042
#define MACH_TYPE_ASPENITE             2043
#define MACH_TYPE_TTC_DKB              2045
#define MACH_TYPE_PCM043               2072
#define MACH_TYPE_SHEEVAPLUG           2097
#define MACH_TYPE_AVENGERS_LITE        2104
#define MACH_TYPE_MX51_BABBAGE         2125
#define MACH_TYPE_RD78X00_MASA         2135
#define MACH_TYPE_DM355_LEOPARD        2138
#define MACH_TYPE_TS219                2139
#define MACH_TYPE_PCA100               2149
#define MACH_TYPE_DAVINCI_DA850_EVM    2157
#define MACH_TYPE_AT91SAM9G10EK        2159
#define MACH_TYPE_OMAP_4430SDP         2160
#define MACH_TYPE_MAGX_ZN5             2162
#define MACH_TYPE_BTMAVB101            2172
#define MACH_TYPE_BTMAWB101            2173
#define MACH_TYPE_OMAP3_TORPEDO        2178
#define MACH_TYPE_ANW6410              2183
#define MACH_TYPE_IMX27_VISSTRIM_M10   2187
#define MACH_TYPE_PORTUXG20            2191
#define MACH_TYPE_SMDKC110             2193
#define MACH_TYPE_OMAP3517EVM          2200
#define MACH_TYPE_NETSPACE_V2          2201
#define MACH_TYPE_NETSPACE_MAX_V2      2202
#define MACH_TYPE_D2NET_V2             2203
#define MACH_TYPE_NET2BIG_V2           2204
#define MACH_TYPE_NET5BIG_V2           2206
#define MACH_TYPE_INETSPACE_V2         2208
#define MACH_TYPE_AT91SAM9G45EKES      2212
#define MACH_TYPE_PC7302               2220
#define MACH_TYPE_SPEAR600             2236
#define MACH_TYPE_SPEAR300             2237
#define MACH_TYPE_LILLY1131            2239
#define MACH_TYPE_HMT                  2254
#define MACH_TYPE_VEXPRESS             2272
#define MACH_TYPE_D2NET                2282
#define MACH_TYPE_BIGDISK              2283
#define MACH_TYPE_AT91SAM9G20EK_2MMC   2288
#define MACH_TYPE_BCMRING              2289
#define MACH_TYPE_DP6XX                2302
#define MACH_TYPE_MAHIMAHI             2304
#define MACH_TYPE_SMDK6442             2324
#define MACH_TYPE_OPENRD_BASE          2325
#define MACH_TYPE_DEVKIT8000           2330
#define MACH_TYPE_MX51_EFIKAMX         2336
#define MACH_TYPE_CM_T35               2341
#define MACH_TYPE_NET2BIG              2342
#define MACH_TYPE_IGEP0020             2344
#define MACH_TYPE_NUC932EVB            2356
#define MACH_TYPE_OPENRD_CLIENT        2361
#define MACH_TYPE_MX51_EFIKASB         2370
#define MACH_TYPE_MARVELL_JASPER       2382
#define MACH_TYPE_FLINT                2383
#define MACH_TYPE_TAVOREVB3            2384
#define MACH_TYPE_TOUCHBOOK            2393
#define MACH_TYPE_RAUMFELD_RC          2413
#define MACH_TYPE_RAUMFELD_CONNECTOR   2414
#define MACH_TYPE_RAUMFELD_SPEAKER     2415
#define MACH_TYPE_TNETV107X            2418
#define MACH_TYPE_SMDKV210             2456
#define MACH_TYPE_OMAP_ZOOM3           2464
#define MACH_TYPE_OMAP_3630SDP         2465
#define MACH_TYPE_SMARTQ7              2479
#define MACH_TYPE_WATSON_EFM_PLUGIN    2491
#define MACH_TYPE_G4EVM                2493
#define MACH_TYPE_OMAPL138_HAWKBOARD   2495
#define MACH_TYPE_TS41X                2502
#define MACH_TYPE_PHY3250              2511
#define MACH_TYPE_MINI6410             2520
#define MACH_TYPE_MX28EVK              2531
#define MACH_TYPE_SMARTQ5              2534
#define MACH_TYPE_DAVINCI_DM6467TEVM   2548
#define MACH_TYPE_MXT_TD60             2550
#define MACH_TYPE_RIOT_BEI2            2576
#define MACH_TYPE_RIOT_X37             2578
#define MACH_TYPE_CAPC7117             2612
#define MACH_TYPE_ICONTROL             2624
#define MACH_TYPE_QSD8X50A_ST1_5       2627
#define MACH_TYPE_MX23EVK              2629
#define MACH_TYPE_AP4EVB               2630
#define MACH_TYPE_MITYOMAPL138         2650
#define MACH_TYPE_GURUPLUG             2659
#define MACH_TYPE_SPEAR310             2660
#define MACH_TYPE_SPEAR320             2661
#define MACH_TYPE_AQUILA               2676
#define MACH_TYPE_ESATA_SHEEVAPLUG     2678
#define MACH_TYPE_MSM7X30_SURF         2679
#define MACH_TYPE_EA2478DEVKIT         2683
#define MACH_TYPE_TERASTATION_WXL      2697
#define MACH_TYPE_MSM7X25_SURF         2703
#define MACH_TYPE_MSM7X25_FFA          2704
#define MACH_TYPE_MSM7X27_SURF         2705
#define MACH_TYPE_MSM7X27_FFA          2706
#define MACH_TYPE_MSM7X30_FFA          2707
#define MACH_TYPE_QSD8X50_SURF         2708
#define MACH_TYPE_MX53_EVK             2716
#define MACH_TYPE_IGEP0030             2717
#define MACH_TYPE_SBC3530              2722
#define MACH_TYPE_SAARB                2727
#define MACH_TYPE_HARMONY              2731
#define MACH_TYPE_MSM7X30_FLUID        2741
#define MACH_TYPE_CM_T3517             2750
#define MACH_TYPE_WBD222               2753
#define MACH_TYPE_MSM8X60_SURF         2755
#define MACH_TYPE_MSM8X60_SIM          2756
#define MACH_TYPE_TCC8000_SDK          2758
#define MACH_TYPE_NANOS                2759
#define MACH_TYPE_STAMP9G45            2761
#define MACH_TYPE_CNS3420VB            2776
#define MACH_TYPE_OMAP4_PANDA          2791
#define MACH_TYPE_TI8168EVM            2800
#define MACH_TYPE_TETON_BGA            2816
#define MACH_TYPE_EUKREA_CPUIMX25SD    2820
#define MACH_TYPE_EUKREA_CPUIMX35SD    2821
#define MACH_TYPE_EUKREA_CPUIMX51SD    2822
#define MACH_TYPE_EUKREA_CPUIMX51      2823
#define MACH_TYPE_SMDKC210             2838
#define MACH_TYPE_OMAP3_BRAILLO        2839
#define MACH_TYPE_SPYPLUG              2840
#define MACH_TYPE_GINGER               2841
#define MACH_TYPE_TNY_T3530            2842
#define MACH_TYPE_PCA102               2843
#define MACH_TYPE_SPADE                2844
#define MACH_TYPE_MXC25_TOPAZ          2845
#define MACH_TYPE_T5325                2846
#define MACH_TYPE_GW2361               2847
#define MACH_TYPE_ELOG                 2848
#define MACH_TYPE_INCOME               2849
#define MACH_TYPE_BCM589X              2850
#define MACH_TYPE_ETNA                 2851
#define MACH_TYPE_HAWKS                2852
#define MACH_TYPE_MESON                2853
#define MACH_TYPE_XSBASE255            2854
#define MACH_TYPE_PVM2030              2855
#define MACH_TYPE_MIOA502              2856
#define MACH_TYPE_VVBOX_SDORIG2        2857
#define MACH_TYPE_VVBOX_SDLITE2        2858
#define MACH_TYPE_VVBOX_SDPRO4         2859
#define MACH_TYPE_HTC_SPV_M700         2860
#define MACH_TYPE_MX257SX              2861
#define MACH_TYPE_GONI                 2862
#define MACH_TYPE_MSM8X55_SVLTE_FFA    2863
#define MACH_TYPE_MSM8X55_SVLTE_SURF   2864
#define MACH_TYPE_QUICKSTEP            2865
#define MACH_TYPE_DMW96                2866
#define MACH_TYPE_HAMMERHEAD           2867
#define MACH_TYPE_TRIDENT              2868
#define MACH_TYPE_LIGHTNING            2869
#define MACH_TYPE_ICONNECT             2870
#define MACH_TYPE_AUTOBOT              2871
#define MACH_TYPE_COCONUT              2872
#define MACH_TYPE_DURIAN               2873
#define MACH_TYPE_CAYENNE              2874
#define MACH_TYPE_FUJI                 2875
#define MACH_TYPE_SYNOLOGY_6282        2876
#define MACH_TYPE_EM1SY                2877
#define MACH_TYPE_M502                 2878
#define MACH_TYPE_MATRIX518            2879
#define MACH_TYPE_TINY_GURNARD         2880
#define MACH_TYPE_SPEAR1310            2881
#define MACH_TYPE_BV07                 2882
#define MACH_TYPE_MXT_TD61             2883
#define MACH_TYPE_OPENRD_ULTIMATE      2884
#define MACH_TYPE_DEVIXP               2885
#define MACH_TYPE_MICCPT               2886
#define MACH_TYPE_MIC256               2887
#define MACH_TYPE_AS1167               2888
#define MACH_TYPE_OMAP3_IBIZA          2889
#define MACH_TYPE_U5500                2890
#define MACH_TYPE_DAVINCI_PICTO        2891
#define MACH_TYPE_MECHA                2892
#define MACH_TYPE_BUBBA3               2893
#define MACH_TYPE_PUPITRE              2894
#define MACH_TYPE_TEGRA_VOGUE          2896
#define MACH_TYPE_TEGRA_E1165          2897
#define MACH_TYPE_SIMPLENET            2898
#define MACH_TYPE_EC4350TBM            2899
#define MACH_TYPE_PEC_TC               2900
#define MACH_TYPE_PEC_HC2              2901
#define MACH_TYPE_ESL_MOBILIS_A        2902
#define MACH_TYPE_ESL_MOBILIS_B        2903
#define MACH_TYPE_ESL_WAVE_A           2904
#define MACH_TYPE_ESL_WAVE_B           2905
#define MACH_TYPE_UNISENSE_MMM         2906
#define MACH_TYPE_BLUESHARK            2907
#define MACH_TYPE_E10                  2908
#define MACH_TYPE_APP3K_ROBIN          2909
#define MACH_TYPE_POV15HD              2910
#define MACH_TYPE_STELLA               2911
#define MACH_TYPE_LINKSTATION_LSCHL    2913
#define MACH_TYPE_NETWALKER            2914
#define MACH_TYPE_ACSX106              2915
#define MACH_TYPE_ATLAS5_C1            2916
#define MACH_TYPE_NSB3AST              2917
#define MACH_TYPE_GNET_SLC             2918
#define MACH_TYPE_AF4000               2919
#define MACH_TYPE_ARK9431              2920
#define MACH_TYPE_FS_S5PC100           2921
#define MACH_TYPE_OMAP3505NOVA8        2922
#define MACH_TYPE_OMAP3621_EDP1        2923
#define MACH_TYPE_ORATISAES            2924
#define MACH_TYPE_SMDKV310             2925
#define MACH_TYPE_SIEMENS_L0           2926
#define MACH_TYPE_VENTANA              2927
#define MACH_TYPE_WM8505_7IN_NETBOOK   2928
#define MACH_TYPE_EC4350SDB            2929
#define MACH_TYPE_MIMAS                2930
#define MACH_TYPE_TITAN                2931
#define MACH_TYPE_CRANEBOARD           2932
#define MACH_TYPE_ES2440               2933
#define MACH_TYPE_NAJAY_A9263          2934
#define MACH_TYPE_HTCTORNADO           2935
#define MACH_TYPE_DIMM_MX257           2936
#define MACH_TYPE_JIGEN                2937
#define MACH_TYPE_SMDK6450             2938
#define MACH_TYPE_MENO_QNG             2939
#define MACH_TYPE_NS2416               2940
#define MACH_TYPE_RPC353               2941
#define MACH_TYPE_TQ6410               2942
#define MACH_TYPE_SKY6410              2943
#define MACH_TYPE_DYNASTY              2944
#define MACH_TYPE_VIVO                 2945
#define MACH_TYPE_BURY_BL7582          2946
#define MACH_TYPE_BURY_BPS5270         2947
#define MACH_TYPE_BASI                 2948
#define MACH_TYPE_TN200                2949
#define MACH_TYPE_C2MMI                2950
#define MACH_TYPE_MESON_6236M          2951
#define MACH_TYPE_MESON_8626M          2952
#define MACH_TYPE_TUBE                 2953
#define MACH_TYPE_MESSINA              2954
#define MACH_TYPE_MX50_ARM2            2955
#define MACH_TYPE_CETUS9263            2956
#define MACH_TYPE_BROWNSTONE           2957
#define MACH_TYPE_VMX25                2958
#define MACH_TYPE_VMX51                2959
#define MACH_TYPE_ABACUS               2960
#define MACH_TYPE_CM4745               2961
#define MACH_TYPE_ORATISLINK           2962
#define MACH_TYPE_DAVINCI_DM365_DVR    2963
#define MACH_TYPE_NETVIZ               2964
#define MACH_TYPE_FLEXIBITY            2965
#define MACH_TYPE_WLAN_COMPUTER        2966
#define MACH_TYPE_LPC24XX              2967
#define MACH_TYPE_SPICA                2968
#define MACH_TYPE_GPSDISPLAY           2969
#define MACH_TYPE_BIPNET               2970
#define MACH_TYPE_OVERO_CTU_INERTIAL   2971
#define MACH_TYPE_DAVINCI_DM355_MMM    2972
#define MACH_TYPE_PC9260_V2            2973
#define MACH_TYPE_PTX7545              2974
#define MACH_TYPE_TM_EFDC              2975
#define MACH_TYPE_OMAP3_WALDO1         2977
#define MACH_TYPE_FLYER                2978
#define MACH_TYPE_TORNADO3240          2979
#define MACH_TYPE_SOLI_01              2980
#define MACH_TYPE_OMAPL138_EUROPALC    2981
#define MACH_TYPE_HELIOS_V1            2982
#define MACH_TYPE_NETSPACE_LITE_V2     2983
#define MACH_TYPE_SSC                  2984
#define MACH_TYPE_PREMIERWAVE_EN       2985
#define MACH_TYPE_WASABI               2986
#define MACH_TYPE_MX50_RDP             2988
#define MACH_TYPE_UNIVERSAL_C210       2989
#define MACH_TYPE_REAL6410             2990
#define MACH_TYPE_SPX_SAKURA           2991
#define MACH_TYPE_IJ3K_2440            2992
#define MACH_TYPE_OMAP3_BC10           2993
#define MACH_TYPE_THEBE                2994
#define MACH_TYPE_RV082                2995
#define MACH_TYPE_ARMLGUEST            2996
#define MACH_TYPE_TJINC1000            2997
#define MACH_TYPE_DOCKSTAR             2998
#define MACH_TYPE_AX8008               2999
#define MACH_TYPE_GNET_SGCE            3000
#define MACH_TYPE_PXWNAS_500_1000      3001
#define MACH_TYPE_EA20                 3002
#define MACH_TYPE_AWM2                 3003
#define MACH_TYPE_TI8148EVM            3004
#define MACH_TYPE_SEABOARD             3005
#define MACH_TYPE_LINKSTATION_CHLV2    3006
#define MACH_TYPE_TERA_PRO2_RACK       3007
#define MACH_TYPE_RUBYS                3008
#define MACH_TYPE_AQUARIUS             3009
#define MACH_TYPE_MX53_ARD             3010
#define MACH_TYPE_MX53_SMD             3011
#define MACH_TYPE_LSWXL                3012
#define MACH_TYPE_DOVE_AVNG_V3         3013
#define MACH_TYPE_SDI_ESS_9263         3014
#define MACH_TYPE_JOCPU550             3015
#define MACH_TYPE_MSM8X60_RUMI3        3016
#define MACH_TYPE_MSM8X60_FFA          3017
#define MACH_TYPE_YANOMAMI             3018
#define MACH_TYPE_GTA04                3019
#define MACH_TYPE_CM_A510              3020
#define MACH_TYPE_OMAP3_RFS200         3021
#define MACH_TYPE_KX33XX               3022
#define MACH_TYPE_PTX7510              3023
#define MACH_TYPE_TOP9000              3024
#define MACH_TYPE_TEENOTE              3025
#define MACH_TYPE_TS3                  3026
#define MACH_TYPE_A0                   3027
#define MACH_TYPE_FSM9XXX_SURF         3028
#define MACH_TYPE_FSM9XXX_FFA          3029
#define MACH_TYPE_FRRHWCDMA60W         3030
#define MACH_TYPE_REMUS                3031
#define MACH_TYPE_AT91CAP7XDK          3032
#define MACH_TYPE_AT91CAP7STK          3033
#define MACH_TYPE_KT_SBC_SAM9_1        3034
#define MACH_TYPE_ARMADA_XP_DB         3036
#define MACH_TYPE_SPDM                 3037
#define MACH_TYPE_GTIB                 3038
#define MACH_TYPE_DGM3240              3039
#define MACH_TYPE_HTCMEGA              3041
#define MACH_TYPE_TRICORDER            3042
#define MACH_TYPE_TX28                 3043
#define MACH_TYPE_BSTBRD               3044
#define MACH_TYPE_PWB3090              3045
#define MACH_TYPE_IDEA6410             3046
#define MACH_TYPE_QBC9263              3047
#define MACH_TYPE_BORABORA             3048
#define MACH_TYPE_VALDEZ               3049
#define MACH_TYPE_LS9G20               3050
#define MACH_TYPE_MIOS_V1              3051
#define MACH_TYPE_S5PC110_CRESPO       3052
#define MACH_TYPE_CONTROLTEK9G20       3053
#define MACH_TYPE_TIN307               3054
#define MACH_TYPE_TIN510               3055
#define MACH_TYPE_BLUECHEESE           3057
#define MACH_TYPE_TEM3X30              3058
#define MACH_TYPE_HARVEST_DESOTO       3059
#define MACH_TYPE_MSM8X60_QRDC         3060
#define MACH_TYPE_SPEAR900             3061
#define MACH_TYPE_PCONTROL_G20         3062
#define MACH_TYPE_RDSTOR               3063
#define MACH_TYPE_USDLOADER            3064
#define MACH_TYPE_TSOPLOADER           3065
#define MACH_TYPE_KRONOS               3066
#define MACH_TYPE_FFCORE               3067
#define MACH_TYPE_MONE                 3068
#define MACH_TYPE_UNIT2S               3069
#define MACH_TYPE_ACER_A5              3070
#define MACH_TYPE_ETHERPRO_ISP         3071
#define MACH_TYPE_STRETCHS7000         3072
#define MACH_TYPE_P87_SMARTSIM         3073
#define MACH_TYPE_TULIP                3074
#define MACH_TYPE_SUNFLOWER            3075
#define MACH_TYPE_RIB                  3076
#define MACH_TYPE_CLOD                 3077
#define MACH_TYPE_RUMP                 3078
#define MACH_TYPE_TENDERLOIN           3079
#define MACH_TYPE_SHORTLOIN            3080
#define MACH_TYPE_ANTARES              3082
#define MACH_TYPE_WB40N                3083
#define MACH_TYPE_HERRING              3084
#define MACH_TYPE_NAXY400              3085
#define MACH_TYPE_NAXY1200             3086
#define MACH_TYPE_VPR200               3087
#define MACH_TYPE_BUG20                3088
#define MACH_TYPE_GOFLEXNET            3089
#define MACH_TYPE_TORBRECK             3090
#define MACH_TYPE_SAARB_MG1            3091
#define MACH_TYPE_CALLISTO             3092
#define MACH_TYPE_MULTHSU              3093
#define MACH_TYPE_SALUDA               3094
#define MACH_TYPE_PEMP_OMAP3_APOLLO    3095
#define MACH_TYPE_VC0718               3096
#define MACH_TYPE_MVBLX                3097
#define MACH_TYPE_INHAND_APEIRON       3098
#define MACH_TYPE_INHAND_FURY          3099
#define MACH_TYPE_INHAND_SIREN         3100
#define MACH_TYPE_HDNVP                3101
#define MACH_TYPE_SOFTWINNER           3102
#define MACH_TYPE_PRIMA2_EVB           3103
#define MACH_TYPE_NAS6210              3104
#define MACH_TYPE_UNISDEV              3105
#define MACH_TYPE_SBCA11               3106
#define MACH_TYPE_SAGA                 3107
#define MACH_TYPE_NS_K330              3108
#define MACH_TYPE_TANNA                3109
#define MACH_TYPE_IMATE8502            3110
#define MACH_TYPE_ASPEN                3111
#define MACH_TYPE_DAINTREE_CWAC        3112
#define MACH_TYPE_ZMX25                3113
#define MACH_TYPE_MAPLE1               3114
#define MACH_TYPE_QSD8X72_SURF         3115
#define MACH_TYPE_QSD8X72_FFA          3116
#define MACH_TYPE_ABILENE              3117
#define MACH_TYPE_EIGEN_TTR            3118
#define MACH_TYPE_IOMEGA_IX2_200       3119
#define MACH_TYPE_CORETEC_VCX7400      3120
#define MACH_TYPE_SANTIAGO             3121
#define MACH_TYPE_MX257SOL             3122
#define MACH_TYPE_STRASBOURG           3123
#define MACH_TYPE_MSM8X60_FLUID        3124
#define MACH_TYPE_SMARTQV5             3125
#define MACH_TYPE_SMARTQV3             3126
#define MACH_TYPE_SMARTQV7             3127
#define MACH_TYPE_PAZ00                3128
#define MACH_TYPE_ACMENETUSFOXG20      3129
#define MACH_TYPE_FWBD_0404            3131
#define MACH_TYPE_HDGU                 3132
#define MACH_TYPE_PYRAMID              3133
#define MACH_TYPE_EPIPHAN              3134
#define MACH_TYPE_OMAP_BENDER          3135
#define MACH_TYPE_GURNARD              3136
#define MACH_TYPE_GTL_IT5100           3137
#define MACH_TYPE_BCM2708              3138
#define MACH_TYPE_MX51_GGC             3139
#define MACH_TYPE_SHARESPACE           3140
#define MACH_TYPE_HABA_KNX_EXPLORER    3141
#define MACH_TYPE_SIMTEC_KIRKMOD       3142
#define MACH_TYPE_CRUX                 3143
#define MACH_TYPE_MX51_BRAVO           3144
#define MACH_TYPE_CHARON               3145
#define MACH_TYPE_PICOCOM3             3146
#define MACH_TYPE_PICOCOM4             3147
#define MACH_TYPE_SERRANO              3148
#define MACH_TYPE_DOUBLESHOT           3149
#define MACH_TYPE_EVSY                 3150
#define MACH_TYPE_HUASHAN              3151
#define MACH_TYPE_LAUSANNE             3152
#define MACH_TYPE_EMERALD              3153
#define MACH_TYPE_TQMA35               3154
#define MACH_TYPE_MARVEL               3155
#define MACH_TYPE_MANUAE               3156
#define MACH_TYPE_CHACHA               3157
#define MACH_TYPE_LEMON                3158
#define MACH_TYPE_CSC                  3159
#define MACH_TYPE_GIRA_KNXIP_ROUTER    3160
#define MACH_TYPE_T20                  3161
#define MACH_TYPE_HDMINI               3162
#define MACH_TYPE_SCIPHONE_G2          3163
#define MACH_TYPE_EXPRESS              3164
#define MACH_TYPE_EXPRESS_KT           3165
#define MACH_TYPE_MAXIMASP             3166
#define MACH_TYPE_NITROGEN_IMX51       3167
#define MACH_TYPE_NITROGEN_IMX53       3168
#define MACH_TYPE_SUNFIRE              3169
#define MACH_TYPE_AROWANA              3170
#define MACH_TYPE_TEGRA_DAYTONA        3171
#define MACH_TYPE_TEGRA_SWORDFISH      3172
#define MACH_TYPE_EDISON               3173
#define MACH_TYPE_SVP8500V1            3174
#define MACH_TYPE_SVP8500V2            3175
#define MACH_TYPE_SVP5500              3176
#define MACH_TYPE_B5500                3177
#define MACH_TYPE_S5500                3178
#define MACH_TYPE_ICON                 3179
#define MACH_TYPE_ELEPHANT             3180
#define MACH_TYPE_SHOOTER              3182
#define MACH_TYPE_SPADE_LTE            3183
#define MACH_TYPE_PHILHWANI            3184
#define MACH_TYPE_GSNCOMM              3185
#define MACH_TYPE_STRASBOURG_A2        3186
#define MACH_TYPE_MMM                  3187
#define MACH_TYPE_DAVINCI_DM365_BV     3188
#define MACH_TYPE_AG5EVM               3189
#define MACH_TYPE_SC575PLC             3190
#define MACH_TYPE_SC575IPC             3191
#define MACH_TYPE_OMAP3_TDM3730        3192
#define MACH_TYPE_TOP9000_EVAL         3194
#define MACH_TYPE_TOP9000_SU           3195
#define MACH_TYPE_UTM300               3196
#define MACH_TYPE_TSUNAGI              3197
#define MACH_TYPE_TS75XX               3198
#define MACH_TYPE_TS47XX               3200
#define MACH_TYPE_DA850_K5             3201
#define MACH_TYPE_AX502                3202
#define MACH_TYPE_IGEP0032             3203
#define MACH_TYPE_ANTERO               3204
#define MACH_TYPE_SYNERGY              3205
#define MACH_TYPE_ICS_IF_VOIP          3206
#define MACH_TYPE_WLF_CRAGG_6410       3207
#define MACH_TYPE_PUNICA               3208
#define MACH_TYPE_TRIMSLICE            3209
#define MACH_TYPE_MX27_WMULTRA         3210
#define MACH_TYPE_MACKEREL             3211
#define MACH_TYPE_FA9X27               3213
#define MACH_TYPE_NS2816TB             3214
#define MACH_TYPE_NS2816_NTPAD         3215
#define MACH_TYPE_NS2816_NTNB          3216
#define MACH_TYPE_KAEN                 3217
#define MACH_TYPE_NV1000               3218
#define MACH_TYPE_NUC950TS             3219
#define MACH_TYPE_NOKIA_RM680          3220
#define MACH_TYPE_AST2200              3221
#define MACH_TYPE_LEAD                 3222
#define MACH_TYPE_UNINO1               3223
#define MACH_TYPE_GREECO               3224
#define MACH_TYPE_VERDI                3225
#define MACH_TYPE_DM6446_ADBOX         3226
#define MACH_TYPE_QUAD_SALSA           3227
#define MACH_TYPE_ABB_GMA_1_1          3228
#define MACH_TYPE_SVCID                3229
#define MACH_TYPE_MSM8960_SIM          3230
#define MACH_TYPE_MSM8960_RUMI3        3231
#define MACH_TYPE_ICON_G               3232
#define MACH_TYPE_MB3                  3233
#define MACH_TYPE_GSIA18S              3234
#define MACH_TYPE_PIVICC               3235
#define MACH_TYPE_PCM048               3236
#define MACH_TYPE_DDS                  3237
#define MACH_TYPE_CHALTEN_XA1          3238
#define MACH_TYPE_TS48XX               3239
#define MACH_TYPE_TONGA2_TFTTIMER      3240
#define MACH_TYPE_WHISTLER             3241
#define MACH_TYPE_ASL_PHOENIX          3242
#define MACH_TYPE_AT91SAM9263OTLITE    3243
#define MACH_TYPE_DDPLUG               3244
#define MACH_TYPE_D2PLUG               3245
#define MACH_TYPE_KZM9D                3246
#define MACH_TYPE_VERDI_LTE            3247
#define MACH_TYPE_NANOZOOM             3248
#define MACH_TYPE_DM3730_SOM_LV        3249
#define MACH_TYPE_DM3730_TORPEDO       3250
#define MACH_TYPE_ANCHOVY              3251
#define MACH_TYPE_RE2REV20             3253
#define MACH_TYPE_RE2REV21             3254
#define MACH_TYPE_CNS21XX              3255
#define MACH_TYPE_RIDER                3257
#define MACH_TYPE_NSK330               3258
#define MACH_TYPE_CNS2133EVB           3259
#define MACH_TYPE_Z3_816X_MOD          3260
#define MACH_TYPE_Z3_814X_MOD          3261
#define MACH_TYPE_BEECT                3262
#define MACH_TYPE_DMA_THUNDERBUG       3263
#define MACH_TYPE_OMN_AT91SAM9G20      3264
#define MACH_TYPE_MX25_E2S_UC          3265
#define MACH_TYPE_MIONE                3266
#define MACH_TYPE_TOP9000_TCU          3267
#define MACH_TYPE_TOP9000_BSL          3268
#define MACH_TYPE_KINGDOM              3269
#define MACH_TYPE_ARMADILLO460         3270
#define MACH_TYPE_LQ2                  3271
#define MACH_TYPE_SWEDA_TMS2           3272
#define MACH_TYPE_MX53_LOCO            3273
#define MACH_TYPE_ACER_A8              3275
#define MACH_TYPE_ACER_GAUGUIN         3276
#define MACH_TYPE_GUPPY                3277
#define MACH_TYPE_MX61_ARD             3278
#define MACH_TYPE_TX53                 3279
#define MACH_TYPE_OMAPL138_CASE_A3     3280
#define MACH_TYPE_UEMD                 3281
#define MACH_TYPE_CCWMX51MUT           3282
#define MACH_TYPE_ROCKHOPPER           3283
#define MACH_TYPE_ENCORE               3284
#define MACH_TYPE_HKDKC100             3285
#define MACH_TYPE_TS42XX               3286
#define MACH_TYPE_AEBL                 3287
#define MACH_TYPE_WARIO                3288
#define MACH_TYPE_GFS_SPM              3289
#define MACH_TYPE_CM_T3730             3290
#define MACH_TYPE_ISC3                 3291
#define MACH_TYPE_RASCAL               3292
#define MACH_TYPE_HREFV60              3293
#define MACH_TYPE_TPT_2_0              3294
#define MACH_TYPE_SPLENDOR             3296
#define MACH_TYPE_MSM8X60_QT           3298
#define MACH_TYPE_HTC_HD_MINI          3299
#define MACH_TYPE_ATHENE               3300
#define MACH_TYPE_DEEP_R_EK_1          3301
#define MACH_TYPE_VIVOW_CT             3302
#define MACH_TYPE_NERY_1000            3303
#define MACH_TYPE_RFL109145_SSRV       3304
#define MACH_TYPE_NMH                  3305
#define MACH_TYPE_WN802T               3306
#define MACH_TYPE_DRAGONET             3307
#define MACH_TYPE_AT91SAM9263DESK16L   3309
#define MACH_TYPE_BCMHANA_SV           3310
#define MACH_TYPE_BCMHANA_TABLET       3311
#define MACH_TYPE_KOI                  3312
#define MACH_TYPE_TS4800               3313
#define MACH_TYPE_TQMA9263             3314
#define MACH_TYPE_HOLIDAY              3315
#define MACH_TYPE_DMA6410              3316
#define MACH_TYPE_PCATS_OVERLAY        3317
#define MACH_TYPE_HWGW6410             3318
#define MACH_TYPE_SHENZHOU             3319
#define MACH_TYPE_CWME9210             3320
#define MACH_TYPE_CWME9210JS           3321
#define MACH_TYPE_PGS_SITARA           3322
#define MACH_TYPE_COLIBRI_T20          3323
#define MACH_TYPE_W21                  3324
#define MACH_TYPE_POLYSAT1             3325
#define MACH_TYPE_DATAWAY              3326
#define MACH_TYPE_COBRAL138            3327
#define MACH_TYPE_ROVERPCS8            3328
#define MACH_TYPE_MARVELC              3329
#define MACH_TYPE_NAVEFIHID            3330
#define MACH_TYPE_DM365_CV100          3331
#define MACH_TYPE_ABLE                 3332
#define MACH_TYPE_LEGACY               3333
#define MACH_TYPE_ICONG                3334
#define MACH_TYPE_ROVER_G8             3335
#define MACH_TYPE_T5388P               3336
#define MACH_TYPE_DINGO                3337
#define MACH_TYPE_GOFLEXHOME           3338
#define MACH_TYPE_LANREADYFN511        3340
#define MACH_TYPE_OMAP3_BAIA           3341
#define MACH_TYPE_OMAP3SMARTDISPLAY    3342
#define MACH_TYPE_XILINX               3343
#define MACH_TYPE_A2F                  3344
#define MACH_TYPE_SKY25                3345
#define MACH_TYPE_CCMX53               3346
#define MACH_TYPE_CCMX53JS             3347
#define MACH_TYPE_CCWMX53              3348
#define MACH_TYPE_CCWMX53JS            3349
#define MACH_TYPE_FRISMS               3350
#define MACH_TYPE_MSM7X27A_FFA         3351
#define MACH_TYPE_MSM7X27A_SURF        3352
#define MACH_TYPE_MSM7X27A_RUMI3       3353
#define MACH_TYPE_DIMMSAM9G20          3354
#define MACH_TYPE_DIMM_IMX28           3355
#define MACH_TYPE_AMK_A4               3356
#define MACH_TYPE_GNET_SGME            3357
#define MACH_TYPE_SHOOTER_U            3358
#define MACH_TYPE_VMX53                3359
#define MACH_TYPE_RHINO                3360
#define MACH_TYPE_ARMLEX4210           3361
#define MACH_TYPE_SWARCOEXTMODEM       3362
#define MACH_TYPE_PCM049               3364
#define MACH_TYPE_VIGOR                3365
#define MACH_TYPE_OSLO_AMUNDSEN        3366
#define MACH_TYPE_GSL_DIAMOND          3367
#define MACH_TYPE_CV2201               3368
#define MACH_TYPE_CV2202               3369
#define MACH_TYPE_CV2203               3370
#define MACH_TYPE_VIT_IBOX             3371
#define MACH_TYPE_DM6441_ESP           3372
#define MACH_TYPE_AT91SAM9X5EK         3373
#define MACH_TYPE_LIBRA                3374
#define MACH_TYPE_EASYCRRH             3375
#define MACH_TYPE_TRIPEL               3376
#define MACH_TYPE_ENDIAN_MINI          3377
#define MACH_TYPE_XILINX_EP107         3378
#define MACH_TYPE_NURI                 3379
#define MACH_TYPE_JANUS                3380
#define MACH_TYPE_DDNAS                3381
#define MACH_TYPE_TAG                  3382
#define MACH_TYPE_TAGW                 3383
#define MACH_TYPE_NITROGEN_VM_IMX51    3384
#define MACH_TYPE_VIPRINET             3385
#define MACH_TYPE_BOCKW                3386
#define MACH_TYPE_EVA2000              3387
#define MACH_TYPE_STEELYARD            3388
#define MACH_TYPE_MACH_SDH001          3390
#define MACH_TYPE_NSSLSBOARD           3392
#define MACH_TYPE_GENEVA_B5            3393
#define MACH_TYPE_SPEAR1340            3394
#define MACH_TYPE_REXMAS               3395
#define MACH_TYPE_MSM8960_CDP          3396
#define MACH_TYPE_MSM8960_MDP          3397
#define MACH_TYPE_MSM8960_FLUID        3398
#define MACH_TYPE_MSM8960_APQ          3399
#define MACH_TYPE_HELIOS_V2            3400
#define MACH_TYPE_MIF10P               3401
#define MACH_TYPE_IAM28                3402
#define MACH_TYPE_PICASSO              3403
#define MACH_TYPE_MR301A               3404
#define MACH_TYPE_NOTLE                3405
#define MACH_TYPE_EELX2                3406
#define MACH_TYPE_MOON                 3407
#define MACH_TYPE_RUBY                 3408
#define MACH_TYPE_GOLDENGATE           3409
#define MACH_TYPE_CTBU_GEN2            3410
#define MACH_TYPE_KMP_AM17_01          3411
#define MACH_TYPE_WTPLUG               3412
#define MACH_TYPE_MX27SU2              3413
#define MACH_TYPE_NB31                 3414
#define MACH_TYPE_HJSDU                3415
#define MACH_TYPE_TD3_REV1             3416
#define MACH_TYPE_EAG_CI4000           3417
#define MACH_TYPE_NET5BIG_NAND_V2      3418
#define MACH_TYPE_CPX2                 3419
#define MACH_TYPE_NET2BIG_NAND_V2      3420
#define MACH_TYPE_ECUV5                3421
#define MACH_TYPE_HSGX6D               3422
#define MACH_TYPE_DAWAD7               3423
#define MACH_TYPE_SAM9REPEATER         3424
#define MACH_TYPE_GT_I5700             3425
#define MACH_TYPE_CTERA_PLUG_C2        3426
#define MACH_TYPE_MARVELCT             3427
#define MACH_TYPE_AG11005              3428
#define MACH_TYPE_VANGOGH              3430
#define MACH_TYPE_MATRIX505            3431
#define MACH_TYPE_OCE_NIGMA            3432
#define MACH_TYPE_T55                  3433
#define MACH_TYPE_BIO3K                3434
#define MACH_TYPE_EXPRESSCT            3435
#define MACH_TYPE_CARDHU               3436
#define MACH_TYPE_ARUBA                3437
#define MACH_TYPE_BONAIRE              3438
#define MACH_TYPE_NUC700EVB            3439
#define MACH_TYPE_NUC710EVB            3440
#define MACH_TYPE_NUC740EVB            3441
#define MACH_TYPE_NUC745EVB            3442
#define MACH_TYPE_TRANSCEDE            3443
#define MACH_TYPE_MORA                 3444
#define MACH_TYPE_NDA_EVM              3445
#define MACH_TYPE_TIMU                 3446
#define MACH_TYPE_EXPRESSH             3447
#define MACH_TYPE_VERIDIS_A300         3448
#define MACH_TYPE_DM368_LEOPARD        3449
#define MACH_TYPE_OMAP_MCOP            3450
#define MACH_TYPE_TRITIP               3451
#define MACH_TYPE_SM1K                 3452
#define MACH_TYPE_MONCH                3453
#define MACH_TYPE_CURACAO              3454
#define MACH_TYPE_ORIGEN               3455
#define MACH_TYPE_EPC10                3456
#define MACH_TYPE_SGH_I740             3457
#define MACH_TYPE_TUNA                 3458
#define MACH_TYPE_MX51_TULIP           3459
#define MACH_TYPE_MX51_ASTER7          3460
#define MACH_TYPE_ACRO37XBRD           3461
#define MACH_TYPE_ELKE                 3462
#define MACH_TYPE_SBC6000X             3463
#define MACH_TYPE_R1801E               3464
#define MACH_TYPE_H1600                3465
#define MACH_TYPE_MINI210              3466
#define MACH_TYPE_MINI8168             3467
#define MACH_TYPE_PC7308               3468
#define MACH_TYPE_KMM2M01              3470
#define MACH_TYPE_MX51EREBUS           3471
#define MACH_TYPE_WM8650REFBOARD       3472
#define MACH_TYPE_TUXRAIL              3473
#define MACH_TYPE_ARTHUR               3474
#define MACH_TYPE_DOORBOY              3475
#define MACH_TYPE_XARINA               3476
#define MACH_TYPE_ROVERX7              3477
#define MACH_TYPE_SDVR                 3478
#define MACH_TYPE_ACER_MAYA            3479
#define MACH_TYPE_PICO                 3480
#define MACH_TYPE_CWMX233              3481
#define MACH_TYPE_CWAM1808             3482
#define MACH_TYPE_CWDM365              3483
#define MACH_TYPE_MX51_MORAY           3484
#define MACH_TYPE_THALES_CBC           3485
#define MACH_TYPE_BLUEPOINT            3486
#define MACH_TYPE_DIR665               3487
#define MACH_TYPE_ACMEROVER1           3488
#define MACH_TYPE_SHOOTER_CT           3489
#define MACH_TYPE_BLISS                3490
#define MACH_TYPE_BLISSC               3491
#define MACH_TYPE_THALES_ADC           3492
#define MACH_TYPE_UBISYS_P9D_EVP       3493
#define MACH_TYPE_ATDGP318             3494
#define MACH_TYPE_OMAP5_SEVM           3777
#define MACH_TYPE_PICOSAM9G45          3838
#define MACH_TYPE_ARMADILLO_800EVA     3863
#define MACH_TYPE_KZM9G                4140
#define MACH_TYPE_COLIBRI_T30          4493
#define MACH_TYPE_APALIS_T30           4513
#define MACH_TYPE_OMAPL138_LCDK        2495

#ifdef CONFIG_ARCH_EBSA110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EBSA110
# endif
# define machine_is_ebsa110()	(machine_arch_type == MACH_TYPE_EBSA110)
#else
# define machine_is_ebsa110()	(0)
#endif

#ifdef CONFIG_ARCH_RPC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RISCPC
# endif
# define machine_is_riscpc()	(machine_arch_type == MACH_TYPE_RISCPC)
#else
# define machine_is_riscpc()	(0)
#endif

#ifdef CONFIG_ARCH_EBSA285
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EBSA285
# endif
# define machine_is_ebsa285()	(machine_arch_type == MACH_TYPE_EBSA285)
#else
# define machine_is_ebsa285()	(0)
#endif

#ifdef CONFIG_ARCH_NETWINDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETWINDER
# endif
# define machine_is_netwinder()	(machine_arch_type == MACH_TYPE_NETWINDER)
#else
# define machine_is_netwinder()	(0)
#endif

#ifdef CONFIG_ARCH_CATS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CATS
# endif
# define machine_is_cats()	(machine_arch_type == MACH_TYPE_CATS)
#else
# define machine_is_cats()	(0)
#endif

#ifdef CONFIG_ARCH_SHARK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHARK
# endif
# define machine_is_shark()	(machine_arch_type == MACH_TYPE_SHARK)
#else
# define machine_is_shark()	(0)
#endif

#ifdef CONFIG_SA1100_BRUTUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BRUTUS
# endif
# define machine_is_brutus()	(machine_arch_type == MACH_TYPE_BRUTUS)
#else
# define machine_is_brutus()	(0)
#endif

#ifdef CONFIG_ARCH_PERSONAL_SERVER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PERSONAL_SERVER
# endif
# define machine_is_personal_server()	(machine_arch_type == MACH_TYPE_PERSONAL_SERVER)
#else
# define machine_is_personal_server()	(0)
#endif

#ifdef CONFIG_ARCH_L7200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_L7200
# endif
# define machine_is_l7200()	(machine_arch_type == MACH_TYPE_L7200)
#else
# define machine_is_l7200()	(0)
#endif

#ifdef CONFIG_SA1100_PLEB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PLEB
# endif
# define machine_is_pleb()	(machine_arch_type == MACH_TYPE_PLEB)
#else
# define machine_is_pleb()	(0)
#endif

#ifdef CONFIG_ARCH_INTEGRATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INTEGRATOR
# endif
# define machine_is_integrator()	(machine_arch_type == MACH_TYPE_INTEGRATOR)
#else
# define machine_is_integrator()	(0)
#endif

#ifdef CONFIG_SA1100_H3600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3600
# endif
# define machine_is_h3600()	(machine_arch_type == MACH_TYPE_H3600)
#else
# define machine_is_h3600()	(0)
#endif

#ifdef CONFIG_ARCH_P720T
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_P720T
# endif
# define machine_is_p720t()	(machine_arch_type == MACH_TYPE_P720T)
#else
# define machine_is_p720t()	(0)
#endif

#ifdef CONFIG_SA1100_ASSABET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASSABET
# endif
# define machine_is_assabet()	(machine_arch_type == MACH_TYPE_ASSABET)
#else
# define machine_is_assabet()	(0)
#endif

#ifdef CONFIG_SA1100_LART
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LART
# endif
# define machine_is_lart()	(machine_arch_type == MACH_TYPE_LART)
#else
# define machine_is_lart()	(0)
#endif

#ifdef CONFIG_SA1100_GRAPHICSCLIENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRAPHICSCLIENT
# endif
# define machine_is_graphicsclient()	(machine_arch_type == MACH_TYPE_GRAPHICSCLIENT)
#else
# define machine_is_graphicsclient()	(0)
#endif

#ifdef CONFIG_SA1100_XP860
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XP860
# endif
# define machine_is_xp860()	(machine_arch_type == MACH_TYPE_XP860)
#else
# define machine_is_xp860()	(0)
#endif

#ifdef CONFIG_SA1100_CERF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CERF
# endif
# define machine_is_cerf()	(machine_arch_type == MACH_TYPE_CERF)
#else
# define machine_is_cerf()	(0)
#endif

#ifdef CONFIG_SA1100_NANOENGINE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NANOENGINE
# endif
# define machine_is_nanoengine()	(machine_arch_type == MACH_TYPE_NANOENGINE)
#else
# define machine_is_nanoengine()	(0)
#endif

#ifdef CONFIG_SA1100_JORNADA720
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JORNADA720
# endif
# define machine_is_jornada720()	(machine_arch_type == MACH_TYPE_JORNADA720)
#else
# define machine_is_jornada720()	(0)
#endif

#ifdef CONFIG_ARCH_EDB7211
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB7211
# endif
# define machine_is_edb7211()	(machine_arch_type == MACH_TYPE_EDB7211)
#else
# define machine_is_edb7211()	(0)
#endif

#ifdef CONFIG_SA1100_PFS168
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PFS168
# endif
# define machine_is_pfs168()	(machine_arch_type == MACH_TYPE_PFS168)
#else
# define machine_is_pfs168()	(0)
#endif

#ifdef CONFIG_SA1100_FLEXANET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLEXANET
# endif
# define machine_is_flexanet()	(machine_arch_type == MACH_TYPE_FLEXANET)
#else
# define machine_is_flexanet()	(0)
#endif

#ifdef CONFIG_SA1100_SIMPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMPAD
# endif
# define machine_is_simpad()	(machine_arch_type == MACH_TYPE_SIMPAD)
#else
# define machine_is_simpad()	(0)
#endif

#ifdef CONFIG_ARCH_LUBBOCK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LUBBOCK
# endif
# define machine_is_lubbock()	(machine_arch_type == MACH_TYPE_LUBBOCK)
#else
# define machine_is_lubbock()	(0)
#endif

#ifdef CONFIG_ARCH_CLEP7212
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CLEP7212
# endif
# define machine_is_clep7212()	(machine_arch_type == MACH_TYPE_CLEP7212)
#else
# define machine_is_clep7212()	(0)
#endif

#ifdef CONFIG_SA1100_SHANNON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHANNON
# endif
# define machine_is_shannon()	(machine_arch_type == MACH_TYPE_SHANNON)
#else
# define machine_is_shannon()	(0)
#endif

#ifdef CONFIG_SA1100_CONSUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CONSUS
# endif
# define machine_is_consus()	(machine_arch_type == MACH_TYPE_CONSUS)
#else
# define machine_is_consus()	(0)
#endif

#ifdef CONFIG_ARCH_AAED2000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AAED2000
# endif
# define machine_is_aaed2000()	(machine_arch_type == MACH_TYPE_AAED2000)
#else
# define machine_is_aaed2000()	(0)
#endif

#ifdef CONFIG_ARCH_CDB89712
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CDB89712
# endif
# define machine_is_cdb89712()	(machine_arch_type == MACH_TYPE_CDB89712)
#else
# define machine_is_cdb89712()	(0)
#endif

#ifdef CONFIG_SA1100_GRAPHICSMASTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GRAPHICSMASTER
# endif
# define machine_is_graphicsmaster()	(machine_arch_type == MACH_TYPE_GRAPHICSMASTER)
#else
# define machine_is_graphicsmaster()	(0)
#endif

#ifdef CONFIG_SA1100_ADSBITSY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSBITSY
# endif
# define machine_is_adsbitsy()	(machine_arch_type == MACH_TYPE_ADSBITSY)
#else
# define machine_is_adsbitsy()	(0)
#endif

#ifdef CONFIG_ARCH_PXA_IDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXA_IDP
# endif
# define machine_is_pxa_idp()	(machine_arch_type == MACH_TYPE_PXA_IDP)
#else
# define machine_is_pxa_idp()	(0)
#endif

#ifdef CONFIG_SA1100_PT_SYSTEM3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PT_SYSTEM3
# endif
# define machine_is_pt_system3()	(machine_arch_type == MACH_TYPE_PT_SYSTEM3)
#else
# define machine_is_pt_system3()	(0)
#endif

#ifdef CONFIG_ARCH_AUTCPU12
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AUTCPU12
# endif
# define machine_is_autcpu12()	(machine_arch_type == MACH_TYPE_AUTCPU12)
#else
# define machine_is_autcpu12()	(0)
#endif

#ifdef CONFIG_SA1100_H3100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H3100
# endif
# define machine_is_h3100()	(machine_arch_type == MACH_TYPE_H3100)
#else
# define machine_is_h3100()	(0)
#endif

#ifdef CONFIG_SA1100_COLLIE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLLIE
# endif
# define machine_is_collie()	(machine_arch_type == MACH_TYPE_COLLIE)
#else
# define machine_is_collie()	(0)
#endif

#ifdef CONFIG_SA1100_BADGE4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BADGE4
# endif
# define machine_is_badge4()	(machine_arch_type == MACH_TYPE_BADGE4)
#else
# define machine_is_badge4()	(0)
#endif

#ifdef CONFIG_ARCH_FORTUNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FORTUNET
# endif
# define machine_is_fortunet()	(machine_arch_type == MACH_TYPE_FORTUNET)
#else
# define machine_is_fortunet()	(0)
#endif

#ifdef CONFIG_ARCH_MX1ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX1ADS
# endif
# define machine_is_mx1ads()	(machine_arch_type == MACH_TYPE_MX1ADS)
#else
# define machine_is_mx1ads()	(0)
#endif

#ifdef CONFIG_ARCH_H7201
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H7201
# endif
# define machine_is_h7201()	(machine_arch_type == MACH_TYPE_H7201)
#else
# define machine_is_h7201()	(0)
#endif

#ifdef CONFIG_ARCH_H7202
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H7202
# endif
# define machine_is_h7202()	(machine_arch_type == MACH_TYPE_H7202)
#else
# define machine_is_h7202()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80321
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80321
# endif
# define machine_is_iq80321()	(machine_arch_type == MACH_TYPE_IQ80321)
#else
# define machine_is_iq80321()	(0)
#endif

#ifdef CONFIG_ARCH_KS8695
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KS8695
# endif
# define machine_is_ks8695()	(machine_arch_type == MACH_TYPE_KS8695)
#else
# define machine_is_ks8695()	(0)
#endif

#ifdef CONFIG_ARCH_SMDK2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2410
# endif
# define machine_is_smdk2410()	(machine_arch_type == MACH_TYPE_SMDK2410)
#else
# define machine_is_smdk2410()	(0)
#endif

#ifdef CONFIG_ARCH_CEIVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CEIVA
# endif
# define machine_is_ceiva()	(machine_arch_type == MACH_TYPE_CEIVA)
#else
# define machine_is_ceiva()	(0)
#endif

#ifdef CONFIG_MACH_VOICEBLUE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VOICEBLUE
# endif
# define machine_is_voiceblue()	(machine_arch_type == MACH_TYPE_VOICEBLUE)
#else
# define machine_is_voiceblue()	(0)
#endif

#ifdef CONFIG_ARCH_H5400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H5400
# endif
# define machine_is_h5400()	(machine_arch_type == MACH_TYPE_H5400)
#else
# define machine_is_h5400()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_INNOVATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_INNOVATOR
# endif
# define machine_is_omap_innovator()	(machine_arch_type == MACH_TYPE_OMAP_INNOVATOR)
#else
# define machine_is_omap_innovator()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2400
# endif
# define machine_is_ixdp2400()	(machine_arch_type == MACH_TYPE_IXDP2400)
#else
# define machine_is_ixdp2400()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2800
# endif
# define machine_is_ixdp2800()	(machine_arch_type == MACH_TYPE_IXDP2800)
#else
# define machine_is_ixdp2800()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP425
# endif
# define machine_is_ixdp425()	(machine_arch_type == MACH_TYPE_IXDP425)
#else
# define machine_is_ixdp425()	(0)
#endif

#ifdef CONFIG_SA1100_HACKKIT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HACKKIT
# endif
# define machine_is_hackkit()	(machine_arch_type == MACH_TYPE_HACKKIT)
#else
# define machine_is_hackkit()	(0)
#endif

#ifdef CONFIG_ARCH_IXCDP1100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXCDP1100
# endif
# define machine_is_ixcdp1100()	(machine_arch_type == MACH_TYPE_IXCDP1100)
#else
# define machine_is_ixcdp1100()	(0)
#endif

#ifdef CONFIG_ARCH_AT91RM9200DK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200DK
# endif
# define machine_is_at91rm9200dk()	(machine_arch_type == MACH_TYPE_AT91RM9200DK)
#else
# define machine_is_at91rm9200dk()	(0)
#endif

#ifdef CONFIG_ARCH_CINTEGRATOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CINTEGRATOR
# endif
# define machine_is_cintegrator()	(machine_arch_type == MACH_TYPE_CINTEGRATOR)
#else
# define machine_is_cintegrator()	(0)
#endif

#ifdef CONFIG_ARCH_VIPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIPER
# endif
# define machine_is_viper()	(machine_arch_type == MACH_TYPE_VIPER)
#else
# define machine_is_viper()	(0)
#endif

#ifdef CONFIG_ARCH_ADI_COYOTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADI_COYOTE
# endif
# define machine_is_adi_coyote()	(machine_arch_type == MACH_TYPE_ADI_COYOTE)
#else
# define machine_is_adi_coyote()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2401
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2401
# endif
# define machine_is_ixdp2401()	(machine_arch_type == MACH_TYPE_IXDP2401)
#else
# define machine_is_ixdp2401()	(0)
#endif

#ifdef CONFIG_ARCH_IXDP2801
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2801
# endif
# define machine_is_ixdp2801()	(machine_arch_type == MACH_TYPE_IXDP2801)
#else
# define machine_is_ixdp2801()	(0)
#endif

#ifdef CONFIG_ARCH_IQ31244
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ31244
# endif
# define machine_is_iq31244()	(machine_arch_type == MACH_TYPE_IQ31244)
#else
# define machine_is_iq31244()	(0)
#endif

#ifdef CONFIG_ARCH_BAST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BAST
# endif
# define machine_is_bast()	(machine_arch_type == MACH_TYPE_BAST)
#else
# define machine_is_bast()	(0)
#endif

#ifdef CONFIG_ARCH_H1940
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H1940
# endif
# define machine_is_h1940()	(machine_arch_type == MACH_TYPE_H1940)
#else
# define machine_is_h1940()	(0)
#endif

#ifdef CONFIG_ARCH_ENP2611
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENP2611
# endif
# define machine_is_enp2611()	(machine_arch_type == MACH_TYPE_ENP2611)
#else
# define machine_is_enp2611()	(0)
#endif

#ifdef CONFIG_ARCH_S3C2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2440
# endif
# define machine_is_s3c2440()	(machine_arch_type == MACH_TYPE_S3C2440)
#else
# define machine_is_s3c2440()	(0)
#endif

#ifdef CONFIG_ARCH_GUMSTIX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GUMSTIX
# endif
# define machine_is_gumstix()	(machine_arch_type == MACH_TYPE_GUMSTIX)
#else
# define machine_is_gumstix()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H2
# endif
# define machine_is_omap_h2()	(machine_arch_type == MACH_TYPE_OMAP_H2)
#else
# define machine_is_omap_h2()	(0)
#endif

#ifdef CONFIG_MACH_E740
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E740
# endif
# define machine_is_e740()	(machine_arch_type == MACH_TYPE_E740)
#else
# define machine_is_e740()	(0)
#endif

#ifdef CONFIG_ARCH_IQ80331
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80331
# endif
# define machine_is_iq80331()	(machine_arch_type == MACH_TYPE_IQ80331)
#else
# define machine_is_iq80331()	(0)
#endif

#ifdef CONFIG_ARCH_VERSATILE_PB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERSATILE_PB
# endif
# define machine_is_versatile_pb()	(machine_arch_type == MACH_TYPE_VERSATILE_PB)
#else
# define machine_is_versatile_pb()	(0)
#endif

#ifdef CONFIG_MACH_KEV7A400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KEV7A400
# endif
# define machine_is_kev7a400()	(machine_arch_type == MACH_TYPE_KEV7A400)
#else
# define machine_is_kev7a400()	(0)
#endif

#ifdef CONFIG_MACH_LPD7A400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD7A400
# endif
# define machine_is_lpd7a400()	(machine_arch_type == MACH_TYPE_LPD7A400)
#else
# define machine_is_lpd7a400()	(0)
#endif

#ifdef CONFIG_MACH_LPD7A404
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPD7A404
# endif
# define machine_is_lpd7a404()	(machine_arch_type == MACH_TYPE_LPD7A404)
#else
# define machine_is_lpd7a404()	(0)
#endif

#ifdef CONFIG_MACH_CSB337
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB337
# endif
# define machine_is_csb337()	(machine_arch_type == MACH_TYPE_CSB337)
#else
# define machine_is_csb337()	(0)
#endif

#ifdef CONFIG_MACH_MAINSTONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAINSTONE
# endif
# define machine_is_mainstone()	(machine_arch_type == MACH_TYPE_MAINSTONE)
#else
# define machine_is_mainstone()	(0)
#endif

#ifdef CONFIG_MACH_XCEP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XCEP
# endif
# define machine_is_xcep()	(machine_arch_type == MACH_TYPE_XCEP)
#else
# define machine_is_xcep()	(0)
#endif

#ifdef CONFIG_MACH_ARCOM_VULCAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCOM_VULCAN
# endif
# define machine_is_arcom_vulcan()	(machine_arch_type == MACH_TYPE_ARCOM_VULCAN)
#else
# define machine_is_arcom_vulcan()	(0)
#endif

#ifdef CONFIG_MACH_NOMADIK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOMADIK
# endif
# define machine_is_nomadik()	(machine_arch_type == MACH_TYPE_NOMADIK)
#else
# define machine_is_nomadik()	(0)
#endif

#ifdef CONFIG_MACH_CORGI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CORGI
# endif
# define machine_is_corgi()	(machine_arch_type == MACH_TYPE_CORGI)
#else
# define machine_is_corgi()	(0)
#endif

#ifdef CONFIG_MACH_POODLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POODLE
# endif
# define machine_is_poodle()	(machine_arch_type == MACH_TYPE_POODLE)
#else
# define machine_is_poodle()	(0)
#endif

#ifdef CONFIG_MACH_ARMCORE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMCORE
# endif
# define machine_is_armcore()	(machine_arch_type == MACH_TYPE_ARMCORE)
#else
# define machine_is_armcore()	(0)
#endif

#ifdef CONFIG_MACH_MX31ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX31ADS
# endif
# define machine_is_mx31ads()	(machine_arch_type == MACH_TYPE_MX31ADS)
#else
# define machine_is_mx31ads()	(0)
#endif

#ifdef CONFIG_MACH_HIMALAYA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HIMALAYA
# endif
# define machine_is_himalaya()	(machine_arch_type == MACH_TYPE_HIMALAYA)
#else
# define machine_is_himalaya()	(0)
#endif

#ifdef CONFIG_MACH_EDB9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9312
# endif
# define machine_is_edb9312()	(machine_arch_type == MACH_TYPE_EDB9312)
#else
# define machine_is_edb9312()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_GENERIC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_GENERIC
# endif
# define machine_is_omap_generic()	(machine_arch_type == MACH_TYPE_OMAP_GENERIC)
#else
# define machine_is_omap_generic()	(0)
#endif

#ifdef CONFIG_MACH_EDB9301
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9301
# endif
# define machine_is_edb9301()	(machine_arch_type == MACH_TYPE_EDB9301)
#else
# define machine_is_edb9301()	(0)
#endif

#ifdef CONFIG_MACH_EDB9315
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9315
# endif
# define machine_is_edb9315()	(machine_arch_type == MACH_TYPE_EDB9315)
#else
# define machine_is_edb9315()	(0)
#endif

#ifdef CONFIG_MACH_VR1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VR1000
# endif
# define machine_is_vr1000()	(machine_arch_type == MACH_TYPE_VR1000)
#else
# define machine_is_vr1000()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PERSEUS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PERSEUS2
# endif
# define machine_is_omap_perseus2()	(machine_arch_type == MACH_TYPE_OMAP_PERSEUS2)
#else
# define machine_is_omap_perseus2()	(0)
#endif

#ifdef CONFIG_MACH_E800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E800
# endif
# define machine_is_e800()	(machine_arch_type == MACH_TYPE_E800)
#else
# define machine_is_e800()	(0)
#endif

#ifdef CONFIG_MACH_E750
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E750
# endif
# define machine_is_e750()	(machine_arch_type == MACH_TYPE_E750)
#else
# define machine_is_e750()	(0)
#endif

#ifdef CONFIG_MACH_SCB9328
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SCB9328
# endif
# define machine_is_scb9328()	(machine_arch_type == MACH_TYPE_SCB9328)
#else
# define machine_is_scb9328()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H3
# endif
# define machine_is_omap_h3()	(machine_arch_type == MACH_TYPE_OMAP_H3)
#else
# define machine_is_omap_h3()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_H4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_H4
# endif
# define machine_is_omap_h4()	(machine_arch_type == MACH_TYPE_OMAP_H4)
#else
# define machine_is_omap_h4()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_OSK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_OSK
# endif
# define machine_is_omap_osk()	(machine_arch_type == MACH_TYPE_OMAP_OSK)
#else
# define machine_is_omap_osk()	(0)
#endif

#ifdef CONFIG_MACH_TOSA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOSA
# endif
# define machine_is_tosa()	(machine_arch_type == MACH_TYPE_TOSA)
#else
# define machine_is_tosa()	(0)
#endif

#ifdef CONFIG_MACH_AVILA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AVILA
# endif
# define machine_is_avila()	(machine_arch_type == MACH_TYPE_AVILA)
#else
# define machine_is_avila()	(0)
#endif

#ifdef CONFIG_MACH_EDB9302
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9302
# endif
# define machine_is_edb9302()	(machine_arch_type == MACH_TYPE_EDB9302)
#else
# define machine_is_edb9302()	(0)
#endif

#ifdef CONFIG_MACH_HUSKY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HUSKY
# endif
# define machine_is_husky()	(machine_arch_type == MACH_TYPE_HUSKY)
#else
# define machine_is_husky()	(0)
#endif

#ifdef CONFIG_MACH_SHEPHERD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHEPHERD
# endif
# define machine_is_shepherd()	(machine_arch_type == MACH_TYPE_SHEPHERD)
#else
# define machine_is_shepherd()	(0)
#endif

#ifdef CONFIG_MACH_H4700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H4700
# endif
# define machine_is_h4700()	(machine_arch_type == MACH_TYPE_H4700)
#else
# define machine_is_h4700()	(0)
#endif

#ifdef CONFIG_MACH_RX3715
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RX3715
# endif
# define machine_is_rx3715()	(machine_arch_type == MACH_TYPE_RX3715)
#else
# define machine_is_rx3715()	(0)
#endif

#ifdef CONFIG_MACH_NSLU2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSLU2
# endif
# define machine_is_nslu2()	(machine_arch_type == MACH_TYPE_NSLU2)
#else
# define machine_is_nslu2()	(0)
#endif

#ifdef CONFIG_MACH_E400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E400
# endif
# define machine_is_e400()	(machine_arch_type == MACH_TYPE_E400)
#else
# define machine_is_e400()	(0)
#endif

#ifdef CONFIG_MACH_IXDPG425
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDPG425
# endif
# define machine_is_ixdpg425()	(machine_arch_type == MACH_TYPE_IXDPG425)
#else
# define machine_is_ixdpg425()	(0)
#endif

#ifdef CONFIG_MACH_VERSATILE_AB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERSATILE_AB
# endif
# define machine_is_versatile_ab()	(machine_arch_type == MACH_TYPE_VERSATILE_AB)
#else
# define machine_is_versatile_ab()	(0)
#endif

#ifdef CONFIG_MACH_EDB9307
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9307
# endif
# define machine_is_edb9307()	(machine_arch_type == MACH_TYPE_EDB9307)
#else
# define machine_is_edb9307()	(0)
#endif

#ifdef CONFIG_MACH_KB9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KB9200
# endif
# define machine_is_kb9200()	(machine_arch_type == MACH_TYPE_KB9200)
#else
# define machine_is_kb9200()	(0)
#endif

#ifdef CONFIG_MACH_SX1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SX1
# endif
# define machine_is_sx1()	(machine_arch_type == MACH_TYPE_SX1)
#else
# define machine_is_sx1()	(0)
#endif

#ifdef CONFIG_MACH_IXDP465
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP465
# endif
# define machine_is_ixdp465()	(machine_arch_type == MACH_TYPE_IXDP465)
#else
# define machine_is_ixdp465()	(0)
#endif

#ifdef CONFIG_MACH_IXDP2351
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP2351
# endif
# define machine_is_ixdp2351()	(machine_arch_type == MACH_TYPE_IXDP2351)
#else
# define machine_is_ixdp2351()	(0)
#endif

#ifdef CONFIG_MACH_IQ80332
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ80332
# endif
# define machine_is_iq80332()	(machine_arch_type == MACH_TYPE_IQ80332)
#else
# define machine_is_iq80332()	(0)
#endif

#ifdef CONFIG_MACH_GTWX5715
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GTWX5715
# endif
# define machine_is_gtwx5715()	(machine_arch_type == MACH_TYPE_GTWX5715)
#else
# define machine_is_gtwx5715()	(0)
#endif

#ifdef CONFIG_MACH_CSB637
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB637
# endif
# define machine_is_csb637()	(machine_arch_type == MACH_TYPE_CSB637)
#else
# define machine_is_csb637()	(0)
#endif

#ifdef CONFIG_MACH_N30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N30
# endif
# define machine_is_n30()	(machine_arch_type == MACH_TYPE_N30)
#else
# define machine_is_n30()	(0)
#endif

#ifdef CONFIG_MACH_NEC_MP900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEC_MP900
# endif
# define machine_is_nec_mp900()	(machine_arch_type == MACH_TYPE_NEC_MP900)
#else
# define machine_is_nec_mp900()	(0)
#endif

#ifdef CONFIG_MACH_KAFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KAFA
# endif
# define machine_is_kafa()	(machine_arch_type == MACH_TYPE_KAFA)
#else
# define machine_is_kafa()	(0)
#endif

#ifdef CONFIG_MACH_TS72XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS72XX
# endif
# define machine_is_ts72xx()	(machine_arch_type == MACH_TYPE_TS72XX)
#else
# define machine_is_ts72xx()	(0)
#endif

#ifdef CONFIG_MACH_OTOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OTOM
# endif
# define machine_is_otom()	(machine_arch_type == MACH_TYPE_OTOM)
#else
# define machine_is_otom()	(0)
#endif

#ifdef CONFIG_MACH_NEXCODER_2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEXCODER_2440
# endif
# define machine_is_nexcoder_2440()	(machine_arch_type == MACH_TYPE_NEXCODER_2440)
#else
# define machine_is_nexcoder_2440()	(0)
#endif

#ifdef CONFIG_MACH_ECO920
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECO920
# endif
# define machine_is_eco920()	(machine_arch_type == MACH_TYPE_ECO920)
#else
# define machine_is_eco920()	(0)
#endif

#ifdef CONFIG_MACH_ROADRUNNER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROADRUNNER
# endif
# define machine_is_roadrunner()	(machine_arch_type == MACH_TYPE_ROADRUNNER)
#else
# define machine_is_roadrunner()	(0)
#endif

#ifdef CONFIG_MACH_AT91RM9200EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91RM9200EK
# endif
# define machine_is_at91rm9200ek()	(machine_arch_type == MACH_TYPE_AT91RM9200EK)
#else
# define machine_is_at91rm9200ek()	(0)
#endif

#ifdef CONFIG_MACH_SPITZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPITZ
# endif
# define machine_is_spitz()	(machine_arch_type == MACH_TYPE_SPITZ)
#else
# define machine_is_spitz()	(0)
#endif

#ifdef CONFIG_MACH_ADSSPHERE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ADSSPHERE
# endif
# define machine_is_adssphere()	(machine_arch_type == MACH_TYPE_ADSSPHERE)
#else
# define machine_is_adssphere()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI
# endif
# define machine_is_colibri()	(machine_arch_type == MACH_TYPE_COLIBRI)
#else
# define machine_is_colibri()	(0)
#endif

#ifdef CONFIG_MACH_GATEWAY7001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GATEWAY7001
# endif
# define machine_is_gateway7001()	(machine_arch_type == MACH_TYPE_GATEWAY7001)
#else
# define machine_is_gateway7001()	(0)
#endif

#ifdef CONFIG_MACH_PCM027
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM027
# endif
# define machine_is_pcm027()	(machine_arch_type == MACH_TYPE_PCM027)
#else
# define machine_is_pcm027()	(0)
#endif

#ifdef CONFIG_MACH_ANUBIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANUBIS
# endif
# define machine_is_anubis()	(machine_arch_type == MACH_TYPE_ANUBIS)
#else
# define machine_is_anubis()	(0)
#endif

#ifdef CONFIG_MACH_AKITA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AKITA
# endif
# define machine_is_akita()	(machine_arch_type == MACH_TYPE_AKITA)
#else
# define machine_is_akita()	(0)
#endif

#ifdef CONFIG_MACH_E330
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E330
# endif
# define machine_is_e330()	(machine_arch_type == MACH_TYPE_E330)
#else
# define machine_is_e330()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA770
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA770
# endif
# define machine_is_nokia770()	(machine_arch_type == MACH_TYPE_NOKIA770)
#else
# define machine_is_nokia770()	(0)
#endif

#ifdef CONFIG_MACH_CARMEVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CARMEVA
# endif
# define machine_is_carmeva()	(machine_arch_type == MACH_TYPE_CARMEVA)
#else
# define machine_is_carmeva()	(0)
#endif

#ifdef CONFIG_MACH_EDB9315A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9315A
# endif
# define machine_is_edb9315a()	(machine_arch_type == MACH_TYPE_EDB9315A)
#else
# define machine_is_edb9315a()	(0)
#endif

#ifdef CONFIG_MACH_STARGATE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STARGATE2
# endif
# define machine_is_stargate2()	(machine_arch_type == MACH_TYPE_STARGATE2)
#else
# define machine_is_stargate2()	(0)
#endif

#ifdef CONFIG_MACH_INTELMOTE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INTELMOTE2
# endif
# define machine_is_intelmote2()	(machine_arch_type == MACH_TYPE_INTELMOTE2)
#else
# define machine_is_intelmote2()	(0)
#endif

#ifdef CONFIG_MACH_TRIZEPS4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS4
# endif
# define machine_is_trizeps4()	(machine_arch_type == MACH_TYPE_TRIZEPS4)
#else
# define machine_is_trizeps4()	(0)
#endif

#ifdef CONFIG_MACH_PNX4008
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PNX4008
# endif
# define machine_is_pnx4008()	(machine_arch_type == MACH_TYPE_PNX4008)
#else
# define machine_is_pnx4008()	(0)
#endif

#ifdef CONFIG_MACH_IQ81340SC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ81340SC
# endif
# define machine_is_iq81340sc()	(machine_arch_type == MACH_TYPE_IQ81340SC)
#else
# define machine_is_iq81340sc()	(0)
#endif

#ifdef CONFIG_MACH_IQ81340MC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IQ81340MC
# endif
# define machine_is_iq81340mc()	(machine_arch_type == MACH_TYPE_IQ81340MC)
#else
# define machine_is_iq81340mc()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9
# endif
# define machine_is_micro9()	(machine_arch_type == MACH_TYPE_MICRO9)
#else
# define machine_is_micro9()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9L
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9L
# endif
# define machine_is_micro9l()	(machine_arch_type == MACH_TYPE_MICRO9L)
#else
# define machine_is_micro9l()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTE
# endif
# define machine_is_omap_palmte()	(machine_arch_type == MACH_TYPE_OMAP_PALMTE)
#else
# define machine_is_omap_palmte()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_EB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_EB
# endif
# define machine_is_realview_eb()	(machine_arch_type == MACH_TYPE_REALVIEW_EB)
#else
# define machine_is_realview_eb()	(0)
#endif

#ifdef CONFIG_MACH_BORZOI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BORZOI
# endif
# define machine_is_borzoi()	(machine_arch_type == MACH_TYPE_BORZOI)
#else
# define machine_is_borzoi()	(0)
#endif

#ifdef CONFIG_MACH_PALMLD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMLD
# endif
# define machine_is_palmld()	(machine_arch_type == MACH_TYPE_PALMLD)
#else
# define machine_is_palmld()	(0)
#endif

#ifdef CONFIG_MACH_IXDP28X5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IXDP28X5
# endif
# define machine_is_ixdp28x5()	(machine_arch_type == MACH_TYPE_IXDP28X5)
#else
# define machine_is_ixdp28x5()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMTT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMTT
# endif
# define machine_is_omap_palmtt()	(machine_arch_type == MACH_TYPE_OMAP_PALMTT)
#else
# define machine_is_omap_palmtt()	(0)
#endif

#ifdef CONFIG_MACH_ARCOM_ZEUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARCOM_ZEUS
# endif
# define machine_is_arcom_zeus()	(machine_arch_type == MACH_TYPE_ARCOM_ZEUS)
#else
# define machine_is_arcom_zeus()	(0)
#endif

#ifdef CONFIG_MACH_OSIRIS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OSIRIS
# endif
# define machine_is_osiris()	(machine_arch_type == MACH_TYPE_OSIRIS)
#else
# define machine_is_osiris()	(0)
#endif

#ifdef CONFIG_MACH_PALMTE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMTE2
# endif
# define machine_is_palmte2()	(machine_arch_type == MACH_TYPE_PALMTE2)
#else
# define machine_is_palmte2()	(0)
#endif

#ifdef CONFIG_MACH_MX27ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX27ADS
# endif
# define machine_is_mx27ads()	(machine_arch_type == MACH_TYPE_MX27ADS)
#else
# define machine_is_mx27ads()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9261EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9261EK
# endif
# define machine_is_at91sam9261ek()	(machine_arch_type == MACH_TYPE_AT91SAM9261EK)
#else
# define machine_is_at91sam9261ek()	(0)
#endif

#ifdef CONFIG_MACH_LOFT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOFT
# endif
# define machine_is_loft()	(machine_arch_type == MACH_TYPE_LOFT)
#else
# define machine_is_loft()	(0)
#endif

#ifdef CONFIG_MACH_MX21ADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX21ADS
# endif
# define machine_is_mx21ads()	(machine_arch_type == MACH_TYPE_MX21ADS)
#else
# define machine_is_mx21ads()	(0)
#endif

#ifdef CONFIG_MACH_AMS_DELTA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AMS_DELTA
# endif
# define machine_is_ams_delta()	(machine_arch_type == MACH_TYPE_AMS_DELTA)
#else
# define machine_is_ams_delta()	(0)
#endif

#ifdef CONFIG_MACH_NAS100D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAS100D
# endif
# define machine_is_nas100d()	(machine_arch_type == MACH_TYPE_NAS100D)
#else
# define machine_is_nas100d()	(0)
#endif

#ifdef CONFIG_MACH_MAGICIAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAGICIAN
# endif
# define machine_is_magician()	(machine_arch_type == MACH_TYPE_MAGICIAN)
#else
# define machine_is_magician()	(0)
#endif

#ifdef CONFIG_MACH_NXDKN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXDKN
# endif
# define machine_is_nxdkn()	(machine_arch_type == MACH_TYPE_NXDKN)
#else
# define machine_is_nxdkn()	(0)
#endif

#ifdef CONFIG_MACH_PALMTX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMTX
# endif
# define machine_is_palmtx()	(machine_arch_type == MACH_TYPE_PALMTX)
#else
# define machine_is_palmtx()	(0)
#endif

#ifdef CONFIG_MACH_S3C2413
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S3C2413
# endif
# define machine_is_s3c2413()	(machine_arch_type == MACH_TYPE_S3C2413)
#else
# define machine_is_s3c2413()	(0)
#endif

#ifdef CONFIG_MACH_WG302V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WG302V2
# endif
# define machine_is_wg302v2()	(machine_arch_type == MACH_TYPE_WG302V2)
#else
# define machine_is_wg302v2()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_2430SDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_2430SDP
# endif
# define machine_is_omap_2430sdp()	(machine_arch_type == MACH_TYPE_OMAP_2430SDP)
#else
# define machine_is_omap_2430sdp()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_EVM
# endif
# define machine_is_davinci_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_EVM)
#else
# define machine_is_davinci_evm()	(0)
#endif

#ifdef CONFIG_MACH_PALMZ72
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMZ72
# endif
# define machine_is_palmz72()	(machine_arch_type == MACH_TYPE_PALMZ72)
#else
# define machine_is_palmz72()	(0)
#endif

#ifdef CONFIG_MACH_NXDB500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXDB500
# endif
# define machine_is_nxdb500()	(machine_arch_type == MACH_TYPE_NXDB500)
#else
# define machine_is_nxdb500()	(0)
#endif

#ifdef CONFIG_MACH_PALMT5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMT5
# endif
# define machine_is_palmt5()	(machine_arch_type == MACH_TYPE_PALMT5)
#else
# define machine_is_palmt5()	(0)
#endif

#ifdef CONFIG_MACH_PALMTC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PALMTC
# endif
# define machine_is_palmtc()	(machine_arch_type == MACH_TYPE_PALMTC)
#else
# define machine_is_palmtc()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_APOLLON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_APOLLON
# endif
# define machine_is_omap_apollon()	(machine_arch_type == MACH_TYPE_OMAP_APOLLON)
#else
# define machine_is_omap_apollon()	(0)
#endif

#ifdef CONFIG_MACH_ATEB9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATEB9200
# endif
# define machine_is_ateb9200()	(machine_arch_type == MACH_TYPE_ATEB9200)
#else
# define machine_is_ateb9200()	(0)
#endif

#ifdef CONFIG_MACH_N35
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N35
# endif
# define machine_is_n35()	(machine_arch_type == MACH_TYPE_N35)
#else
# define machine_is_n35()	(0)
#endif

#ifdef CONFIG_MACH_LOGICPD_PXA270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LOGICPD_PXA270
# endif
# define machine_is_logicpd_pxa270()	(machine_arch_type == MACH_TYPE_LOGICPD_PXA270)
#else
# define machine_is_logicpd_pxa270()	(0)
#endif

#ifdef CONFIG_MACH_NXEB500HMI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NXEB500HMI
# endif
# define machine_is_nxeb500hmi()	(machine_arch_type == MACH_TYPE_NXEB500HMI)
#else
# define machine_is_nxeb500hmi()	(0)
#endif

#ifdef CONFIG_MACH_ESPRESSO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESPRESSO
# endif
# define machine_is_espresso()	(machine_arch_type == MACH_TYPE_ESPRESSO)
#else
# define machine_is_espresso()	(0)
#endif

#ifdef CONFIG_MACH_RX1950
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RX1950
# endif
# define machine_is_rx1950()	(machine_arch_type == MACH_TYPE_RX1950)
#else
# define machine_is_rx1950()	(0)
#endif

#ifdef CONFIG_MACH_GESBC9312
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GESBC9312
# endif
# define machine_is_gesbc9312()	(machine_arch_type == MACH_TYPE_GESBC9312)
#else
# define machine_is_gesbc9312()	(0)
#endif

#ifdef CONFIG_MACH_PICOTUX2XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOTUX2XX
# endif
# define machine_is_picotux2xx()	(machine_arch_type == MACH_TYPE_PICOTUX2XX)
#else
# define machine_is_picotux2xx()	(0)
#endif

#ifdef CONFIG_MACH_DSMG600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSMG600
# endif
# define machine_is_dsmg600()	(machine_arch_type == MACH_TYPE_DSMG600)
#else
# define machine_is_dsmg600()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_FSAMPLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_FSAMPLE
# endif
# define machine_is_omap_fsample()	(machine_arch_type == MACH_TYPE_OMAP_FSAMPLE)
#else
# define machine_is_omap_fsample()	(0)
#endif

#ifdef CONFIG_MACH_SNAPPER_CL15
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SNAPPER_CL15
# endif
# define machine_is_snapper_cl15()	(machine_arch_type == MACH_TYPE_SNAPPER_CL15)
#else
# define machine_is_snapper_cl15()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_PALMZ71
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_PALMZ71
# endif
# define machine_is_omap_palmz71()	(machine_arch_type == MACH_TYPE_OMAP_PALMZ71)
#else
# define machine_is_omap_palmz71()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2412
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2412
# endif
# define machine_is_smdk2412()	(machine_arch_type == MACH_TYPE_SMDK2412)
#else
# define machine_is_smdk2412()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2413
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2413
# endif
# define machine_is_smdk2413()	(machine_arch_type == MACH_TYPE_SMDK2413)
#else
# define machine_is_smdk2413()	(0)
#endif

#ifdef CONFIG_MACH_AML_M5900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AML_M5900
# endif
# define machine_is_aml_m5900()	(machine_arch_type == MACH_TYPE_AML_M5900)
#else
# define machine_is_aml_m5900()	(0)
#endif

#ifdef CONFIG_MACH_ECBAT91
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECBAT91
# endif
# define machine_is_ecbat91()	(machine_arch_type == MACH_TYPE_ECBAT91)
#else
# define machine_is_ecbat91()	(0)
#endif

#ifdef CONFIG_MACH_ONEARM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ONEARM
# endif
# define machine_is_onearm()	(machine_arch_type == MACH_TYPE_ONEARM)
#else
# define machine_is_onearm()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2443
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2443
# endif
# define machine_is_smdk2443()	(machine_arch_type == MACH_TYPE_SMDK2443)
#else
# define machine_is_smdk2443()	(0)
#endif

#ifdef CONFIG_MACH_FSG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FSG
# endif
# define machine_is_fsg()	(machine_arch_type == MACH_TYPE_FSG)
#else
# define machine_is_fsg()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9260EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9260EK
# endif
# define machine_is_at91sam9260ek()	(machine_arch_type == MACH_TYPE_AT91SAM9260EK)
#else
# define machine_is_at91sam9260ek()	(0)
#endif

#ifdef CONFIG_MACH_GLANTANK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GLANTANK
# endif
# define machine_is_glantank()	(machine_arch_type == MACH_TYPE_GLANTANK)
#else
# define machine_is_glantank()	(0)
#endif

#ifdef CONFIG_MACH_N2100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_N2100
# endif
# define machine_is_n2100()	(machine_arch_type == MACH_TYPE_N2100)
#else
# define machine_is_n2100()	(0)
#endif

#ifdef CONFIG_MACH_QT2410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QT2410
# endif
# define machine_is_qt2410()	(machine_arch_type == MACH_TYPE_QT2410)
#else
# define machine_is_qt2410()	(0)
#endif

#ifdef CONFIG_MACH_KIXRP435
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KIXRP435
# endif
# define machine_is_kixrp435()	(machine_arch_type == MACH_TYPE_KIXRP435)
#else
# define machine_is_kixrp435()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9360DEV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9360DEV
# endif
# define machine_is_cc9p9360dev()	(machine_arch_type == MACH_TYPE_CC9P9360DEV)
#else
# define machine_is_cc9p9360dev()	(0)
#endif

#ifdef CONFIG_MACH_EDB9302A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9302A
# endif
# define machine_is_edb9302a()	(machine_arch_type == MACH_TYPE_EDB9302A)
#else
# define machine_is_edb9302a()	(0)
#endif

#ifdef CONFIG_MACH_EDB9307A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDB9307A
# endif
# define machine_is_edb9307a()	(machine_arch_type == MACH_TYPE_EDB9307A)
#else
# define machine_is_edb9307a()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_3430SDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_3430SDP
# endif
# define machine_is_omap_3430sdp()	(machine_arch_type == MACH_TYPE_OMAP_3430SDP)
#else
# define machine_is_omap_3430sdp()	(0)
#endif

#ifdef CONFIG_MACH_VSTMS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VSTMS
# endif
# define machine_is_vstms()	(machine_arch_type == MACH_TYPE_VSTMS)
#else
# define machine_is_vstms()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9M
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9M
# endif
# define machine_is_micro9m()	(machine_arch_type == MACH_TYPE_MICRO9M)
#else
# define machine_is_micro9m()	(0)
#endif

#ifdef CONFIG_MACH_BUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BUG
# endif
# define machine_is_bug()	(machine_arch_type == MACH_TYPE_BUG)
#else
# define machine_is_bug()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9263EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9263EK
# endif
# define machine_is_at91sam9263ek()	(machine_arch_type == MACH_TYPE_AT91SAM9263EK)
#else
# define machine_is_at91sam9263ek()	(0)
#endif

#ifdef CONFIG_MACH_EM7210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EM7210
# endif
# define machine_is_em7210()	(machine_arch_type == MACH_TYPE_EM7210)
#else
# define machine_is_em7210()	(0)
#endif

#ifdef CONFIG_MACH_VPAC270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VPAC270
# endif
# define machine_is_vpac270()	(machine_arch_type == MACH_TYPE_VPAC270)
#else
# define machine_is_vpac270()	(0)
#endif

#ifdef CONFIG_MACH_TREO680
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TREO680
# endif
# define machine_is_treo680()	(machine_arch_type == MACH_TYPE_TREO680)
#else
# define machine_is_treo680()	(0)
#endif

#ifdef CONFIG_MACH_ZYLONITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZYLONITE
# endif
# define machine_is_zylonite()	(machine_arch_type == MACH_TYPE_ZYLONITE)
#else
# define machine_is_zylonite()	(0)
#endif

#ifdef CONFIG_MACH_MX31LITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX31LITE
# endif
# define machine_is_mx31lite()	(machine_arch_type == MACH_TYPE_MX31LITE)
#else
# define machine_is_mx31lite()	(0)
#endif

#ifdef CONFIG_MACH_MIOA701
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIOA701
# endif
# define machine_is_mioa701()	(machine_arch_type == MACH_TYPE_MIOA701)
#else
# define machine_is_mioa701()	(0)
#endif

#ifdef CONFIG_MACH_ARMADILLO5X0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMADILLO5X0
# endif
# define machine_is_armadillo5x0()	(machine_arch_type == MACH_TYPE_ARMADILLO5X0)
#else
# define machine_is_armadillo5x0()	(0)
#endif

#ifdef CONFIG_MACH_CC9P9360JS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CC9P9360JS
# endif
# define machine_is_cc9p9360js()	(machine_arch_type == MACH_TYPE_CC9P9360JS)
#else
# define machine_is_cc9p9360js()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA_N800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA_N800
# endif
# define machine_is_nokia_n800()	(machine_arch_type == MACH_TYPE_NOKIA_N800)
#else
# define machine_is_nokia_n800()	(0)
#endif

#ifdef CONFIG_MACH_EP80219
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EP80219
# endif
# define machine_is_ep80219()	(machine_arch_type == MACH_TYPE_EP80219)
#else
# define machine_is_ep80219()	(0)
#endif

#ifdef CONFIG_MACH_GORAMO_MLR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GORAMO_MLR
# endif
# define machine_is_goramo_mlr()	(machine_arch_type == MACH_TYPE_GORAMO_MLR)
#else
# define machine_is_goramo_mlr()	(0)
#endif

#ifdef CONFIG_MACH_EM_X270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EM_X270
# endif
# define machine_is_em_x270()	(machine_arch_type == MACH_TYPE_EM_X270)
#else
# define machine_is_em_x270()	(0)
#endif

#ifdef CONFIG_MACH_NEO1973_GTA02
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEO1973_GTA02
# endif
# define machine_is_neo1973_gta02()	(machine_arch_type == MACH_TYPE_NEO1973_GTA02)
#else
# define machine_is_neo1973_gta02()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9RLEK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9RLEK
# endif
# define machine_is_at91sam9rlek()	(machine_arch_type == MACH_TYPE_AT91SAM9RLEK)
#else
# define machine_is_at91sam9rlek()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI320
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI320
# endif
# define machine_is_colibri320()	(machine_arch_type == MACH_TYPE_COLIBRI320)
#else
# define machine_is_colibri320()	(0)
#endif

#ifdef CONFIG_MACH_CAM60
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CAM60
# endif
# define machine_is_cam60()	(machine_arch_type == MACH_TYPE_CAM60)
#else
# define machine_is_cam60()	(0)
#endif

#ifdef CONFIG_MACH_AT91EB01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91EB01
# endif
# define machine_is_at91eb01()	(machine_arch_type == MACH_TYPE_AT91EB01)
#else
# define machine_is_at91eb01()	(0)
#endif

#ifdef CONFIG_MACH_DB88F5281
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DB88F5281
# endif
# define machine_is_db88f5281()	(machine_arch_type == MACH_TYPE_DB88F5281)
#else
# define machine_is_db88f5281()	(0)
#endif

#ifdef CONFIG_MACH_CSB726
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSB726
# endif
# define machine_is_csb726()	(machine_arch_type == MACH_TYPE_CSB726)
#else
# define machine_is_csb726()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM6467_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM6467_EVM
# endif
# define machine_is_davinci_dm6467_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_DM6467_EVM)
#else
# define machine_is_davinci_dm6467_evm()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM355_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM355_EVM
# endif
# define machine_is_davinci_dm355_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_DM355_EVM)
#else
# define machine_is_davinci_dm355_evm()	(0)
#endif

#ifdef CONFIG_MACH_LITTLETON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LITTLETON
# endif
# define machine_is_littleton()	(machine_arch_type == MACH_TYPE_LITTLETON)
#else
# define machine_is_littleton()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_PB11MP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_PB11MP
# endif
# define machine_is_realview_pb11mp()	(machine_arch_type == MACH_TYPE_REALVIEW_PB11MP)
#else
# define machine_is_realview_pb11mp()	(0)
#endif

#ifdef CONFIG_MACH_MX27_3DS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX27_3DS
# endif
# define machine_is_mx27_3ds()	(machine_arch_type == MACH_TYPE_MX27_3DS)
#else
# define machine_is_mx27_3ds()	(0)
#endif

#ifdef CONFIG_MACH_HALIBUT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HALIBUT
# endif
# define machine_is_halibut()	(machine_arch_type == MACH_TYPE_HALIBUT)
#else
# define machine_is_halibut()	(0)
#endif

#ifdef CONFIG_MACH_TROUT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TROUT
# endif
# define machine_is_trout()	(machine_arch_type == MACH_TYPE_TROUT)
#else
# define machine_is_trout()	(0)
#endif

#ifdef CONFIG_MACH_TCT_HAMMER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TCT_HAMMER
# endif
# define machine_is_tct_hammer()	(machine_arch_type == MACH_TYPE_TCT_HAMMER)
#else
# define machine_is_tct_hammer()	(0)
#endif

#ifdef CONFIG_MACH_HERALD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HERALD
# endif
# define machine_is_herald()	(machine_arch_type == MACH_TYPE_HERALD)
#else
# define machine_is_herald()	(0)
#endif

#ifdef CONFIG_MACH_SIM_ONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIM_ONE
# endif
# define machine_is_sim_one()	(machine_arch_type == MACH_TYPE_SIM_ONE)
#else
# define machine_is_sim_one()	(0)
#endif

#ifdef CONFIG_MACH_JIVE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JIVE
# endif
# define machine_is_jive()	(machine_arch_type == MACH_TYPE_JIVE)
#else
# define machine_is_jive()	(0)
#endif

#ifdef CONFIG_MACH_SAM9_L9260
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAM9_L9260
# endif
# define machine_is_sam9_l9260()	(machine_arch_type == MACH_TYPE_SAM9_L9260)
#else
# define machine_is_sam9_l9260()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_PB1176
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_PB1176
# endif
# define machine_is_realview_pb1176()	(machine_arch_type == MACH_TYPE_REALVIEW_PB1176)
#else
# define machine_is_realview_pb1176()	(0)
#endif

#ifdef CONFIG_MACH_YL9200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_YL9200
# endif
# define machine_is_yl9200()	(machine_arch_type == MACH_TYPE_YL9200)
#else
# define machine_is_yl9200()	(0)
#endif

#ifdef CONFIG_MACH_RD88F5182
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F5182
# endif
# define machine_is_rd88f5182()	(machine_arch_type == MACH_TYPE_RD88F5182)
#else
# define machine_is_rd88f5182()	(0)
#endif

#ifdef CONFIG_MACH_KUROBOX_PRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KUROBOX_PRO
# endif
# define machine_is_kurobox_pro()	(machine_arch_type == MACH_TYPE_KUROBOX_PRO)
#else
# define machine_is_kurobox_pro()	(0)
#endif

#ifdef CONFIG_MACH_MX31_3DS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX31_3DS
# endif
# define machine_is_mx31_3ds()	(machine_arch_type == MACH_TYPE_MX31_3DS)
#else
# define machine_is_mx31_3ds()	(0)
#endif

#ifdef CONFIG_MACH_QONG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QONG
# endif
# define machine_is_qong()	(machine_arch_type == MACH_TYPE_QONG)
#else
# define machine_is_qong()	(0)
#endif

#ifdef CONFIG_MACH_OMAP2EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP2EVM
# endif
# define machine_is_omap2evm()	(machine_arch_type == MACH_TYPE_OMAP2EVM)
#else
# define machine_is_omap2evm()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3EVM
# endif
# define machine_is_omap3evm()	(machine_arch_type == MACH_TYPE_OMAP3EVM)
#else
# define machine_is_omap3evm()	(0)
#endif

#ifdef CONFIG_MACH_DNS323
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DNS323
# endif
# define machine_is_dns323()	(machine_arch_type == MACH_TYPE_DNS323)
#else
# define machine_is_dns323()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_BEAGLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_BEAGLE
# endif
# define machine_is_omap3_beagle()	(machine_arch_type == MACH_TYPE_OMAP3_BEAGLE)
#else
# define machine_is_omap3_beagle()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA_N810
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA_N810
# endif
# define machine_is_nokia_n810()	(machine_arch_type == MACH_TYPE_NOKIA_N810)
#else
# define machine_is_nokia_n810()	(0)
#endif

#ifdef CONFIG_MACH_PCM038
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM038
# endif
# define machine_is_pcm038()	(machine_arch_type == MACH_TYPE_PCM038)
#else
# define machine_is_pcm038()	(0)
#endif

#ifdef CONFIG_MACH_TS209
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS209
# endif
# define machine_is_ts_x09()	(machine_arch_type == MACH_TYPE_TS209)
#else
# define machine_is_ts_x09()	(0)
#endif

#ifdef CONFIG_MACH_AT91CAP9ADK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91CAP9ADK
# endif
# define machine_is_at91cap9adk()	(machine_arch_type == MACH_TYPE_AT91CAP9ADK)
#else
# define machine_is_at91cap9adk()	(0)
#endif

#ifdef CONFIG_MACH_MX31MOBOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX31MOBOARD
# endif
# define machine_is_mx31moboard()	(machine_arch_type == MACH_TYPE_MX31MOBOARD)
#else
# define machine_is_mx31moboard()	(0)
#endif

#ifdef CONFIG_MACH_TERASTATION_PRO2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TERASTATION_PRO2
# endif
# define machine_is_terastation_pro2()	(machine_arch_type == MACH_TYPE_TERASTATION_PRO2)
#else
# define machine_is_terastation_pro2()	(0)
#endif

#ifdef CONFIG_MACH_LINKSTATION_PRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINKSTATION_PRO
# endif
# define machine_is_linkstation_pro()	(machine_arch_type == MACH_TYPE_LINKSTATION_PRO)
#else
# define machine_is_linkstation_pro()	(0)
#endif

#ifdef CONFIG_MACH_E350
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E350
# endif
# define machine_is_e350()	(machine_arch_type == MACH_TYPE_E350)
#else
# define machine_is_e350()	(0)
#endif

#ifdef CONFIG_MACH_TS409
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS409
# endif
# define machine_is_ts409()	(machine_arch_type == MACH_TYPE_TS409)
#else
# define machine_is_ts409()	(0)
#endif

#ifdef CONFIG_MACH_CM_X300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_X300
# endif
# define machine_is_cm_x300()	(machine_arch_type == MACH_TYPE_CM_X300)
#else
# define machine_is_cm_x300()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9G20EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9G20EK
# endif
# define machine_is_at91sam9g20ek()	(machine_arch_type == MACH_TYPE_AT91SAM9G20EK)
#else
# define machine_is_at91sam9g20ek()	(0)
#endif

#ifdef CONFIG_MACH_SMDK6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK6410
# endif
# define machine_is_smdk6410()	(machine_arch_type == MACH_TYPE_SMDK6410)
#else
# define machine_is_smdk6410()	(0)
#endif

#ifdef CONFIG_MACH_U300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_U300
# endif
# define machine_is_u300()	(machine_arch_type == MACH_TYPE_U300)
#else
# define machine_is_u300()	(0)
#endif

#ifdef CONFIG_MACH_WRT350N_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WRT350N_V2
# endif
# define machine_is_wrt350n_v2()	(machine_arch_type == MACH_TYPE_WRT350N_V2)
#else
# define machine_is_wrt350n_v2()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_LDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_LDP
# endif
# define machine_is_omap_ldp()	(machine_arch_type == MACH_TYPE_OMAP_LDP)
#else
# define machine_is_omap_ldp()	(0)
#endif

#ifdef CONFIG_MACH_MX35_3DS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX35_3DS
# endif
# define machine_is_mx35_3ds()	(machine_arch_type == MACH_TYPE_MX35_3DS)
#else
# define machine_is_mx35_3ds()	(0)
#endif

#ifdef CONFIG_MACH_NEUROS_OSD2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEUROS_OSD2
# endif
# define machine_is_neuros_osd2()	(machine_arch_type == MACH_TYPE_NEUROS_OSD2)
#else
# define machine_is_neuros_osd2()	(0)
#endif

#ifdef CONFIG_MACH_TRIZEPS4WL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIZEPS4WL
# endif
# define machine_is_trizeps4wl()	(machine_arch_type == MACH_TYPE_TRIZEPS4WL)
#else
# define machine_is_trizeps4wl()	(0)
#endif

#ifdef CONFIG_MACH_TS78XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS78XX
# endif
# define machine_is_ts78xx()	(machine_arch_type == MACH_TYPE_TS78XX)
#else
# define machine_is_ts78xx()	(0)
#endif

#ifdef CONFIG_MACH_SFFSDR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SFFSDR
# endif
# define machine_is_sffsdr()	(machine_arch_type == MACH_TYPE_SFFSDR)
#else
# define machine_is_sffsdr()	(0)
#endif

#ifdef CONFIG_MACH_PCM037
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM037
# endif
# define machine_is_pcm037()	(machine_arch_type == MACH_TYPE_PCM037)
#else
# define machine_is_pcm037()	(0)
#endif

#ifdef CONFIG_MACH_DB88F6281_BP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DB88F6281_BP
# endif
# define machine_is_db88f6281_bp()	(machine_arch_type == MACH_TYPE_DB88F6281_BP)
#else
# define machine_is_db88f6281_bp()	(0)
#endif

#ifdef CONFIG_MACH_RD88F6192_NAS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F6192_NAS
# endif
# define machine_is_rd88f6192_nas()	(machine_arch_type == MACH_TYPE_RD88F6192_NAS)
#else
# define machine_is_rd88f6192_nas()	(0)
#endif

#ifdef CONFIG_MACH_RD88F6281
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F6281
# endif
# define machine_is_rd88f6281()	(machine_arch_type == MACH_TYPE_RD88F6281)
#else
# define machine_is_rd88f6281()	(0)
#endif

#ifdef CONFIG_MACH_DB78X00_BP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DB78X00_BP
# endif
# define machine_is_db78x00_bp()	(machine_arch_type == MACH_TYPE_DB78X00_BP)
#else
# define machine_is_db78x00_bp()	(0)
#endif

#ifdef CONFIG_MACH_SMDK2416
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK2416
# endif
# define machine_is_smdk2416()	(machine_arch_type == MACH_TYPE_SMDK2416)
#else
# define machine_is_smdk2416()	(0)
#endif

#ifdef CONFIG_MACH_WBD111
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WBD111
# endif
# define machine_is_wbd111()	(machine_arch_type == MACH_TYPE_WBD111)
#else
# define machine_is_wbd111()	(0)
#endif

#ifdef CONFIG_MACH_MV2120
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MV2120
# endif
# define machine_is_mv2120()	(machine_arch_type == MACH_TYPE_MV2120)
#else
# define machine_is_mv2120()	(0)
#endif

#ifdef CONFIG_MACH_MX51_3DS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_3DS
# endif
# define machine_is_mx51_3ds()	(machine_arch_type == MACH_TYPE_MX51_3DS)
#else
# define machine_is_mx51_3ds()	(0)
#endif

#ifdef CONFIG_MACH_IMX27LITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IMX27LITE
# endif
# define machine_is_imx27lite()	(machine_arch_type == MACH_TYPE_IMX27LITE)
#else
# define machine_is_imx27lite()	(0)
#endif

#ifdef CONFIG_MACH_USB_A9260
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_USB_A9260
# endif
# define machine_is_usb_a9260()	(machine_arch_type == MACH_TYPE_USB_A9260)
#else
# define machine_is_usb_a9260()	(0)
#endif

#ifdef CONFIG_MACH_USB_A9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_USB_A9263
# endif
# define machine_is_usb_a9263()	(machine_arch_type == MACH_TYPE_USB_A9263)
#else
# define machine_is_usb_a9263()	(0)
#endif

#ifdef CONFIG_MACH_QIL_A9260
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QIL_A9260
# endif
# define machine_is_qil_a9260()	(machine_arch_type == MACH_TYPE_QIL_A9260)
#else
# define machine_is_qil_a9260()	(0)
#endif

#ifdef CONFIG_MACH_KZM_ARM11_01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KZM_ARM11_01
# endif
# define machine_is_kzm_arm11_01()	(machine_arch_type == MACH_TYPE_KZM_ARM11_01)
#else
# define machine_is_kzm_arm11_01()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA_N810_WIMAX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA_N810_WIMAX
# endif
# define machine_is_nokia_n810_wimax()	(machine_arch_type == MACH_TYPE_NOKIA_N810_WIMAX)
#else
# define machine_is_nokia_n810_wimax()	(0)
#endif

#ifdef CONFIG_MACH_SAPPHIRE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAPPHIRE
# endif
# define machine_is_sapphire()	(machine_arch_type == MACH_TYPE_SAPPHIRE)
#else
# define machine_is_sapphire()	(0)
#endif

#ifdef CONFIG_MACH_STMP37XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STMP37XX
# endif
# define machine_is_stmp37xx()	(machine_arch_type == MACH_TYPE_STMP37XX)
#else
# define machine_is_stmp37xx()	(0)
#endif

#ifdef CONFIG_MACH_STMP378X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STMP378X
# endif
# define machine_is_stmp378x()	(machine_arch_type == MACH_TYPE_STMP378X)
#else
# define machine_is_stmp378x()	(0)
#endif

#ifdef CONFIG_MACH_EZX_A780
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_A780
# endif
# define machine_is_ezx_a780()	(machine_arch_type == MACH_TYPE_EZX_A780)
#else
# define machine_is_ezx_a780()	(0)
#endif

#ifdef CONFIG_MACH_EZX_E680
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_E680
# endif
# define machine_is_ezx_e680()	(machine_arch_type == MACH_TYPE_EZX_E680)
#else
# define machine_is_ezx_e680()	(0)
#endif

#ifdef CONFIG_MACH_EZX_A1200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_A1200
# endif
# define machine_is_ezx_a1200()	(machine_arch_type == MACH_TYPE_EZX_A1200)
#else
# define machine_is_ezx_a1200()	(0)
#endif

#ifdef CONFIG_MACH_EZX_E6
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_E6
# endif
# define machine_is_ezx_e6()	(machine_arch_type == MACH_TYPE_EZX_E6)
#else
# define machine_is_ezx_e6()	(0)
#endif

#ifdef CONFIG_MACH_EZX_E2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_E2
# endif
# define machine_is_ezx_e2()	(machine_arch_type == MACH_TYPE_EZX_E2)
#else
# define machine_is_ezx_e2()	(0)
#endif

#ifdef CONFIG_MACH_EZX_A910
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EZX_A910
# endif
# define machine_is_ezx_a910()	(machine_arch_type == MACH_TYPE_EZX_A910)
#else
# define machine_is_ezx_a910()	(0)
#endif

#ifdef CONFIG_MACH_EDMINI_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDMINI_V2
# endif
# define machine_is_edmini_v2()	(machine_arch_type == MACH_TYPE_EDMINI_V2)
#else
# define machine_is_edmini_v2()	(0)
#endif

#ifdef CONFIG_MACH_ZIPIT2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZIPIT2
# endif
# define machine_is_zipit2()	(machine_arch_type == MACH_TYPE_ZIPIT2)
#else
# define machine_is_zipit2()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_PANDORA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_PANDORA
# endif
# define machine_is_omap3_pandora()	(machine_arch_type == MACH_TYPE_OMAP3_PANDORA)
#else
# define machine_is_omap3_pandora()	(0)
#endif

#ifdef CONFIG_MACH_MSS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSS2
# endif
# define machine_is_mss2()	(machine_arch_type == MACH_TYPE_MSS2)
#else
# define machine_is_mss2()	(0)
#endif

#ifdef CONFIG_MACH_LB88RC8480
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LB88RC8480
# endif
# define machine_is_lb88rc8480()	(machine_arch_type == MACH_TYPE_LB88RC8480)
#else
# define machine_is_lb88rc8480()	(0)
#endif

#ifdef CONFIG_MACH_MX25_3DS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX25_3DS
# endif
# define machine_is_mx25_3ds()	(machine_arch_type == MACH_TYPE_MX25_3DS)
#else
# define machine_is_mx25_3ds()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3530_LV_SOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3530_LV_SOM
# endif
# define machine_is_omap3530_lv_som()	(machine_arch_type == MACH_TYPE_OMAP3530_LV_SOM)
#else
# define machine_is_omap3530_lv_som()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DA830_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DA830_EVM
# endif
# define machine_is_davinci_da830_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_DA830_EVM)
#else
# define machine_is_davinci_da830_evm()	(0)
#endif

#ifdef CONFIG_MACH_AT572D940HFEB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT572D940HFEB
# endif
# define machine_is_at572d940hfek()	(machine_arch_type == MACH_TYPE_AT572D940HFEB)
#else
# define machine_is_at572d940hfek()	(0)
#endif

#ifdef CONFIG_MACH_DOVE_DB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DOVE_DB
# endif
# define machine_is_dove_db()	(machine_arch_type == MACH_TYPE_DOVE_DB)
#else
# define machine_is_dove_db()	(0)
#endif

#ifdef CONFIG_MACH_OVERO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OVERO
# endif
# define machine_is_overo()	(machine_arch_type == MACH_TYPE_OVERO)
#else
# define machine_is_overo()	(0)
#endif

#ifdef CONFIG_MACH_AT2440EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT2440EVB
# endif
# define machine_is_at2440evb()	(machine_arch_type == MACH_TYPE_AT2440EVB)
#else
# define machine_is_at2440evb()	(0)
#endif

#ifdef CONFIG_MACH_NEOCORE926
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NEOCORE926
# endif
# define machine_is_neocore926()	(machine_arch_type == MACH_TYPE_NEOCORE926)
#else
# define machine_is_neocore926()	(0)
#endif

#ifdef CONFIG_MACH_WNR854T
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WNR854T
# endif
# define machine_is_wnr854t()	(machine_arch_type == MACH_TYPE_WNR854T)
#else
# define machine_is_wnr854t()	(0)
#endif

#ifdef CONFIG_MACH_RD88F5181L_GE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F5181L_GE
# endif
# define machine_is_rd88f5181l_ge()	(machine_arch_type == MACH_TYPE_RD88F5181L_GE)
#else
# define machine_is_rd88f5181l_ge()	(0)
#endif

#ifdef CONFIG_MACH_RD88F5181L_FXO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F5181L_FXO
# endif
# define machine_is_rd88f5181l_fxo()	(machine_arch_type == MACH_TYPE_RD88F5181L_FXO)
#else
# define machine_is_rd88f5181l_fxo()	(0)
#endif

#ifdef CONFIG_MACH_STAMP9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STAMP9G20
# endif
# define machine_is_stamp9g20()	(machine_arch_type == MACH_TYPE_STAMP9G20)
#else
# define machine_is_stamp9g20()	(0)
#endif

#ifdef CONFIG_MACH_SMDKC100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDKC100
# endif
# define machine_is_smdkc100()	(machine_arch_type == MACH_TYPE_SMDKC100)
#else
# define machine_is_smdkc100()	(0)
#endif

#ifdef CONFIG_MACH_TAVOREVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAVOREVB
# endif
# define machine_is_tavorevb()	(machine_arch_type == MACH_TYPE_TAVOREVB)
#else
# define machine_is_tavorevb()	(0)
#endif

#ifdef CONFIG_MACH_SAAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAAR
# endif
# define machine_is_saar()	(machine_arch_type == MACH_TYPE_SAAR)
#else
# define machine_is_saar()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9M10G45EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9M10G45EK
# endif
# define machine_is_at91sam9m10g45ek()	(machine_arch_type == MACH_TYPE_AT91SAM9M10G45EK)
#else
# define machine_is_at91sam9m10g45ek()	(0)
#endif

#ifdef CONFIG_MACH_MXLADS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXLADS
# endif
# define machine_is_mxlads()	(machine_arch_type == MACH_TYPE_MXLADS)
#else
# define machine_is_mxlads()	(0)
#endif

#ifdef CONFIG_MACH_LINKSTATION_MINI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINKSTATION_MINI
# endif
# define machine_is_linkstation_mini()	(machine_arch_type == MACH_TYPE_LINKSTATION_MINI)
#else
# define machine_is_linkstation_mini()	(0)
#endif

#ifdef CONFIG_MACH_AFEB9260
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AFEB9260
# endif
# define machine_is_afeb9260()	(machine_arch_type == MACH_TYPE_AFEB9260)
#else
# define machine_is_afeb9260()	(0)
#endif

#ifdef CONFIG_MACH_IMX27IPCAM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IMX27IPCAM
# endif
# define machine_is_imx27ipcam()	(machine_arch_type == MACH_TYPE_IMX27IPCAM)
#else
# define machine_is_imx27ipcam()	(0)
#endif

#ifdef CONFIG_MACH_RD88F6183AP_GE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD88F6183AP_GE
# endif
# define machine_is_rd88f6183ap_ge()	(machine_arch_type == MACH_TYPE_RD88F6183AP_GE)
#else
# define machine_is_rd88f6183ap_ge()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_PBA8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_PBA8
# endif
# define machine_is_realview_pba8()	(machine_arch_type == MACH_TYPE_REALVIEW_PBA8)
#else
# define machine_is_realview_pba8()	(0)
#endif

#ifdef CONFIG_MACH_REALVIEW_PBX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REALVIEW_PBX
# endif
# define machine_is_realview_pbx()	(machine_arch_type == MACH_TYPE_REALVIEW_PBX)
#else
# define machine_is_realview_pbx()	(0)
#endif

#ifdef CONFIG_MACH_MICRO9S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICRO9S
# endif
# define machine_is_micro9s()	(machine_arch_type == MACH_TYPE_MICRO9S)
#else
# define machine_is_micro9s()	(0)
#endif

#ifdef CONFIG_MACH_RUT100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RUT100
# endif
# define machine_is_rut100()	(machine_arch_type == MACH_TYPE_RUT100)
#else
# define machine_is_rut100()	(0)
#endif

#ifdef CONFIG_MACH_G3EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_G3EVM
# endif
# define machine_is_g3evm()	(machine_arch_type == MACH_TYPE_G3EVM)
#else
# define machine_is_g3evm()	(0)
#endif

#ifdef CONFIG_MACH_W90P910EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_W90P910EVB
# endif
# define machine_is_w90p910evb()	(machine_arch_type == MACH_TYPE_W90P910EVB)
#else
# define machine_is_w90p910evb()	(0)
#endif

#ifdef CONFIG_MACH_W90P950EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_W90P950EVB
# endif
# define machine_is_w90p950evb()	(machine_arch_type == MACH_TYPE_W90P950EVB)
#else
# define machine_is_w90p950evb()	(0)
#endif

#ifdef CONFIG_MACH_W90N960EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_W90N960EVB
# endif
# define machine_is_w90n960evb()	(machine_arch_type == MACH_TYPE_W90N960EVB)
#else
# define machine_is_w90n960evb()	(0)
#endif

#ifdef CONFIG_MACH_MV88F6281GTW_GE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MV88F6281GTW_GE
# endif
# define machine_is_mv88f6281gtw_ge()	(machine_arch_type == MACH_TYPE_MV88F6281GTW_GE)
#else
# define machine_is_mv88f6281gtw_ge()	(0)
#endif

#ifdef CONFIG_MACH_NCP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NCP
# endif
# define machine_is_ncp()	(machine_arch_type == MACH_TYPE_NCP)
#else
# define machine_is_ncp()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM365_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM365_EVM
# endif
# define machine_is_davinci_dm365_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_DM365_EVM)
#else
# define machine_is_davinci_dm365_evm()	(0)
#endif

#ifdef CONFIG_MACH_CENTRO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CENTRO
# endif
# define machine_is_centro()	(machine_arch_type == MACH_TYPE_CENTRO)
#else
# define machine_is_centro()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA_RX51
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA_RX51
# endif
# define machine_is_nokia_rx51()	(machine_arch_type == MACH_TYPE_NOKIA_RX51)
#else
# define machine_is_nokia_rx51()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_ZOOM2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_ZOOM2
# endif
# define machine_is_omap_zoom2()	(machine_arch_type == MACH_TYPE_OMAP_ZOOM2)
#else
# define machine_is_omap_zoom2()	(0)
#endif

#ifdef CONFIG_MACH_ACS5K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACS5K
# endif
# define machine_is_acs5k()	(machine_arch_type == MACH_TYPE_ACS5K)
#else
# define machine_is_acs5k()	(0)
#endif

#ifdef CONFIG_MACH_SNAPPER_9260
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SNAPPER_9260
# endif
# define machine_is_snapper_9260()	(machine_arch_type == MACH_TYPE_SNAPPER_9260)
#else
# define machine_is_snapper_9260()	(0)
#endif

#ifdef CONFIG_MACH_DSM320
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DSM320
# endif
# define machine_is_dsm320()	(machine_arch_type == MACH_TYPE_DSM320)
#else
# define machine_is_dsm320()	(0)
#endif

#ifdef CONFIG_MACH_EXEDA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXEDA
# endif
# define machine_is_exeda()	(machine_arch_type == MACH_TYPE_EXEDA)
#else
# define machine_is_exeda()	(0)
#endif

#ifdef CONFIG_MACH_MINI2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MINI2440
# endif
# define machine_is_mini2440()	(machine_arch_type == MACH_TYPE_MINI2440)
#else
# define machine_is_mini2440()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI300
# endif
# define machine_is_colibri300()	(machine_arch_type == MACH_TYPE_COLIBRI300)
#else
# define machine_is_colibri300()	(0)
#endif

#ifdef CONFIG_MACH_LINKSTATION_LS_HGL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINKSTATION_LS_HGL
# endif
# define machine_is_linkstation_ls_hgl()	(machine_arch_type == MACH_TYPE_LINKSTATION_LS_HGL)
#else
# define machine_is_linkstation_ls_hgl()	(0)
#endif

#ifdef CONFIG_MACH_CPUAT9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPUAT9G20
# endif
# define machine_is_cpuat9g20()	(machine_arch_type == MACH_TYPE_CPUAT9G20)
#else
# define machine_is_cpuat9g20()	(0)
#endif

#ifdef CONFIG_MACH_SMDK6440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK6440
# endif
# define machine_is_smdk6440()	(machine_arch_type == MACH_TYPE_SMDK6440)
#else
# define machine_is_smdk6440()	(0)
#endif

#ifdef CONFIG_MACH_NAS4220B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAS4220B
# endif
# define machine_is_nas4220b()	(machine_arch_type == MACH_TYPE_NAS4220B)
#else
# define machine_is_nas4220b()	(0)
#endif

#ifdef CONFIG_MACH_ZYLONITE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZYLONITE2
# endif
# define machine_is_zylonite2()	(machine_arch_type == MACH_TYPE_ZYLONITE2)
#else
# define machine_is_zylonite2()	(0)
#endif

#ifdef CONFIG_MACH_ASPENITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASPENITE
# endif
# define machine_is_aspenite()	(machine_arch_type == MACH_TYPE_ASPENITE)
#else
# define machine_is_aspenite()	(0)
#endif

#ifdef CONFIG_MACH_TTC_DKB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TTC_DKB
# endif
# define machine_is_ttc_dkb()	(machine_arch_type == MACH_TYPE_TTC_DKB)
#else
# define machine_is_ttc_dkb()	(0)
#endif

#ifdef CONFIG_MACH_PCM043
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM043
# endif
# define machine_is_pcm043()	(machine_arch_type == MACH_TYPE_PCM043)
#else
# define machine_is_pcm043()	(0)
#endif

#ifdef CONFIG_MACH_SHEEVAPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHEEVAPLUG
# endif
# define machine_is_sheevaplug()	(machine_arch_type == MACH_TYPE_SHEEVAPLUG)
#else
# define machine_is_sheevaplug()	(0)
#endif

#ifdef CONFIG_MACH_AVENGERS_LITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AVENGERS_LITE
# endif
# define machine_is_avengers_lite()	(machine_arch_type == MACH_TYPE_AVENGERS_LITE)
#else
# define machine_is_avengers_lite()	(0)
#endif

#ifdef CONFIG_MACH_MX51_BABBAGE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_BABBAGE
# endif
# define machine_is_mx51_babbage()	(machine_arch_type == MACH_TYPE_MX51_BABBAGE)
#else
# define machine_is_mx51_babbage()	(0)
#endif

#ifdef CONFIG_MACH_RD78X00_MASA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RD78X00_MASA
# endif
# define machine_is_rd78x00_masa()	(machine_arch_type == MACH_TYPE_RD78X00_MASA)
#else
# define machine_is_rd78x00_masa()	(0)
#endif

#ifdef CONFIG_MACH_DM355_LEOPARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM355_LEOPARD
# endif
# define machine_is_dm355_leopard()	(machine_arch_type == MACH_TYPE_DM355_LEOPARD)
#else
# define machine_is_dm355_leopard()	(0)
#endif

#ifdef CONFIG_MACH_TS219
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS219
# endif
# define machine_is_ts219()	(machine_arch_type == MACH_TYPE_TS219)
#else
# define machine_is_ts219()	(0)
#endif

#ifdef CONFIG_MACH_PCA100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCA100
# endif
# define machine_is_pca100()	(machine_arch_type == MACH_TYPE_PCA100)
#else
# define machine_is_pca100()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DA850_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DA850_EVM
# endif
# define machine_is_davinci_da850_evm()	(machine_arch_type == MACH_TYPE_DAVINCI_DA850_EVM)
#else
# define machine_is_davinci_da850_evm()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9G10EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9G10EK
# endif
# define machine_is_at91sam9g10ek()	(machine_arch_type == MACH_TYPE_AT91SAM9G10EK)
#else
# define machine_is_at91sam9g10ek()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_4430SDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_4430SDP
# endif
# define machine_is_omap_4430sdp()	(machine_arch_type == MACH_TYPE_OMAP_4430SDP)
#else
# define machine_is_omap_4430sdp()	(0)
#endif

#ifdef CONFIG_MACH_MAGX_ZN5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAGX_ZN5
# endif
# define machine_is_magx_zn5()	(machine_arch_type == MACH_TYPE_MAGX_ZN5)
#else
# define machine_is_magx_zn5()	(0)
#endif

#ifdef CONFIG_MACH_BTMAVB101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BTMAVB101
# endif
# define machine_is_btmavb101()	(machine_arch_type == MACH_TYPE_BTMAVB101)
#else
# define machine_is_btmavb101()	(0)
#endif

#ifdef CONFIG_MACH_BTMAWB101
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BTMAWB101
# endif
# define machine_is_btmawb101()	(machine_arch_type == MACH_TYPE_BTMAWB101)
#else
# define machine_is_btmawb101()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_TORPEDO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_TORPEDO
# endif
# define machine_is_omap3_torpedo()	(machine_arch_type == MACH_TYPE_OMAP3_TORPEDO)
#else
# define machine_is_omap3_torpedo()	(0)
#endif

#ifdef CONFIG_MACH_ANW6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANW6410
# endif
# define machine_is_anw6410()	(machine_arch_type == MACH_TYPE_ANW6410)
#else
# define machine_is_anw6410()	(0)
#endif

#ifdef CONFIG_MACH_IMX27_VISSTRIM_M10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IMX27_VISSTRIM_M10
# endif
# define machine_is_imx27_visstrim_m10()	(machine_arch_type == MACH_TYPE_IMX27_VISSTRIM_M10)
#else
# define machine_is_imx27_visstrim_m10()	(0)
#endif

#ifdef CONFIG_MACH_PORTUXG20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PORTUXG20
# endif
# define machine_is_portuxg20()	(machine_arch_type == MACH_TYPE_PORTUXG20)
#else
# define machine_is_portuxg20()	(0)
#endif

#ifdef CONFIG_MACH_SMDKC110
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDKC110
# endif
# define machine_is_smdkc110()	(machine_arch_type == MACH_TYPE_SMDKC110)
#else
# define machine_is_smdkc110()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3517EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3517EVM
# endif
# define machine_is_omap3517evm()	(machine_arch_type == MACH_TYPE_OMAP3517EVM)
#else
# define machine_is_omap3517evm()	(0)
#endif

#ifdef CONFIG_MACH_NETSPACE_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETSPACE_V2
# endif
# define machine_is_netspace_v2()	(machine_arch_type == MACH_TYPE_NETSPACE_V2)
#else
# define machine_is_netspace_v2()	(0)
#endif

#ifdef CONFIG_MACH_NETSPACE_MAX_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETSPACE_MAX_V2
# endif
# define machine_is_netspace_max_v2()	(machine_arch_type == MACH_TYPE_NETSPACE_MAX_V2)
#else
# define machine_is_netspace_max_v2()	(0)
#endif

#ifdef CONFIG_MACH_D2NET_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_D2NET_V2
# endif
# define machine_is_d2net_v2()	(machine_arch_type == MACH_TYPE_D2NET_V2)
#else
# define machine_is_d2net_v2()	(0)
#endif

#ifdef CONFIG_MACH_NET2BIG_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NET2BIG_V2
# endif
# define machine_is_net2big_v2()	(machine_arch_type == MACH_TYPE_NET2BIG_V2)
#else
# define machine_is_net2big_v2()	(0)
#endif

#ifdef CONFIG_MACH_NET5BIG_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NET5BIG_V2
# endif
# define machine_is_net5big_v2()	(machine_arch_type == MACH_TYPE_NET5BIG_V2)
#else
# define machine_is_net5big_v2()	(0)
#endif

#ifdef CONFIG_MACH_INETSPACE_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INETSPACE_V2
# endif
# define machine_is_inetspace_v2()	(machine_arch_type == MACH_TYPE_INETSPACE_V2)
#else
# define machine_is_inetspace_v2()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9G45EKES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9G45EKES
# endif
# define machine_is_at91sam9g45ekes()	(machine_arch_type == MACH_TYPE_AT91SAM9G45EKES)
#else
# define machine_is_at91sam9g45ekes()	(0)
#endif

#ifdef CONFIG_MACH_PC7302
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PC7302
# endif
# define machine_is_pc7302()	(machine_arch_type == MACH_TYPE_PC7302)
#else
# define machine_is_pc7302()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR600
# endif
# define machine_is_spear600()	(machine_arch_type == MACH_TYPE_SPEAR600)
#else
# define machine_is_spear600()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR300
# endif
# define machine_is_spear300()	(machine_arch_type == MACH_TYPE_SPEAR300)
#else
# define machine_is_spear300()	(0)
#endif

#ifdef CONFIG_MACH_LILLY1131
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LILLY1131
# endif
# define machine_is_lilly1131()	(machine_arch_type == MACH_TYPE_LILLY1131)
#else
# define machine_is_lilly1131()	(0)
#endif

#ifdef CONFIG_MACH_HMT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HMT
# endif
# define machine_is_hmt()	(machine_arch_type == MACH_TYPE_HMT)
#else
# define machine_is_hmt()	(0)
#endif

#ifdef CONFIG_MACH_VEXPRESS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VEXPRESS
# endif
# define machine_is_vexpress()	(machine_arch_type == MACH_TYPE_VEXPRESS)
#else
# define machine_is_vexpress()	(0)
#endif

#ifdef CONFIG_MACH_D2NET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_D2NET
# endif
# define machine_is_d2net()	(machine_arch_type == MACH_TYPE_D2NET)
#else
# define machine_is_d2net()	(0)
#endif

#ifdef CONFIG_MACH_BIGDISK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BIGDISK
# endif
# define machine_is_bigdisk()	(machine_arch_type == MACH_TYPE_BIGDISK)
#else
# define machine_is_bigdisk()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9G20EK_2MMC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9G20EK_2MMC
# endif
# define machine_is_at91sam9g20ek_2mmc()	(machine_arch_type == MACH_TYPE_AT91SAM9G20EK_2MMC)
#else
# define machine_is_at91sam9g20ek_2mmc()	(0)
#endif

#ifdef CONFIG_MACH_BCMRING
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCMRING
# endif
# define machine_is_bcmring()	(machine_arch_type == MACH_TYPE_BCMRING)
#else
# define machine_is_bcmring()	(0)
#endif

#ifdef CONFIG_MACH_DP6XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DP6XX
# endif
# define machine_is_dp6xx()	(machine_arch_type == MACH_TYPE_DP6XX)
#else
# define machine_is_dp6xx()	(0)
#endif

#ifdef CONFIG_MACH_MAHIMAHI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAHIMAHI
# endif
# define machine_is_mahimahi()	(machine_arch_type == MACH_TYPE_MAHIMAHI)
#else
# define machine_is_mahimahi()	(0)
#endif

#ifdef CONFIG_MACH_SMDK6442
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK6442
# endif
# define machine_is_smdk6442()	(machine_arch_type == MACH_TYPE_SMDK6442)
#else
# define machine_is_smdk6442()	(0)
#endif

#ifdef CONFIG_MACH_OPENRD_BASE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPENRD_BASE
# endif
# define machine_is_openrd_base()	(machine_arch_type == MACH_TYPE_OPENRD_BASE)
#else
# define machine_is_openrd_base()	(0)
#endif

#ifdef CONFIG_MACH_DEVKIT8000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEVKIT8000
# endif
# define machine_is_devkit8000()	(machine_arch_type == MACH_TYPE_DEVKIT8000)
#else
# define machine_is_devkit8000()	(0)
#endif

#ifdef CONFIG_MACH_MX51_EFIKAMX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_EFIKAMX
# endif
# define machine_is_mx51_efikamx()	(machine_arch_type == MACH_TYPE_MX51_EFIKAMX)
#else
# define machine_is_mx51_efikamx()	(0)
#endif

#ifdef CONFIG_MACH_CM_T35
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_T35
# endif
# define machine_is_cm_t35()	(machine_arch_type == MACH_TYPE_CM_T35)
#else
# define machine_is_cm_t35()	(0)
#endif

#ifdef CONFIG_MACH_NET2BIG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NET2BIG
# endif
# define machine_is_net2big()	(machine_arch_type == MACH_TYPE_NET2BIG)
#else
# define machine_is_net2big()	(0)
#endif

#ifdef CONFIG_MACH_IGEP0020
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IGEP0020
# endif
# define machine_is_igep0020()	(machine_arch_type == MACH_TYPE_IGEP0020)
#else
# define machine_is_igep0020()	(0)
#endif

#ifdef CONFIG_MACH_NUC932EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC932EVB
# endif
# define machine_is_nuc932evb()	(machine_arch_type == MACH_TYPE_NUC932EVB)
#else
# define machine_is_nuc932evb()	(0)
#endif

#ifdef CONFIG_MACH_OPENRD_CLIENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPENRD_CLIENT
# endif
# define machine_is_openrd_client()	(machine_arch_type == MACH_TYPE_OPENRD_CLIENT)
#else
# define machine_is_openrd_client()	(0)
#endif

#ifdef CONFIG_MACH_MX51_EFIKASB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_EFIKASB
# endif
# define machine_is_mx51_efikasb()	(machine_arch_type == MACH_TYPE_MX51_EFIKASB)
#else
# define machine_is_mx51_efikasb()	(0)
#endif

#ifdef CONFIG_MACH_MARVELL_JASPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARVELL_JASPER
# endif
# define machine_is_marvell_jasper()	(machine_arch_type == MACH_TYPE_MARVELL_JASPER)
#else
# define machine_is_marvell_jasper()	(0)
#endif

#ifdef CONFIG_MACH_FLINT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLINT
# endif
# define machine_is_flint()	(machine_arch_type == MACH_TYPE_FLINT)
#else
# define machine_is_flint()	(0)
#endif

#ifdef CONFIG_MACH_TAVOREVB3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAVOREVB3
# endif
# define machine_is_tavorevb3()	(machine_arch_type == MACH_TYPE_TAVOREVB3)
#else
# define machine_is_tavorevb3()	(0)
#endif

#ifdef CONFIG_MACH_TOUCHBOOK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOUCHBOOK
# endif
# define machine_is_touchbook()	(machine_arch_type == MACH_TYPE_TOUCHBOOK)
#else
# define machine_is_touchbook()	(0)
#endif

#ifdef CONFIG_MACH_RAUMFELD_RC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RAUMFELD_RC
# endif
# define machine_is_raumfeld_rc()	(machine_arch_type == MACH_TYPE_RAUMFELD_RC)
#else
# define machine_is_raumfeld_rc()	(0)
#endif

#ifdef CONFIG_MACH_RAUMFELD_CONNECTOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RAUMFELD_CONNECTOR
# endif
# define machine_is_raumfeld_connector()	(machine_arch_type == MACH_TYPE_RAUMFELD_CONNECTOR)
#else
# define machine_is_raumfeld_connector()	(0)
#endif

#ifdef CONFIG_MACH_RAUMFELD_SPEAKER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RAUMFELD_SPEAKER
# endif
# define machine_is_raumfeld_speaker()	(machine_arch_type == MACH_TYPE_RAUMFELD_SPEAKER)
#else
# define machine_is_raumfeld_speaker()	(0)
#endif

#ifdef CONFIG_MACH_TNETV107X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TNETV107X
# endif
# define machine_is_tnetv107x()	(machine_arch_type == MACH_TYPE_TNETV107X)
#else
# define machine_is_tnetv107x()	(0)
#endif

#ifdef CONFIG_MACH_SMDKV210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDKV210
# endif
# define machine_is_smdkv210()	(machine_arch_type == MACH_TYPE_SMDKV210)
#else
# define machine_is_smdkv210()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_ZOOM3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_ZOOM3
# endif
# define machine_is_omap_zoom3()	(machine_arch_type == MACH_TYPE_OMAP_ZOOM3)
#else
# define machine_is_omap_zoom3()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_3630SDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_3630SDP
# endif
# define machine_is_omap_3630sdp()	(machine_arch_type == MACH_TYPE_OMAP_3630SDP)
#else
# define machine_is_omap_3630sdp()	(0)
#endif

#ifdef CONFIG_MACH_SMARTQ7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMARTQ7
# endif
# define machine_is_smartq7()	(machine_arch_type == MACH_TYPE_SMARTQ7)
#else
# define machine_is_smartq7()	(0)
#endif

#ifdef CONFIG_MACH_WATSON_EFM_PLUGIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WATSON_EFM_PLUGIN
# endif
# define machine_is_watson_efm_plugin()	(machine_arch_type == MACH_TYPE_WATSON_EFM_PLUGIN)
#else
# define machine_is_watson_efm_plugin()	(0)
#endif

#ifdef CONFIG_MACH_G4EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_G4EVM
# endif
# define machine_is_g4evm()	(machine_arch_type == MACH_TYPE_G4EVM)
#else
# define machine_is_g4evm()	(0)
#endif

#ifdef CONFIG_MACH_OMAPL138_HAWKBOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAPL138_HAWKBOARD
# endif
# define machine_is_omapl138_hawkboard()	(machine_arch_type == MACH_TYPE_OMAPL138_HAWKBOARD)
#else
# define machine_is_omapl138_hawkboard()	(0)
#endif

#ifdef CONFIG_MACH_TS41X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS41X
# endif
# define machine_is_ts41x()	(machine_arch_type == MACH_TYPE_TS41X)
#else
# define machine_is_ts41x()	(0)
#endif

#ifdef CONFIG_MACH_PHY3250
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PHY3250
# endif
# define machine_is_phy3250()	(machine_arch_type == MACH_TYPE_PHY3250)
#else
# define machine_is_phy3250()	(0)
#endif

#ifdef CONFIG_MACH_MINI6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MINI6410
# endif
# define machine_is_mini6410()	(machine_arch_type == MACH_TYPE_MINI6410)
#else
# define machine_is_mini6410()	(0)
#endif

#ifdef CONFIG_MACH_MX28EVK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX28EVK
# endif
# define machine_is_mx28evk()	(machine_arch_type == MACH_TYPE_MX28EVK)
#else
# define machine_is_mx28evk()	(0)
#endif

#ifdef CONFIG_MACH_SMARTQ5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMARTQ5
# endif
# define machine_is_smartq5()	(machine_arch_type == MACH_TYPE_SMARTQ5)
#else
# define machine_is_smartq5()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM6467TEVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM6467TEVM
# endif
# define machine_is_davinci_dm6467tevm()	(machine_arch_type == MACH_TYPE_DAVINCI_DM6467TEVM)
#else
# define machine_is_davinci_dm6467tevm()	(0)
#endif

#ifdef CONFIG_MACH_MXT_TD60
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXT_TD60
# endif
# define machine_is_mxt_td60()	(machine_arch_type == MACH_TYPE_MXT_TD60)
#else
# define machine_is_mxt_td60()	(0)
#endif

#ifdef CONFIG_MACH_RIOT_BEI2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RIOT_BEI2
# endif
# define machine_is_riot_bei2()	(machine_arch_type == MACH_TYPE_RIOT_BEI2)
#else
# define machine_is_riot_bei2()	(0)
#endif

#ifdef CONFIG_MACH_RIOT_X37
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RIOT_X37
# endif
# define machine_is_riot_x37()	(machine_arch_type == MACH_TYPE_RIOT_X37)
#else
# define machine_is_riot_x37()	(0)
#endif

#ifdef CONFIG_MACH_CAPC7117
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CAPC7117
# endif
# define machine_is_capc7117()	(machine_arch_type == MACH_TYPE_CAPC7117)
#else
# define machine_is_capc7117()	(0)
#endif

#ifdef CONFIG_MACH_ICONTROL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICONTROL
# endif
# define machine_is_icontrol()	(machine_arch_type == MACH_TYPE_ICONTROL)
#else
# define machine_is_icontrol()	(0)
#endif

#ifdef CONFIG_MACH_QSD8X50A_ST1_5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QSD8X50A_ST1_5
# endif
# define machine_is_qsd8x50a_st1_5()	(machine_arch_type == MACH_TYPE_QSD8X50A_ST1_5)
#else
# define machine_is_qsd8x50a_st1_5()	(0)
#endif

#ifdef CONFIG_MACH_MX23EVK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX23EVK
# endif
# define machine_is_mx23evk()	(machine_arch_type == MACH_TYPE_MX23EVK)
#else
# define machine_is_mx23evk()	(0)
#endif

#ifdef CONFIG_MACH_AP4EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AP4EVB
# endif
# define machine_is_ap4evb()	(machine_arch_type == MACH_TYPE_AP4EVB)
#else
# define machine_is_ap4evb()	(0)
#endif

#ifdef CONFIG_MACH_MITYOMAPL138
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MITYOMAPL138
# endif
# define machine_is_mityomapl138()	(machine_arch_type == MACH_TYPE_MITYOMAPL138)
#else
# define machine_is_mityomapl138()	(0)
#endif

#ifdef CONFIG_MACH_GURUPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GURUPLUG
# endif
# define machine_is_guruplug()	(machine_arch_type == MACH_TYPE_GURUPLUG)
#else
# define machine_is_guruplug()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR310
# endif
# define machine_is_spear310()	(machine_arch_type == MACH_TYPE_SPEAR310)
#else
# define machine_is_spear310()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR320
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR320
# endif
# define machine_is_spear320()	(machine_arch_type == MACH_TYPE_SPEAR320)
#else
# define machine_is_spear320()	(0)
#endif

#ifdef CONFIG_MACH_AQUILA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AQUILA
# endif
# define machine_is_aquila()	(machine_arch_type == MACH_TYPE_AQUILA)
#else
# define machine_is_aquila()	(0)
#endif

#ifdef CONFIG_MACH_ESATA_SHEEVAPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESATA_SHEEVAPLUG
# endif
# define machine_is_sheeva_esata()	(machine_arch_type == MACH_TYPE_ESATA_SHEEVAPLUG)
#else
# define machine_is_sheeva_esata()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X30_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X30_SURF
# endif
# define machine_is_msm7x30_surf()	(machine_arch_type == MACH_TYPE_MSM7X30_SURF)
#else
# define machine_is_msm7x30_surf()	(0)
#endif

#ifdef CONFIG_MACH_EA2478DEVKIT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EA2478DEVKIT
# endif
# define machine_is_ea2478devkit()	(machine_arch_type == MACH_TYPE_EA2478DEVKIT)
#else
# define machine_is_ea2478devkit()	(0)
#endif

#ifdef CONFIG_MACH_TERASTATION_WXL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TERASTATION_WXL
# endif
# define machine_is_terastation_wxl()	(machine_arch_type == MACH_TYPE_TERASTATION_WXL)
#else
# define machine_is_terastation_wxl()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X25_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X25_SURF
# endif
# define machine_is_msm7x25_surf()	(machine_arch_type == MACH_TYPE_MSM7X25_SURF)
#else
# define machine_is_msm7x25_surf()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X25_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X25_FFA
# endif
# define machine_is_msm7x25_ffa()	(machine_arch_type == MACH_TYPE_MSM7X25_FFA)
#else
# define machine_is_msm7x25_ffa()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X27_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X27_SURF
# endif
# define machine_is_msm7x27_surf()	(machine_arch_type == MACH_TYPE_MSM7X27_SURF)
#else
# define machine_is_msm7x27_surf()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X27_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X27_FFA
# endif
# define machine_is_msm7x27_ffa()	(machine_arch_type == MACH_TYPE_MSM7X27_FFA)
#else
# define machine_is_msm7x27_ffa()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X30_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X30_FFA
# endif
# define machine_is_msm7x30_ffa()	(machine_arch_type == MACH_TYPE_MSM7X30_FFA)
#else
# define machine_is_msm7x30_ffa()	(0)
#endif

#ifdef CONFIG_MACH_QSD8X50_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QSD8X50_SURF
# endif
# define machine_is_qsd8x50_surf()	(machine_arch_type == MACH_TYPE_QSD8X50_SURF)
#else
# define machine_is_qsd8x50_surf()	(0)
#endif

#ifdef CONFIG_MACH_MX53_EVK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX53_EVK
# endif
# define machine_is_mx53_evk()	(machine_arch_type == MACH_TYPE_MX53_EVK)
#else
# define machine_is_mx53_evk()	(0)
#endif

#ifdef CONFIG_MACH_IGEP0030
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IGEP0030
# endif
# define machine_is_igep0030()	(machine_arch_type == MACH_TYPE_IGEP0030)
#else
# define machine_is_igep0030()	(0)
#endif

#ifdef CONFIG_MACH_SBC3530
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBC3530
# endif
# define machine_is_sbc3530()	(machine_arch_type == MACH_TYPE_SBC3530)
#else
# define machine_is_sbc3530()	(0)
#endif

#ifdef CONFIG_MACH_SAARB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAARB
# endif
# define machine_is_saarb()	(machine_arch_type == MACH_TYPE_SAARB)
#else
# define machine_is_saarb()	(0)
#endif

#ifdef CONFIG_MACH_HARMONY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HARMONY
# endif
# define machine_is_harmony()	(machine_arch_type == MACH_TYPE_HARMONY)
#else
# define machine_is_harmony()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X30_FLUID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X30_FLUID
# endif
# define machine_is_msm7x30_fluid()	(machine_arch_type == MACH_TYPE_MSM7X30_FLUID)
#else
# define machine_is_msm7x30_fluid()	(0)
#endif

#ifdef CONFIG_MACH_CM_T3517
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_T3517
# endif
# define machine_is_cm_t3517()	(machine_arch_type == MACH_TYPE_CM_T3517)
#else
# define machine_is_cm_t3517()	(0)
#endif

#ifdef CONFIG_MACH_WBD222
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WBD222
# endif
# define machine_is_wbd222()	(machine_arch_type == MACH_TYPE_WBD222)
#else
# define machine_is_wbd222()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_SURF
# endif
# define machine_is_msm8x60_surf()	(machine_arch_type == MACH_TYPE_MSM8X60_SURF)
#else
# define machine_is_msm8x60_surf()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_SIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_SIM
# endif
# define machine_is_msm8x60_sim()	(machine_arch_type == MACH_TYPE_MSM8X60_SIM)
#else
# define machine_is_msm8x60_sim()	(0)
#endif

#ifdef CONFIG_MACH_TCC8000_SDK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TCC8000_SDK
# endif
# define machine_is_tcc8000_sdk()	(machine_arch_type == MACH_TYPE_TCC8000_SDK)
#else
# define machine_is_tcc8000_sdk()	(0)
#endif

#ifdef CONFIG_MACH_NANOS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NANOS
# endif
# define machine_is_nanos()	(machine_arch_type == MACH_TYPE_NANOS)
#else
# define machine_is_nanos()	(0)
#endif

#ifdef CONFIG_MACH_STAMP9G45
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STAMP9G45
# endif
# define machine_is_stamp9g45()	(machine_arch_type == MACH_TYPE_STAMP9G45)
#else
# define machine_is_stamp9g45()	(0)
#endif

#ifdef CONFIG_MACH_CNS3420VB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CNS3420VB
# endif
# define machine_is_cns3420vb()	(machine_arch_type == MACH_TYPE_CNS3420VB)
#else
# define machine_is_cns3420vb()	(0)
#endif

#ifdef CONFIG_MACH_OMAP4_PANDA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP4_PANDA
# endif
# define machine_is_omap4_panda()	(machine_arch_type == MACH_TYPE_OMAP4_PANDA)
#else
# define machine_is_omap4_panda()	(0)
#endif

#ifdef CONFIG_MACH_TI8168EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TI8168EVM
# endif
# define machine_is_ti8168evm()	(machine_arch_type == MACH_TYPE_TI8168EVM)
#else
# define machine_is_ti8168evm()	(0)
#endif

#ifdef CONFIG_MACH_TETON_BGA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TETON_BGA
# endif
# define machine_is_teton_bga()	(machine_arch_type == MACH_TYPE_TETON_BGA)
#else
# define machine_is_teton_bga()	(0)
#endif

#ifdef CONFIG_MACH_EUKREA_CPUIMX25SD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EUKREA_CPUIMX25SD
# endif
# define machine_is_eukrea_cpuimx25sd()	(machine_arch_type == MACH_TYPE_EUKREA_CPUIMX25SD)
#else
# define machine_is_eukrea_cpuimx25sd()	(0)
#endif

#ifdef CONFIG_MACH_EUKREA_CPUIMX35SD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EUKREA_CPUIMX35SD
# endif
# define machine_is_eukrea_cpuimx35sd()	(machine_arch_type == MACH_TYPE_EUKREA_CPUIMX35SD)
#else
# define machine_is_eukrea_cpuimx35sd()	(0)
#endif

#ifdef CONFIG_MACH_EUKREA_CPUIMX51SD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EUKREA_CPUIMX51SD
# endif
# define machine_is_eukrea_cpuimx51sd()	(machine_arch_type == MACH_TYPE_EUKREA_CPUIMX51SD)
#else
# define machine_is_eukrea_cpuimx51sd()	(0)
#endif

#ifdef CONFIG_MACH_EUKREA_CPUIMX51
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EUKREA_CPUIMX51
# endif
# define machine_is_eukrea_cpuimx51()	(machine_arch_type == MACH_TYPE_EUKREA_CPUIMX51)
#else
# define machine_is_eukrea_cpuimx51()	(0)
#endif

#ifdef CONFIG_MACH_SMDKC210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDKC210
# endif
# define machine_is_smdkc210()	(machine_arch_type == MACH_TYPE_SMDKC210)
#else
# define machine_is_smdkc210()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_BRAILLO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_BRAILLO
# endif
# define machine_is_omap3_braillo()	(machine_arch_type == MACH_TYPE_OMAP3_BRAILLO)
#else
# define machine_is_omap3_braillo()	(0)
#endif

#ifdef CONFIG_MACH_SPYPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPYPLUG
# endif
# define machine_is_spyplug()	(machine_arch_type == MACH_TYPE_SPYPLUG)
#else
# define machine_is_spyplug()	(0)
#endif

#ifdef CONFIG_MACH_GINGER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GINGER
# endif
# define machine_is_ginger()	(machine_arch_type == MACH_TYPE_GINGER)
#else
# define machine_is_ginger()	(0)
#endif

#ifdef CONFIG_MACH_TNY_T3530
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TNY_T3530
# endif
# define machine_is_tny_t3530()	(machine_arch_type == MACH_TYPE_TNY_T3530)
#else
# define machine_is_tny_t3530()	(0)
#endif

#ifdef CONFIG_MACH_PCA102
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCA102
# endif
# define machine_is_pca102()	(machine_arch_type == MACH_TYPE_PCA102)
#else
# define machine_is_pca102()	(0)
#endif

#ifdef CONFIG_MACH_SPADE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPADE
# endif
# define machine_is_spade()	(machine_arch_type == MACH_TYPE_SPADE)
#else
# define machine_is_spade()	(0)
#endif

#ifdef CONFIG_MACH_MXC25_TOPAZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXC25_TOPAZ
# endif
# define machine_is_mxc25_topaz()	(machine_arch_type == MACH_TYPE_MXC25_TOPAZ)
#else
# define machine_is_mxc25_topaz()	(0)
#endif

#ifdef CONFIG_MACH_T5325
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T5325
# endif
# define machine_is_t5325()	(machine_arch_type == MACH_TYPE_T5325)
#else
# define machine_is_t5325()	(0)
#endif

#ifdef CONFIG_MACH_GW2361
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GW2361
# endif
# define machine_is_gw2361()	(machine_arch_type == MACH_TYPE_GW2361)
#else
# define machine_is_gw2361()	(0)
#endif

#ifdef CONFIG_MACH_ELOG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELOG
# endif
# define machine_is_elog()	(machine_arch_type == MACH_TYPE_ELOG)
#else
# define machine_is_elog()	(0)
#endif

#ifdef CONFIG_MACH_INCOME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INCOME
# endif
# define machine_is_income()	(machine_arch_type == MACH_TYPE_INCOME)
#else
# define machine_is_income()	(0)
#endif

#ifdef CONFIG_MACH_BCM589X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCM589X
# endif
# define machine_is_bcm589x()	(machine_arch_type == MACH_TYPE_BCM589X)
#else
# define machine_is_bcm589x()	(0)
#endif

#ifdef CONFIG_MACH_ETNA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETNA
# endif
# define machine_is_etna()	(machine_arch_type == MACH_TYPE_ETNA)
#else
# define machine_is_etna()	(0)
#endif

#ifdef CONFIG_MACH_HAWKS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HAWKS
# endif
# define machine_is_hawks()	(machine_arch_type == MACH_TYPE_HAWKS)
#else
# define machine_is_hawks()	(0)
#endif

#ifdef CONFIG_MACH_MESON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MESON
# endif
# define machine_is_meson()	(machine_arch_type == MACH_TYPE_MESON)
#else
# define machine_is_meson()	(0)
#endif

#ifdef CONFIG_MACH_XSBASE255
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XSBASE255
# endif
# define machine_is_xsbase255()	(machine_arch_type == MACH_TYPE_XSBASE255)
#else
# define machine_is_xsbase255()	(0)
#endif

#ifdef CONFIG_MACH_PVM2030
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PVM2030
# endif
# define machine_is_pvm2030()	(machine_arch_type == MACH_TYPE_PVM2030)
#else
# define machine_is_pvm2030()	(0)
#endif

#ifdef CONFIG_MACH_MIOA502
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIOA502
# endif
# define machine_is_mioa502()	(machine_arch_type == MACH_TYPE_MIOA502)
#else
# define machine_is_mioa502()	(0)
#endif

#ifdef CONFIG_MACH_VVBOX_SDORIG2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VVBOX_SDORIG2
# endif
# define machine_is_vvbox_sdorig2()	(machine_arch_type == MACH_TYPE_VVBOX_SDORIG2)
#else
# define machine_is_vvbox_sdorig2()	(0)
#endif

#ifdef CONFIG_MACH_VVBOX_SDLITE2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VVBOX_SDLITE2
# endif
# define machine_is_vvbox_sdlite2()	(machine_arch_type == MACH_TYPE_VVBOX_SDLITE2)
#else
# define machine_is_vvbox_sdlite2()	(0)
#endif

#ifdef CONFIG_MACH_VVBOX_SDPRO4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VVBOX_SDPRO4
# endif
# define machine_is_vvbox_sdpro4()	(machine_arch_type == MACH_TYPE_VVBOX_SDPRO4)
#else
# define machine_is_vvbox_sdpro4()	(0)
#endif

#ifdef CONFIG_MACH_HTC_SPV_M700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTC_SPV_M700
# endif
# define machine_is_htc_spv_m700()	(machine_arch_type == MACH_TYPE_HTC_SPV_M700)
#else
# define machine_is_htc_spv_m700()	(0)
#endif

#ifdef CONFIG_MACH_MX257SX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX257SX
# endif
# define machine_is_mx257sx()	(machine_arch_type == MACH_TYPE_MX257SX)
#else
# define machine_is_mx257sx()	(0)
#endif

#ifdef CONFIG_MACH_GONI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GONI
# endif
# define machine_is_goni()	(machine_arch_type == MACH_TYPE_GONI)
#else
# define machine_is_goni()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X55_SVLTE_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X55_SVLTE_FFA
# endif
# define machine_is_msm8x55_svlte_ffa()	(machine_arch_type == MACH_TYPE_MSM8X55_SVLTE_FFA)
#else
# define machine_is_msm8x55_svlte_ffa()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X55_SVLTE_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X55_SVLTE_SURF
# endif
# define machine_is_msm8x55_svlte_surf()	(machine_arch_type == MACH_TYPE_MSM8X55_SVLTE_SURF)
#else
# define machine_is_msm8x55_svlte_surf()	(0)
#endif

#ifdef CONFIG_MACH_QUICKSTEP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QUICKSTEP
# endif
# define machine_is_quickstep()	(machine_arch_type == MACH_TYPE_QUICKSTEP)
#else
# define machine_is_quickstep()	(0)
#endif

#ifdef CONFIG_MACH_DMW96
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DMW96
# endif
# define machine_is_dmw96()	(machine_arch_type == MACH_TYPE_DMW96)
#else
# define machine_is_dmw96()	(0)
#endif

#ifdef CONFIG_MACH_HAMMERHEAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HAMMERHEAD
# endif
# define machine_is_hammerhead()	(machine_arch_type == MACH_TYPE_HAMMERHEAD)
#else
# define machine_is_hammerhead()	(0)
#endif

#ifdef CONFIG_MACH_TRIDENT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIDENT
# endif
# define machine_is_trident()	(machine_arch_type == MACH_TYPE_TRIDENT)
#else
# define machine_is_trident()	(0)
#endif

#ifdef CONFIG_MACH_LIGHTNING
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LIGHTNING
# endif
# define machine_is_lightning()	(machine_arch_type == MACH_TYPE_LIGHTNING)
#else
# define machine_is_lightning()	(0)
#endif

#ifdef CONFIG_MACH_ICONNECT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICONNECT
# endif
# define machine_is_iconnect()	(machine_arch_type == MACH_TYPE_ICONNECT)
#else
# define machine_is_iconnect()	(0)
#endif

#ifdef CONFIG_MACH_AUTOBOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AUTOBOT
# endif
# define machine_is_autobot()	(machine_arch_type == MACH_TYPE_AUTOBOT)
#else
# define machine_is_autobot()	(0)
#endif

#ifdef CONFIG_MACH_COCONUT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COCONUT
# endif
# define machine_is_coconut()	(machine_arch_type == MACH_TYPE_COCONUT)
#else
# define machine_is_coconut()	(0)
#endif

#ifdef CONFIG_MACH_DURIAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DURIAN
# endif
# define machine_is_durian()	(machine_arch_type == MACH_TYPE_DURIAN)
#else
# define machine_is_durian()	(0)
#endif

#ifdef CONFIG_MACH_CAYENNE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CAYENNE
# endif
# define machine_is_cayenne()	(machine_arch_type == MACH_TYPE_CAYENNE)
#else
# define machine_is_cayenne()	(0)
#endif

#ifdef CONFIG_MACH_FUJI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FUJI
# endif
# define machine_is_fuji()	(machine_arch_type == MACH_TYPE_FUJI)
#else
# define machine_is_fuji()	(0)
#endif

#ifdef CONFIG_MACH_SYNOLOGY_6282
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SYNOLOGY_6282
# endif
# define machine_is_synology_6282()	(machine_arch_type == MACH_TYPE_SYNOLOGY_6282)
#else
# define machine_is_synology_6282()	(0)
#endif

#ifdef CONFIG_MACH_EM1SY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EM1SY
# endif
# define machine_is_em1sy()	(machine_arch_type == MACH_TYPE_EM1SY)
#else
# define machine_is_em1sy()	(0)
#endif

#ifdef CONFIG_MACH_M502
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_M502
# endif
# define machine_is_m502()	(machine_arch_type == MACH_TYPE_M502)
#else
# define machine_is_m502()	(0)
#endif

#ifdef CONFIG_MACH_MATRIX518
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MATRIX518
# endif
# define machine_is_matrix518()	(machine_arch_type == MACH_TYPE_MATRIX518)
#else
# define machine_is_matrix518()	(0)
#endif

#ifdef CONFIG_MACH_TINY_GURNARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TINY_GURNARD
# endif
# define machine_is_tiny_gurnard()	(machine_arch_type == MACH_TYPE_TINY_GURNARD)
#else
# define machine_is_tiny_gurnard()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR1310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR1310
# endif
# define machine_is_spear1310()	(machine_arch_type == MACH_TYPE_SPEAR1310)
#else
# define machine_is_spear1310()	(0)
#endif

#ifdef CONFIG_MACH_BV07
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BV07
# endif
# define machine_is_bv07()	(machine_arch_type == MACH_TYPE_BV07)
#else
# define machine_is_bv07()	(0)
#endif

#ifdef CONFIG_MACH_MXT_TD61
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MXT_TD61
# endif
# define machine_is_mxt_td61()	(machine_arch_type == MACH_TYPE_MXT_TD61)
#else
# define machine_is_mxt_td61()	(0)
#endif

#ifdef CONFIG_MACH_OPENRD_ULTIMATE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OPENRD_ULTIMATE
# endif
# define machine_is_openrd_ultimate()	(machine_arch_type == MACH_TYPE_OPENRD_ULTIMATE)
#else
# define machine_is_openrd_ultimate()	(0)
#endif

#ifdef CONFIG_MACH_DEVIXP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEVIXP
# endif
# define machine_is_devixp()	(machine_arch_type == MACH_TYPE_DEVIXP)
#else
# define machine_is_devixp()	(0)
#endif

#ifdef CONFIG_MACH_MICCPT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MICCPT
# endif
# define machine_is_miccpt()	(machine_arch_type == MACH_TYPE_MICCPT)
#else
# define machine_is_miccpt()	(0)
#endif

#ifdef CONFIG_MACH_MIC256
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIC256
# endif
# define machine_is_mic256()	(machine_arch_type == MACH_TYPE_MIC256)
#else
# define machine_is_mic256()	(0)
#endif

#ifdef CONFIG_MACH_AS1167
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AS1167
# endif
# define machine_is_as1167()	(machine_arch_type == MACH_TYPE_AS1167)
#else
# define machine_is_as1167()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_IBIZA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_IBIZA
# endif
# define machine_is_omap3_ibiza()	(machine_arch_type == MACH_TYPE_OMAP3_IBIZA)
#else
# define machine_is_omap3_ibiza()	(0)
#endif

#ifdef CONFIG_MACH_U5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_U5500
# endif
# define machine_is_u5500()	(machine_arch_type == MACH_TYPE_U5500)
#else
# define machine_is_u5500()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_PICTO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_PICTO
# endif
# define machine_is_davinci_picto()	(machine_arch_type == MACH_TYPE_DAVINCI_PICTO)
#else
# define machine_is_davinci_picto()	(0)
#endif

#ifdef CONFIG_MACH_MECHA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MECHA
# endif
# define machine_is_mecha()	(machine_arch_type == MACH_TYPE_MECHA)
#else
# define machine_is_mecha()	(0)
#endif

#ifdef CONFIG_MACH_BUBBA3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BUBBA3
# endif
# define machine_is_bubba3()	(machine_arch_type == MACH_TYPE_BUBBA3)
#else
# define machine_is_bubba3()	(0)
#endif

#ifdef CONFIG_MACH_PUPITRE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PUPITRE
# endif
# define machine_is_pupitre()	(machine_arch_type == MACH_TYPE_PUPITRE)
#else
# define machine_is_pupitre()	(0)
#endif

#ifdef CONFIG_MACH_TEGRA_VOGUE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEGRA_VOGUE
# endif
# define machine_is_tegra_vogue()	(machine_arch_type == MACH_TYPE_TEGRA_VOGUE)
#else
# define machine_is_tegra_vogue()	(0)
#endif

#ifdef CONFIG_MACH_TEGRA_E1165
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEGRA_E1165
# endif
# define machine_is_tegra_e1165()	(machine_arch_type == MACH_TYPE_TEGRA_E1165)
#else
# define machine_is_tegra_e1165()	(0)
#endif

#ifdef CONFIG_MACH_SIMPLENET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMPLENET
# endif
# define machine_is_simplenet()	(machine_arch_type == MACH_TYPE_SIMPLENET)
#else
# define machine_is_simplenet()	(0)
#endif

#ifdef CONFIG_MACH_EC4350TBM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EC4350TBM
# endif
# define machine_is_ec4350tbm()	(machine_arch_type == MACH_TYPE_EC4350TBM)
#else
# define machine_is_ec4350tbm()	(0)
#endif

#ifdef CONFIG_MACH_PEC_TC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PEC_TC
# endif
# define machine_is_pec_tc()	(machine_arch_type == MACH_TYPE_PEC_TC)
#else
# define machine_is_pec_tc()	(0)
#endif

#ifdef CONFIG_MACH_PEC_HC2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PEC_HC2
# endif
# define machine_is_pec_hc2()	(machine_arch_type == MACH_TYPE_PEC_HC2)
#else
# define machine_is_pec_hc2()	(0)
#endif

#ifdef CONFIG_MACH_ESL_MOBILIS_A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_MOBILIS_A
# endif
# define machine_is_esl_mobilis_a()	(machine_arch_type == MACH_TYPE_ESL_MOBILIS_A)
#else
# define machine_is_esl_mobilis_a()	(0)
#endif

#ifdef CONFIG_MACH_ESL_MOBILIS_B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_MOBILIS_B
# endif
# define machine_is_esl_mobilis_b()	(machine_arch_type == MACH_TYPE_ESL_MOBILIS_B)
#else
# define machine_is_esl_mobilis_b()	(0)
#endif

#ifdef CONFIG_MACH_ESL_WAVE_A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_WAVE_A
# endif
# define machine_is_esl_wave_a()	(machine_arch_type == MACH_TYPE_ESL_WAVE_A)
#else
# define machine_is_esl_wave_a()	(0)
#endif

#ifdef CONFIG_MACH_ESL_WAVE_B
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ESL_WAVE_B
# endif
# define machine_is_esl_wave_b()	(machine_arch_type == MACH_TYPE_ESL_WAVE_B)
#else
# define machine_is_esl_wave_b()	(0)
#endif

#ifdef CONFIG_MACH_UNISENSE_MMM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNISENSE_MMM
# endif
# define machine_is_unisense_mmm()	(machine_arch_type == MACH_TYPE_UNISENSE_MMM)
#else
# define machine_is_unisense_mmm()	(0)
#endif

#ifdef CONFIG_MACH_BLUESHARK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUESHARK
# endif
# define machine_is_blueshark()	(machine_arch_type == MACH_TYPE_BLUESHARK)
#else
# define machine_is_blueshark()	(0)
#endif

#ifdef CONFIG_MACH_E10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_E10
# endif
# define machine_is_e10()	(machine_arch_type == MACH_TYPE_E10)
#else
# define machine_is_e10()	(0)
#endif

#ifdef CONFIG_MACH_APP3K_ROBIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_APP3K_ROBIN
# endif
# define machine_is_app3k_robin()	(machine_arch_type == MACH_TYPE_APP3K_ROBIN)
#else
# define machine_is_app3k_robin()	(0)
#endif

#ifdef CONFIG_MACH_POV15HD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POV15HD
# endif
# define machine_is_pov15hd()	(machine_arch_type == MACH_TYPE_POV15HD)
#else
# define machine_is_pov15hd()	(0)
#endif

#ifdef CONFIG_MACH_STELLA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STELLA
# endif
# define machine_is_stella()	(machine_arch_type == MACH_TYPE_STELLA)
#else
# define machine_is_stella()	(0)
#endif

#ifdef CONFIG_MACH_LINKSTATION_LSCHL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINKSTATION_LSCHL
# endif
# define machine_is_linkstation_lschl()	(machine_arch_type == MACH_TYPE_LINKSTATION_LSCHL)
#else
# define machine_is_linkstation_lschl()	(0)
#endif

#ifdef CONFIG_MACH_NETWALKER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETWALKER
# endif
# define machine_is_netwalker()	(machine_arch_type == MACH_TYPE_NETWALKER)
#else
# define machine_is_netwalker()	(0)
#endif

#ifdef CONFIG_MACH_ACSX106
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACSX106
# endif
# define machine_is_acsx106()	(machine_arch_type == MACH_TYPE_ACSX106)
#else
# define machine_is_acsx106()	(0)
#endif

#ifdef CONFIG_MACH_ATLAS5_C1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATLAS5_C1
# endif
# define machine_is_atlas5_c1()	(machine_arch_type == MACH_TYPE_ATLAS5_C1)
#else
# define machine_is_atlas5_c1()	(0)
#endif

#ifdef CONFIG_MACH_NSB3AST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSB3AST
# endif
# define machine_is_nsb3ast()	(machine_arch_type == MACH_TYPE_NSB3AST)
#else
# define machine_is_nsb3ast()	(0)
#endif

#ifdef CONFIG_MACH_GNET_SLC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GNET_SLC
# endif
# define machine_is_gnet_slc()	(machine_arch_type == MACH_TYPE_GNET_SLC)
#else
# define machine_is_gnet_slc()	(0)
#endif

#ifdef CONFIG_MACH_AF4000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AF4000
# endif
# define machine_is_af4000()	(machine_arch_type == MACH_TYPE_AF4000)
#else
# define machine_is_af4000()	(0)
#endif

#ifdef CONFIG_MACH_ARK9431
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARK9431
# endif
# define machine_is_ark9431()	(machine_arch_type == MACH_TYPE_ARK9431)
#else
# define machine_is_ark9431()	(0)
#endif

#ifdef CONFIG_MACH_FS_S5PC100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FS_S5PC100
# endif
# define machine_is_fs_s5pc100()	(machine_arch_type == MACH_TYPE_FS_S5PC100)
#else
# define machine_is_fs_s5pc100()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3505NOVA8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3505NOVA8
# endif
# define machine_is_omap3505nova8()	(machine_arch_type == MACH_TYPE_OMAP3505NOVA8)
#else
# define machine_is_omap3505nova8()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3621_EDP1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3621_EDP1
# endif
# define machine_is_omap3621_edp1()	(machine_arch_type == MACH_TYPE_OMAP3621_EDP1)
#else
# define machine_is_omap3621_edp1()	(0)
#endif

#ifdef CONFIG_MACH_ORATISAES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ORATISAES
# endif
# define machine_is_oratisaes()	(machine_arch_type == MACH_TYPE_ORATISAES)
#else
# define machine_is_oratisaes()	(0)
#endif

#ifdef CONFIG_MACH_SMDKV310
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDKV310
# endif
# define machine_is_smdkv310()	(machine_arch_type == MACH_TYPE_SMDKV310)
#else
# define machine_is_smdkv310()	(0)
#endif

#ifdef CONFIG_MACH_SIEMENS_L0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIEMENS_L0
# endif
# define machine_is_siemens_l0()	(machine_arch_type == MACH_TYPE_SIEMENS_L0)
#else
# define machine_is_siemens_l0()	(0)
#endif

#ifdef CONFIG_MACH_VENTANA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VENTANA
# endif
# define machine_is_ventana()	(machine_arch_type == MACH_TYPE_VENTANA)
#else
# define machine_is_ventana()	(0)
#endif

#ifdef CONFIG_MACH_WM8505_7IN_NETBOOK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WM8505_7IN_NETBOOK
# endif
# define machine_is_wm8505_7in_netbook()	(machine_arch_type == MACH_TYPE_WM8505_7IN_NETBOOK)
#else
# define machine_is_wm8505_7in_netbook()	(0)
#endif

#ifdef CONFIG_MACH_EC4350SDB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EC4350SDB
# endif
# define machine_is_ec4350sdb()	(machine_arch_type == MACH_TYPE_EC4350SDB)
#else
# define machine_is_ec4350sdb()	(0)
#endif

#ifdef CONFIG_MACH_MIMAS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIMAS
# endif
# define machine_is_mimas()	(machine_arch_type == MACH_TYPE_MIMAS)
#else
# define machine_is_mimas()	(0)
#endif

#ifdef CONFIG_MACH_TITAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TITAN
# endif
# define machine_is_titan()	(machine_arch_type == MACH_TYPE_TITAN)
#else
# define machine_is_titan()	(0)
#endif

#ifdef CONFIG_MACH_CRANEBOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CRANEBOARD
# endif
# define machine_is_craneboard()	(machine_arch_type == MACH_TYPE_CRANEBOARD)
#else
# define machine_is_craneboard()	(0)
#endif

#ifdef CONFIG_MACH_ES2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ES2440
# endif
# define machine_is_es2440()	(machine_arch_type == MACH_TYPE_ES2440)
#else
# define machine_is_es2440()	(0)
#endif

#ifdef CONFIG_MACH_NAJAY_A9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAJAY_A9263
# endif
# define machine_is_najay_a9263()	(machine_arch_type == MACH_TYPE_NAJAY_A9263)
#else
# define machine_is_najay_a9263()	(0)
#endif

#ifdef CONFIG_MACH_HTCTORNADO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCTORNADO
# endif
# define machine_is_htctornado()	(machine_arch_type == MACH_TYPE_HTCTORNADO)
#else
# define machine_is_htctornado()	(0)
#endif

#ifdef CONFIG_MACH_DIMM_MX257
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DIMM_MX257
# endif
# define machine_is_dimm_mx257()	(machine_arch_type == MACH_TYPE_DIMM_MX257)
#else
# define machine_is_dimm_mx257()	(0)
#endif

#ifdef CONFIG_MACH_JIGEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JIGEN
# endif
# define machine_is_jigen301()	(machine_arch_type == MACH_TYPE_JIGEN)
#else
# define machine_is_jigen301()	(0)
#endif

#ifdef CONFIG_MACH_SMDK6450
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMDK6450
# endif
# define machine_is_smdk6450()	(machine_arch_type == MACH_TYPE_SMDK6450)
#else
# define machine_is_smdk6450()	(0)
#endif

#ifdef CONFIG_MACH_MENO_QNG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MENO_QNG
# endif
# define machine_is_meno_qng()	(machine_arch_type == MACH_TYPE_MENO_QNG)
#else
# define machine_is_meno_qng()	(0)
#endif

#ifdef CONFIG_MACH_NS2416
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS2416
# endif
# define machine_is_ns2416()	(machine_arch_type == MACH_TYPE_NS2416)
#else
# define machine_is_ns2416()	(0)
#endif

#ifdef CONFIG_MACH_RPC353
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RPC353
# endif
# define machine_is_rpc353()	(machine_arch_type == MACH_TYPE_RPC353)
#else
# define machine_is_rpc353()	(0)
#endif

#ifdef CONFIG_MACH_TQ6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TQ6410
# endif
# define machine_is_tq6410()	(machine_arch_type == MACH_TYPE_TQ6410)
#else
# define machine_is_tq6410()	(0)
#endif

#ifdef CONFIG_MACH_SKY6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKY6410
# endif
# define machine_is_sky6410()	(machine_arch_type == MACH_TYPE_SKY6410)
#else
# define machine_is_sky6410()	(0)
#endif

#ifdef CONFIG_MACH_DYNASTY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DYNASTY
# endif
# define machine_is_dynasty()	(machine_arch_type == MACH_TYPE_DYNASTY)
#else
# define machine_is_dynasty()	(0)
#endif

#ifdef CONFIG_MACH_VIVO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIVO
# endif
# define machine_is_vivo()	(machine_arch_type == MACH_TYPE_VIVO)
#else
# define machine_is_vivo()	(0)
#endif

#ifdef CONFIG_MACH_BURY_BL7582
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BURY_BL7582
# endif
# define machine_is_bury_bl7582()	(machine_arch_type == MACH_TYPE_BURY_BL7582)
#else
# define machine_is_bury_bl7582()	(0)
#endif

#ifdef CONFIG_MACH_BURY_BPS5270
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BURY_BPS5270
# endif
# define machine_is_bury_bps5270()	(machine_arch_type == MACH_TYPE_BURY_BPS5270)
#else
# define machine_is_bury_bps5270()	(0)
#endif

#ifdef CONFIG_MACH_BASI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BASI
# endif
# define machine_is_basi()	(machine_arch_type == MACH_TYPE_BASI)
#else
# define machine_is_basi()	(0)
#endif

#ifdef CONFIG_MACH_TN200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TN200
# endif
# define machine_is_tn200()	(machine_arch_type == MACH_TYPE_TN200)
#else
# define machine_is_tn200()	(0)
#endif

#ifdef CONFIG_MACH_C2MMI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_C2MMI
# endif
# define machine_is_c2mmi()	(machine_arch_type == MACH_TYPE_C2MMI)
#else
# define machine_is_c2mmi()	(0)
#endif

#ifdef CONFIG_MACH_MESON_6236M
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MESON_6236M
# endif
# define machine_is_meson_6236m()	(machine_arch_type == MACH_TYPE_MESON_6236M)
#else
# define machine_is_meson_6236m()	(0)
#endif

#ifdef CONFIG_MACH_MESON_8626M
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MESON_8626M
# endif
# define machine_is_meson_8626m()	(machine_arch_type == MACH_TYPE_MESON_8626M)
#else
# define machine_is_meson_8626m()	(0)
#endif

#ifdef CONFIG_MACH_TUBE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TUBE
# endif
# define machine_is_tube()	(machine_arch_type == MACH_TYPE_TUBE)
#else
# define machine_is_tube()	(0)
#endif

#ifdef CONFIG_MACH_MESSINA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MESSINA
# endif
# define machine_is_messina()	(machine_arch_type == MACH_TYPE_MESSINA)
#else
# define machine_is_messina()	(0)
#endif

#ifdef CONFIG_MACH_MX50_ARM2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX50_ARM2
# endif
# define machine_is_mx50_arm2()	(machine_arch_type == MACH_TYPE_MX50_ARM2)
#else
# define machine_is_mx50_arm2()	(0)
#endif

#ifdef CONFIG_MACH_CETUS9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CETUS9263
# endif
# define machine_is_cetus9263()	(machine_arch_type == MACH_TYPE_CETUS9263)
#else
# define machine_is_cetus9263()	(0)
#endif

#ifdef CONFIG_MACH_BROWNSTONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BROWNSTONE
# endif
# define machine_is_brownstone()	(machine_arch_type == MACH_TYPE_BROWNSTONE)
#else
# define machine_is_brownstone()	(0)
#endif

#ifdef CONFIG_MACH_VMX25
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VMX25
# endif
# define machine_is_vmx25()	(machine_arch_type == MACH_TYPE_VMX25)
#else
# define machine_is_vmx25()	(0)
#endif

#ifdef CONFIG_MACH_VMX51
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VMX51
# endif
# define machine_is_vmx51()	(machine_arch_type == MACH_TYPE_VMX51)
#else
# define machine_is_vmx51()	(0)
#endif

#ifdef CONFIG_MACH_ABACUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ABACUS
# endif
# define machine_is_abacus()	(machine_arch_type == MACH_TYPE_ABACUS)
#else
# define machine_is_abacus()	(0)
#endif

#ifdef CONFIG_MACH_CM4745
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM4745
# endif
# define machine_is_cm4745()	(machine_arch_type == MACH_TYPE_CM4745)
#else
# define machine_is_cm4745()	(0)
#endif

#ifdef CONFIG_MACH_ORATISLINK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ORATISLINK
# endif
# define machine_is_oratislink()	(machine_arch_type == MACH_TYPE_ORATISLINK)
#else
# define machine_is_oratislink()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM365_DVR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM365_DVR
# endif
# define machine_is_davinci_dm365_dvr()	(machine_arch_type == MACH_TYPE_DAVINCI_DM365_DVR)
#else
# define machine_is_davinci_dm365_dvr()	(0)
#endif

#ifdef CONFIG_MACH_NETVIZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETVIZ
# endif
# define machine_is_netviz()	(machine_arch_type == MACH_TYPE_NETVIZ)
#else
# define machine_is_netviz()	(0)
#endif

#ifdef CONFIG_MACH_FLEXIBITY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLEXIBITY
# endif
# define machine_is_flexibity()	(machine_arch_type == MACH_TYPE_FLEXIBITY)
#else
# define machine_is_flexibity()	(0)
#endif

#ifdef CONFIG_MACH_WLAN_COMPUTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WLAN_COMPUTER
# endif
# define machine_is_wlan_computer()	(machine_arch_type == MACH_TYPE_WLAN_COMPUTER)
#else
# define machine_is_wlan_computer()	(0)
#endif

#ifdef CONFIG_MACH_LPC24XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LPC24XX
# endif
# define machine_is_lpc24xx()	(machine_arch_type == MACH_TYPE_LPC24XX)
#else
# define machine_is_lpc24xx()	(0)
#endif

#ifdef CONFIG_MACH_SPICA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPICA
# endif
# define machine_is_spica()	(machine_arch_type == MACH_TYPE_SPICA)
#else
# define machine_is_spica()	(0)
#endif

#ifdef CONFIG_MACH_GPSDISPLAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GPSDISPLAY
# endif
# define machine_is_gpsdisplay()	(machine_arch_type == MACH_TYPE_GPSDISPLAY)
#else
# define machine_is_gpsdisplay()	(0)
#endif

#ifdef CONFIG_MACH_BIPNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BIPNET
# endif
# define machine_is_bipnet()	(machine_arch_type == MACH_TYPE_BIPNET)
#else
# define machine_is_bipnet()	(0)
#endif

#ifdef CONFIG_MACH_OVERO_CTU_INERTIAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OVERO_CTU_INERTIAL
# endif
# define machine_is_overo_ctu_inertial()	(machine_arch_type == MACH_TYPE_OVERO_CTU_INERTIAL)
#else
# define machine_is_overo_ctu_inertial()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM355_MMM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM355_MMM
# endif
# define machine_is_davinci_dm355_mmm()	(machine_arch_type == MACH_TYPE_DAVINCI_DM355_MMM)
#else
# define machine_is_davinci_dm355_mmm()	(0)
#endif

#ifdef CONFIG_MACH_PC9260_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PC9260_V2
# endif
# define machine_is_pc9260_v2()	(machine_arch_type == MACH_TYPE_PC9260_V2)
#else
# define machine_is_pc9260_v2()	(0)
#endif

#ifdef CONFIG_MACH_PTX7545
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PTX7545
# endif
# define machine_is_ptx7545()	(machine_arch_type == MACH_TYPE_PTX7545)
#else
# define machine_is_ptx7545()	(0)
#endif

#ifdef CONFIG_MACH_TM_EFDC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TM_EFDC
# endif
# define machine_is_tm_efdc()	(machine_arch_type == MACH_TYPE_TM_EFDC)
#else
# define machine_is_tm_efdc()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_WALDO1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_WALDO1
# endif
# define machine_is_omap3_waldo1()	(machine_arch_type == MACH_TYPE_OMAP3_WALDO1)
#else
# define machine_is_omap3_waldo1()	(0)
#endif

#ifdef CONFIG_MACH_FLYER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FLYER
# endif
# define machine_is_flyer()	(machine_arch_type == MACH_TYPE_FLYER)
#else
# define machine_is_flyer()	(0)
#endif

#ifdef CONFIG_MACH_TORNADO3240
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TORNADO3240
# endif
# define machine_is_tornado3240()	(machine_arch_type == MACH_TYPE_TORNADO3240)
#else
# define machine_is_tornado3240()	(0)
#endif

#ifdef CONFIG_MACH_SOLI_01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SOLI_01
# endif
# define machine_is_soli_01()	(machine_arch_type == MACH_TYPE_SOLI_01)
#else
# define machine_is_soli_01()	(0)
#endif

#ifdef CONFIG_MACH_OMAPL138_EUROPALC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAPL138_EUROPALC
# endif
# define machine_is_omapl138_europalc()	(machine_arch_type == MACH_TYPE_OMAPL138_EUROPALC)
#else
# define machine_is_omapl138_europalc()	(0)
#endif

#ifdef CONFIG_MACH_HELIOS_V1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HELIOS_V1
# endif
# define machine_is_helios_v1()	(machine_arch_type == MACH_TYPE_HELIOS_V1)
#else
# define machine_is_helios_v1()	(0)
#endif

#ifdef CONFIG_MACH_NETSPACE_LITE_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NETSPACE_LITE_V2
# endif
# define machine_is_netspace_lite_v2()	(machine_arch_type == MACH_TYPE_NETSPACE_LITE_V2)
#else
# define machine_is_netspace_lite_v2()	(0)
#endif

#ifdef CONFIG_MACH_SSC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SSC
# endif
# define machine_is_ssc()	(machine_arch_type == MACH_TYPE_SSC)
#else
# define machine_is_ssc()	(0)
#endif

#ifdef CONFIG_MACH_PREMIERWAVE_EN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PREMIERWAVE_EN
# endif
# define machine_is_premierwave_en()	(machine_arch_type == MACH_TYPE_PREMIERWAVE_EN)
#else
# define machine_is_premierwave_en()	(0)
#endif

#ifdef CONFIG_MACH_WASABI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WASABI
# endif
# define machine_is_wasabi()	(machine_arch_type == MACH_TYPE_WASABI)
#else
# define machine_is_wasabi()	(0)
#endif

#ifdef CONFIG_MACH_MX50_RDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX50_RDP
# endif
# define machine_is_mx50_rdp()	(machine_arch_type == MACH_TYPE_MX50_RDP)
#else
# define machine_is_mx50_rdp()	(0)
#endif

#ifdef CONFIG_MACH_UNIVERSAL_C210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNIVERSAL_C210
# endif
# define machine_is_universal_c210()	(machine_arch_type == MACH_TYPE_UNIVERSAL_C210)
#else
# define machine_is_universal_c210()	(0)
#endif

#ifdef CONFIG_MACH_REAL6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REAL6410
# endif
# define machine_is_real6410()	(machine_arch_type == MACH_TYPE_REAL6410)
#else
# define machine_is_real6410()	(0)
#endif

#ifdef CONFIG_MACH_SPX_SAKURA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPX_SAKURA
# endif
# define machine_is_spx_sakura()	(machine_arch_type == MACH_TYPE_SPX_SAKURA)
#else
# define machine_is_spx_sakura()	(0)
#endif

#ifdef CONFIG_MACH_IJ3K_2440
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IJ3K_2440
# endif
# define machine_is_ij3k_2440()	(machine_arch_type == MACH_TYPE_IJ3K_2440)
#else
# define machine_is_ij3k_2440()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_BC10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_BC10
# endif
# define machine_is_omap3_bc10()	(machine_arch_type == MACH_TYPE_OMAP3_BC10)
#else
# define machine_is_omap3_bc10()	(0)
#endif

#ifdef CONFIG_MACH_THEBE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_THEBE
# endif
# define machine_is_thebe()	(machine_arch_type == MACH_TYPE_THEBE)
#else
# define machine_is_thebe()	(0)
#endif

#ifdef CONFIG_MACH_RV082
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RV082
# endif
# define machine_is_rv082()	(machine_arch_type == MACH_TYPE_RV082)
#else
# define machine_is_rv082()	(0)
#endif

#ifdef CONFIG_MACH_ARMLGUEST
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMLGUEST
# endif
# define machine_is_armlguest()	(machine_arch_type == MACH_TYPE_ARMLGUEST)
#else
# define machine_is_armlguest()	(0)
#endif

#ifdef CONFIG_MACH_TJINC1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TJINC1000
# endif
# define machine_is_tjinc1000()	(machine_arch_type == MACH_TYPE_TJINC1000)
#else
# define machine_is_tjinc1000()	(0)
#endif

#ifdef CONFIG_MACH_DOCKSTAR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DOCKSTAR
# endif
# define machine_is_dockstar()	(machine_arch_type == MACH_TYPE_DOCKSTAR)
#else
# define machine_is_dockstar()	(0)
#endif

#ifdef CONFIG_MACH_AX8008
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AX8008
# endif
# define machine_is_ax8008()	(machine_arch_type == MACH_TYPE_AX8008)
#else
# define machine_is_ax8008()	(0)
#endif

#ifdef CONFIG_MACH_GNET_SGCE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GNET_SGCE
# endif
# define machine_is_gnet_sgce()	(machine_arch_type == MACH_TYPE_GNET_SGCE)
#else
# define machine_is_gnet_sgce()	(0)
#endif

#ifdef CONFIG_MACH_PXWNAS_500_1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PXWNAS_500_1000
# endif
# define machine_is_pxwnas_500_1000()	(machine_arch_type == MACH_TYPE_PXWNAS_500_1000)
#else
# define machine_is_pxwnas_500_1000()	(0)
#endif

#ifdef CONFIG_MACH_EA20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EA20
# endif
# define machine_is_ea20()	(machine_arch_type == MACH_TYPE_EA20)
#else
# define machine_is_ea20()	(0)
#endif

#ifdef CONFIG_MACH_AWM2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AWM2
# endif
# define machine_is_awm2()	(machine_arch_type == MACH_TYPE_AWM2)
#else
# define machine_is_awm2()	(0)
#endif

#ifdef CONFIG_MACH_TI8148EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TI8148EVM
# endif
# define machine_is_ti8148evm()	(machine_arch_type == MACH_TYPE_TI8148EVM)
#else
# define machine_is_ti8148evm()	(0)
#endif

#ifdef CONFIG_MACH_SEABOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SEABOARD
# endif
# define machine_is_seaboard()	(machine_arch_type == MACH_TYPE_SEABOARD)
#else
# define machine_is_seaboard()	(0)
#endif

#ifdef CONFIG_MACH_LINKSTATION_CHLV2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LINKSTATION_CHLV2
# endif
# define machine_is_linkstation_chlv2()	(machine_arch_type == MACH_TYPE_LINKSTATION_CHLV2)
#else
# define machine_is_linkstation_chlv2()	(0)
#endif

#ifdef CONFIG_MACH_TERA_PRO2_RACK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TERA_PRO2_RACK
# endif
# define machine_is_tera_pro2_rack()	(machine_arch_type == MACH_TYPE_TERA_PRO2_RACK)
#else
# define machine_is_tera_pro2_rack()	(0)
#endif

#ifdef CONFIG_MACH_RUBYS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RUBYS
# endif
# define machine_is_rubys()	(machine_arch_type == MACH_TYPE_RUBYS)
#else
# define machine_is_rubys()	(0)
#endif

#ifdef CONFIG_MACH_AQUARIUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AQUARIUS
# endif
# define machine_is_aquarius()	(machine_arch_type == MACH_TYPE_AQUARIUS)
#else
# define machine_is_aquarius()	(0)
#endif

#ifdef CONFIG_MACH_MX53_ARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX53_ARD
# endif
# define machine_is_mx53_ard()	(machine_arch_type == MACH_TYPE_MX53_ARD)
#else
# define machine_is_mx53_ard()	(0)
#endif

#ifdef CONFIG_MACH_MX53_SMD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX53_SMD
# endif
# define machine_is_mx53_smd()	(machine_arch_type == MACH_TYPE_MX53_SMD)
#else
# define machine_is_mx53_smd()	(0)
#endif

#ifdef CONFIG_MACH_LSWXL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LSWXL
# endif
# define machine_is_lswxl()	(machine_arch_type == MACH_TYPE_LSWXL)
#else
# define machine_is_lswxl()	(0)
#endif

#ifdef CONFIG_MACH_DOVE_AVNG_V3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DOVE_AVNG_V3
# endif
# define machine_is_dove_avng_v3()	(machine_arch_type == MACH_TYPE_DOVE_AVNG_V3)
#else
# define machine_is_dove_avng_v3()	(0)
#endif

#ifdef CONFIG_MACH_SDI_ESS_9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SDI_ESS_9263
# endif
# define machine_is_sdi_ess_9263()	(machine_arch_type == MACH_TYPE_SDI_ESS_9263)
#else
# define machine_is_sdi_ess_9263()	(0)
#endif

#ifdef CONFIG_MACH_JOCPU550
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JOCPU550
# endif
# define machine_is_jocpu550()	(machine_arch_type == MACH_TYPE_JOCPU550)
#else
# define machine_is_jocpu550()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_RUMI3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_RUMI3
# endif
# define machine_is_msm8x60_rumi3()	(machine_arch_type == MACH_TYPE_MSM8X60_RUMI3)
#else
# define machine_is_msm8x60_rumi3()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_FFA
# endif
# define machine_is_msm8x60_ffa()	(machine_arch_type == MACH_TYPE_MSM8X60_FFA)
#else
# define machine_is_msm8x60_ffa()	(0)
#endif

#ifdef CONFIG_MACH_YANOMAMI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_YANOMAMI
# endif
# define machine_is_yanomami()	(machine_arch_type == MACH_TYPE_YANOMAMI)
#else
# define machine_is_yanomami()	(0)
#endif

#ifdef CONFIG_MACH_GTA04
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GTA04
# endif
# define machine_is_gta04()	(machine_arch_type == MACH_TYPE_GTA04)
#else
# define machine_is_gta04()	(0)
#endif

#ifdef CONFIG_MACH_CM_A510
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_A510
# endif
# define machine_is_cm_a510()	(machine_arch_type == MACH_TYPE_CM_A510)
#else
# define machine_is_cm_a510()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_RFS200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_RFS200
# endif
# define machine_is_omap3_rfs200()	(machine_arch_type == MACH_TYPE_OMAP3_RFS200)
#else
# define machine_is_omap3_rfs200()	(0)
#endif

#ifdef CONFIG_MACH_KX33XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KX33XX
# endif
# define machine_is_kx33xx()	(machine_arch_type == MACH_TYPE_KX33XX)
#else
# define machine_is_kx33xx()	(0)
#endif

#ifdef CONFIG_MACH_PTX7510
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PTX7510
# endif
# define machine_is_ptx7510()	(machine_arch_type == MACH_TYPE_PTX7510)
#else
# define machine_is_ptx7510()	(0)
#endif

#ifdef CONFIG_MACH_TOP9000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOP9000
# endif
# define machine_is_top9000()	(machine_arch_type == MACH_TYPE_TOP9000)
#else
# define machine_is_top9000()	(0)
#endif

#ifdef CONFIG_MACH_TEENOTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEENOTE
# endif
# define machine_is_teenote()	(machine_arch_type == MACH_TYPE_TEENOTE)
#else
# define machine_is_teenote()	(0)
#endif

#ifdef CONFIG_MACH_TS3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS3
# endif
# define machine_is_ts3()	(machine_arch_type == MACH_TYPE_TS3)
#else
# define machine_is_ts3()	(0)
#endif

#ifdef CONFIG_MACH_A0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A0
# endif
# define machine_is_a0()	(machine_arch_type == MACH_TYPE_A0)
#else
# define machine_is_a0()	(0)
#endif

#ifdef CONFIG_MACH_FSM9XXX_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FSM9XXX_SURF
# endif
# define machine_is_fsm9xxx_surf()	(machine_arch_type == MACH_TYPE_FSM9XXX_SURF)
#else
# define machine_is_fsm9xxx_surf()	(0)
#endif

#ifdef CONFIG_MACH_FSM9XXX_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FSM9XXX_FFA
# endif
# define machine_is_fsm9xxx_ffa()	(machine_arch_type == MACH_TYPE_FSM9XXX_FFA)
#else
# define machine_is_fsm9xxx_ffa()	(0)
#endif

#ifdef CONFIG_MACH_FRRHWCDMA60W
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FRRHWCDMA60W
# endif
# define machine_is_frrhwcdma60w()	(machine_arch_type == MACH_TYPE_FRRHWCDMA60W)
#else
# define machine_is_frrhwcdma60w()	(0)
#endif

#ifdef CONFIG_MACH_REMUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REMUS
# endif
# define machine_is_remus()	(machine_arch_type == MACH_TYPE_REMUS)
#else
# define machine_is_remus()	(0)
#endif

#ifdef CONFIG_MACH_AT91CAP7XDK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91CAP7XDK
# endif
# define machine_is_at91cap7xdk()	(machine_arch_type == MACH_TYPE_AT91CAP7XDK)
#else
# define machine_is_at91cap7xdk()	(0)
#endif

#ifdef CONFIG_MACH_AT91CAP7STK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91CAP7STK
# endif
# define machine_is_at91cap7stk()	(machine_arch_type == MACH_TYPE_AT91CAP7STK)
#else
# define machine_is_at91cap7stk()	(0)
#endif

#ifdef CONFIG_MACH_KT_SBC_SAM9_1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KT_SBC_SAM9_1
# endif
# define machine_is_kt_sbc_sam9_1()	(machine_arch_type == MACH_TYPE_KT_SBC_SAM9_1)
#else
# define machine_is_kt_sbc_sam9_1()	(0)
#endif

#ifdef CONFIG_MACH_ARMADA_XP_DB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMADA_XP_DB
# endif
# define machine_is_armada_xp_db()	(machine_arch_type == MACH_TYPE_ARMADA_XP_DB)
#else
# define machine_is_armada_xp_db()	(0)
#endif

#ifdef CONFIG_MACH_SPDM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPDM
# endif
# define machine_is_spdm()	(machine_arch_type == MACH_TYPE_SPDM)
#else
# define machine_is_spdm()	(0)
#endif

#ifdef CONFIG_MACH_GTIB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GTIB
# endif
# define machine_is_gtib()	(machine_arch_type == MACH_TYPE_GTIB)
#else
# define machine_is_gtib()	(0)
#endif

#ifdef CONFIG_MACH_DGM3240
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DGM3240
# endif
# define machine_is_dgm3240()	(machine_arch_type == MACH_TYPE_DGM3240)
#else
# define machine_is_dgm3240()	(0)
#endif

#ifdef CONFIG_MACH_HTCMEGA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTCMEGA
# endif
# define machine_is_htcmega()	(machine_arch_type == MACH_TYPE_HTCMEGA)
#else
# define machine_is_htcmega()	(0)
#endif

#ifdef CONFIG_MACH_TRICORDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRICORDER
# endif
# define machine_is_tricorder()	(machine_arch_type == MACH_TYPE_TRICORDER)
#else
# define machine_is_tricorder()	(0)
#endif

#ifdef CONFIG_MACH_TX28
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TX28
# endif
# define machine_is_tx28()	(machine_arch_type == MACH_TYPE_TX28)
#else
# define machine_is_tx28()	(0)
#endif

#ifdef CONFIG_MACH_BSTBRD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BSTBRD
# endif
# define machine_is_bstbrd()	(machine_arch_type == MACH_TYPE_BSTBRD)
#else
# define machine_is_bstbrd()	(0)
#endif

#ifdef CONFIG_MACH_PWB3090
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PWB3090
# endif
# define machine_is_pwb3090()	(machine_arch_type == MACH_TYPE_PWB3090)
#else
# define machine_is_pwb3090()	(0)
#endif

#ifdef CONFIG_MACH_IDEA6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IDEA6410
# endif
# define machine_is_idea6410()	(machine_arch_type == MACH_TYPE_IDEA6410)
#else
# define machine_is_idea6410()	(0)
#endif

#ifdef CONFIG_MACH_QBC9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QBC9263
# endif
# define machine_is_qbc9263()	(machine_arch_type == MACH_TYPE_QBC9263)
#else
# define machine_is_qbc9263()	(0)
#endif

#ifdef CONFIG_MACH_BORABORA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BORABORA
# endif
# define machine_is_borabora()	(machine_arch_type == MACH_TYPE_BORABORA)
#else
# define machine_is_borabora()	(0)
#endif

#ifdef CONFIG_MACH_VALDEZ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VALDEZ
# endif
# define machine_is_valdez()	(machine_arch_type == MACH_TYPE_VALDEZ)
#else
# define machine_is_valdez()	(0)
#endif

#ifdef CONFIG_MACH_LS9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LS9G20
# endif
# define machine_is_ls9g20()	(machine_arch_type == MACH_TYPE_LS9G20)
#else
# define machine_is_ls9g20()	(0)
#endif

#ifdef CONFIG_MACH_MIOS_V1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIOS_V1
# endif
# define machine_is_mios_v1()	(machine_arch_type == MACH_TYPE_MIOS_V1)
#else
# define machine_is_mios_v1()	(0)
#endif

#ifdef CONFIG_MACH_S5PC110_CRESPO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5PC110_CRESPO
# endif
# define machine_is_s5pc110_crespo()	(machine_arch_type == MACH_TYPE_S5PC110_CRESPO)
#else
# define machine_is_s5pc110_crespo()	(0)
#endif

#ifdef CONFIG_MACH_CONTROLTEK9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CONTROLTEK9G20
# endif
# define machine_is_controltek9g20()	(machine_arch_type == MACH_TYPE_CONTROLTEK9G20)
#else
# define machine_is_controltek9g20()	(0)
#endif

#ifdef CONFIG_MACH_TIN307
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TIN307
# endif
# define machine_is_tin307()	(machine_arch_type == MACH_TYPE_TIN307)
#else
# define machine_is_tin307()	(0)
#endif

#ifdef CONFIG_MACH_TIN510
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TIN510
# endif
# define machine_is_tin510()	(machine_arch_type == MACH_TYPE_TIN510)
#else
# define machine_is_tin510()	(0)
#endif

#ifdef CONFIG_MACH_BLUECHEESE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUECHEESE
# endif
# define machine_is_bluecheese()	(machine_arch_type == MACH_TYPE_BLUECHEESE)
#else
# define machine_is_bluecheese()	(0)
#endif

#ifdef CONFIG_MACH_TEM3X30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEM3X30
# endif
# define machine_is_tem3x30()	(machine_arch_type == MACH_TYPE_TEM3X30)
#else
# define machine_is_tem3x30()	(0)
#endif

#ifdef CONFIG_MACH_HARVEST_DESOTO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HARVEST_DESOTO
# endif
# define machine_is_harvest_desoto()	(machine_arch_type == MACH_TYPE_HARVEST_DESOTO)
#else
# define machine_is_harvest_desoto()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_QRDC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_QRDC
# endif
# define machine_is_msm8x60_qrdc()	(machine_arch_type == MACH_TYPE_MSM8X60_QRDC)
#else
# define machine_is_msm8x60_qrdc()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR900
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR900
# endif
# define machine_is_spear900()	(machine_arch_type == MACH_TYPE_SPEAR900)
#else
# define machine_is_spear900()	(0)
#endif

#ifdef CONFIG_MACH_PCONTROL_G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCONTROL_G20
# endif
# define machine_is_pcontrol_g20()	(machine_arch_type == MACH_TYPE_PCONTROL_G20)
#else
# define machine_is_pcontrol_g20()	(0)
#endif

#ifdef CONFIG_MACH_RDSTOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RDSTOR
# endif
# define machine_is_rdstor()	(machine_arch_type == MACH_TYPE_RDSTOR)
#else
# define machine_is_rdstor()	(0)
#endif

#ifdef CONFIG_MACH_USDLOADER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_USDLOADER
# endif
# define machine_is_usdloader()	(machine_arch_type == MACH_TYPE_USDLOADER)
#else
# define machine_is_usdloader()	(0)
#endif

#ifdef CONFIG_MACH_TSOPLOADER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TSOPLOADER
# endif
# define machine_is_tsoploader()	(machine_arch_type == MACH_TYPE_TSOPLOADER)
#else
# define machine_is_tsoploader()	(0)
#endif

#ifdef CONFIG_MACH_KRONOS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KRONOS
# endif
# define machine_is_kronos()	(machine_arch_type == MACH_TYPE_KRONOS)
#else
# define machine_is_kronos()	(0)
#endif

#ifdef CONFIG_MACH_FFCORE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FFCORE
# endif
# define machine_is_ffcore()	(machine_arch_type == MACH_TYPE_FFCORE)
#else
# define machine_is_ffcore()	(0)
#endif

#ifdef CONFIG_MACH_MONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MONE
# endif
# define machine_is_mone()	(machine_arch_type == MACH_TYPE_MONE)
#else
# define machine_is_mone()	(0)
#endif

#ifdef CONFIG_MACH_UNIT2S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNIT2S
# endif
# define machine_is_unit2s()	(machine_arch_type == MACH_TYPE_UNIT2S)
#else
# define machine_is_unit2s()	(0)
#endif

#ifdef CONFIG_MACH_ACER_A5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACER_A5
# endif
# define machine_is_acer_a5()	(machine_arch_type == MACH_TYPE_ACER_A5)
#else
# define machine_is_acer_a5()	(0)
#endif

#ifdef CONFIG_MACH_ETHERPRO_ISP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ETHERPRO_ISP
# endif
# define machine_is_etherpro_isp()	(machine_arch_type == MACH_TYPE_ETHERPRO_ISP)
#else
# define machine_is_etherpro_isp()	(0)
#endif

#ifdef CONFIG_MACH_STRETCHS7000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STRETCHS7000
# endif
# define machine_is_stretchs7000()	(machine_arch_type == MACH_TYPE_STRETCHS7000)
#else
# define machine_is_stretchs7000()	(0)
#endif

#ifdef CONFIG_MACH_P87_SMARTSIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_P87_SMARTSIM
# endif
# define machine_is_p87_smartsim()	(machine_arch_type == MACH_TYPE_P87_SMARTSIM)
#else
# define machine_is_p87_smartsim()	(0)
#endif

#ifdef CONFIG_MACH_TULIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TULIP
# endif
# define machine_is_tulip()	(machine_arch_type == MACH_TYPE_TULIP)
#else
# define machine_is_tulip()	(0)
#endif

#ifdef CONFIG_MACH_SUNFLOWER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SUNFLOWER
# endif
# define machine_is_sunflower()	(machine_arch_type == MACH_TYPE_SUNFLOWER)
#else
# define machine_is_sunflower()	(0)
#endif

#ifdef CONFIG_MACH_RIB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RIB
# endif
# define machine_is_rib()	(machine_arch_type == MACH_TYPE_RIB)
#else
# define machine_is_rib()	(0)
#endif

#ifdef CONFIG_MACH_CLOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CLOD
# endif
# define machine_is_clod()	(machine_arch_type == MACH_TYPE_CLOD)
#else
# define machine_is_clod()	(0)
#endif

#ifdef CONFIG_MACH_RUMP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RUMP
# endif
# define machine_is_rump()	(machine_arch_type == MACH_TYPE_RUMP)
#else
# define machine_is_rump()	(0)
#endif

#ifdef CONFIG_MACH_TENDERLOIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TENDERLOIN
# endif
# define machine_is_tenderloin()	(machine_arch_type == MACH_TYPE_TENDERLOIN)
#else
# define machine_is_tenderloin()	(0)
#endif

#ifdef CONFIG_MACH_SHORTLOIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHORTLOIN
# endif
# define machine_is_shortloin()	(machine_arch_type == MACH_TYPE_SHORTLOIN)
#else
# define machine_is_shortloin()	(0)
#endif

#ifdef CONFIG_MACH_ANTARES
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANTARES
# endif
# define machine_is_antares()	(machine_arch_type == MACH_TYPE_ANTARES)
#else
# define machine_is_antares()	(0)
#endif

#ifdef CONFIG_MACH_WB40N
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WB40N
# endif
# define machine_is_wb40n()	(machine_arch_type == MACH_TYPE_WB40N)
#else
# define machine_is_wb40n()	(0)
#endif

#ifdef CONFIG_MACH_HERRING
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HERRING
# endif
# define machine_is_herring()	(machine_arch_type == MACH_TYPE_HERRING)
#else
# define machine_is_herring()	(0)
#endif

#ifdef CONFIG_MACH_NAXY400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAXY400
# endif
# define machine_is_naxy400()	(machine_arch_type == MACH_TYPE_NAXY400)
#else
# define machine_is_naxy400()	(0)
#endif

#ifdef CONFIG_MACH_NAXY1200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAXY1200
# endif
# define machine_is_naxy1200()	(machine_arch_type == MACH_TYPE_NAXY1200)
#else
# define machine_is_naxy1200()	(0)
#endif

#ifdef CONFIG_MACH_VPR200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VPR200
# endif
# define machine_is_vpr200()	(machine_arch_type == MACH_TYPE_VPR200)
#else
# define machine_is_vpr200()	(0)
#endif

#ifdef CONFIG_MACH_BUG20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BUG20
# endif
# define machine_is_bug20()	(machine_arch_type == MACH_TYPE_BUG20)
#else
# define machine_is_bug20()	(0)
#endif

#ifdef CONFIG_MACH_GOFLEXNET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GOFLEXNET
# endif
# define machine_is_goflexnet()	(machine_arch_type == MACH_TYPE_GOFLEXNET)
#else
# define machine_is_goflexnet()	(0)
#endif

#ifdef CONFIG_MACH_TORBRECK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TORBRECK
# endif
# define machine_is_torbreck()	(machine_arch_type == MACH_TYPE_TORBRECK)
#else
# define machine_is_torbreck()	(0)
#endif

#ifdef CONFIG_MACH_SAARB_MG1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAARB_MG1
# endif
# define machine_is_saarb_mg1()	(machine_arch_type == MACH_TYPE_SAARB_MG1)
#else
# define machine_is_saarb_mg1()	(0)
#endif

#ifdef CONFIG_MACH_CALLISTO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CALLISTO
# endif
# define machine_is_callisto()	(machine_arch_type == MACH_TYPE_CALLISTO)
#else
# define machine_is_callisto()	(0)
#endif

#ifdef CONFIG_MACH_MULTHSU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MULTHSU
# endif
# define machine_is_multhsu()	(machine_arch_type == MACH_TYPE_MULTHSU)
#else
# define machine_is_multhsu()	(0)
#endif

#ifdef CONFIG_MACH_SALUDA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SALUDA
# endif
# define machine_is_saluda()	(machine_arch_type == MACH_TYPE_SALUDA)
#else
# define machine_is_saluda()	(0)
#endif

#ifdef CONFIG_MACH_PEMP_OMAP3_APOLLO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PEMP_OMAP3_APOLLO
# endif
# define machine_is_pemp_omap3_apollo()	(machine_arch_type == MACH_TYPE_PEMP_OMAP3_APOLLO)
#else
# define machine_is_pemp_omap3_apollo()	(0)
#endif

#ifdef CONFIG_MACH_VC0718
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VC0718
# endif
# define machine_is_vc0718()	(machine_arch_type == MACH_TYPE_VC0718)
#else
# define machine_is_vc0718()	(0)
#endif

#ifdef CONFIG_MACH_MVBLX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MVBLX
# endif
# define machine_is_mvblx()	(machine_arch_type == MACH_TYPE_MVBLX)
#else
# define machine_is_mvblx()	(0)
#endif

#ifdef CONFIG_MACH_INHAND_APEIRON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHAND_APEIRON
# endif
# define machine_is_inhand_apeiron()	(machine_arch_type == MACH_TYPE_INHAND_APEIRON)
#else
# define machine_is_inhand_apeiron()	(0)
#endif

#ifdef CONFIG_MACH_INHAND_FURY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHAND_FURY
# endif
# define machine_is_inhand_fury()	(machine_arch_type == MACH_TYPE_INHAND_FURY)
#else
# define machine_is_inhand_fury()	(0)
#endif

#ifdef CONFIG_MACH_INHAND_SIREN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_INHAND_SIREN
# endif
# define machine_is_inhand_siren()	(machine_arch_type == MACH_TYPE_INHAND_SIREN)
#else
# define machine_is_inhand_siren()	(0)
#endif

#ifdef CONFIG_MACH_HDNVP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HDNVP
# endif
# define machine_is_hdnvp()	(machine_arch_type == MACH_TYPE_HDNVP)
#else
# define machine_is_hdnvp()	(0)
#endif

#ifdef CONFIG_MACH_SOFTWINNER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SOFTWINNER
# endif
# define machine_is_softwinner()	(machine_arch_type == MACH_TYPE_SOFTWINNER)
#else
# define machine_is_softwinner()	(0)
#endif

#ifdef CONFIG_MACH_PRIMA2_EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PRIMA2_EVB
# endif
# define machine_is_prima2_evb()	(machine_arch_type == MACH_TYPE_PRIMA2_EVB)
#else
# define machine_is_prima2_evb()	(0)
#endif

#ifdef CONFIG_MACH_NAS6210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAS6210
# endif
# define machine_is_nas6210()	(machine_arch_type == MACH_TYPE_NAS6210)
#else
# define machine_is_nas6210()	(0)
#endif

#ifdef CONFIG_MACH_UNISDEV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNISDEV
# endif
# define machine_is_unisdev()	(machine_arch_type == MACH_TYPE_UNISDEV)
#else
# define machine_is_unisdev()	(0)
#endif

#ifdef CONFIG_MACH_SBCA11
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBCA11
# endif
# define machine_is_sbca11()	(machine_arch_type == MACH_TYPE_SBCA11)
#else
# define machine_is_sbca11()	(0)
#endif

#ifdef CONFIG_MACH_SAGA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAGA
# endif
# define machine_is_saga()	(machine_arch_type == MACH_TYPE_SAGA)
#else
# define machine_is_saga()	(0)
#endif

#ifdef CONFIG_MACH_NS_K330
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS_K330
# endif
# define machine_is_ns_k330()	(machine_arch_type == MACH_TYPE_NS_K330)
#else
# define machine_is_ns_k330()	(0)
#endif

#ifdef CONFIG_MACH_TANNA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TANNA
# endif
# define machine_is_tanna()	(machine_arch_type == MACH_TYPE_TANNA)
#else
# define machine_is_tanna()	(0)
#endif

#ifdef CONFIG_MACH_IMATE8502
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IMATE8502
# endif
# define machine_is_imate8502()	(machine_arch_type == MACH_TYPE_IMATE8502)
#else
# define machine_is_imate8502()	(0)
#endif

#ifdef CONFIG_MACH_ASPEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASPEN
# endif
# define machine_is_aspen()	(machine_arch_type == MACH_TYPE_ASPEN)
#else
# define machine_is_aspen()	(0)
#endif

#ifdef CONFIG_MACH_DAINTREE_CWAC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAINTREE_CWAC
# endif
# define machine_is_daintree_cwac()	(machine_arch_type == MACH_TYPE_DAINTREE_CWAC)
#else
# define machine_is_daintree_cwac()	(0)
#endif

#ifdef CONFIG_MACH_ZMX25
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ZMX25
# endif
# define machine_is_zmx25()	(machine_arch_type == MACH_TYPE_ZMX25)
#else
# define machine_is_zmx25()	(0)
#endif

#ifdef CONFIG_MACH_MAPLE1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAPLE1
# endif
# define machine_is_maple1()	(machine_arch_type == MACH_TYPE_MAPLE1)
#else
# define machine_is_maple1()	(0)
#endif

#ifdef CONFIG_MACH_QSD8X72_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QSD8X72_SURF
# endif
# define machine_is_qsd8x72_surf()	(machine_arch_type == MACH_TYPE_QSD8X72_SURF)
#else
# define machine_is_qsd8x72_surf()	(0)
#endif

#ifdef CONFIG_MACH_QSD8X72_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QSD8X72_FFA
# endif
# define machine_is_qsd8x72_ffa()	(machine_arch_type == MACH_TYPE_QSD8X72_FFA)
#else
# define machine_is_qsd8x72_ffa()	(0)
#endif

#ifdef CONFIG_MACH_ABILENE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ABILENE
# endif
# define machine_is_abilene()	(machine_arch_type == MACH_TYPE_ABILENE)
#else
# define machine_is_abilene()	(0)
#endif

#ifdef CONFIG_MACH_EIGEN_TTR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EIGEN_TTR
# endif
# define machine_is_eigen_ttr()	(machine_arch_type == MACH_TYPE_EIGEN_TTR)
#else
# define machine_is_eigen_ttr()	(0)
#endif

#ifdef CONFIG_MACH_IOMEGA_IX2_200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IOMEGA_IX2_200
# endif
# define machine_is_iomega_ix2_200()	(machine_arch_type == MACH_TYPE_IOMEGA_IX2_200)
#else
# define machine_is_iomega_ix2_200()	(0)
#endif

#ifdef CONFIG_MACH_CORETEC_VCX7400
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CORETEC_VCX7400
# endif
# define machine_is_coretec_vcx7400()	(machine_arch_type == MACH_TYPE_CORETEC_VCX7400)
#else
# define machine_is_coretec_vcx7400()	(0)
#endif

#ifdef CONFIG_MACH_SANTIAGO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SANTIAGO
# endif
# define machine_is_santiago()	(machine_arch_type == MACH_TYPE_SANTIAGO)
#else
# define machine_is_santiago()	(0)
#endif

#ifdef CONFIG_MACH_MX257SOL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX257SOL
# endif
# define machine_is_mx257sol()	(machine_arch_type == MACH_TYPE_MX257SOL)
#else
# define machine_is_mx257sol()	(0)
#endif

#ifdef CONFIG_MACH_STRASBOURG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STRASBOURG
# endif
# define machine_is_strasbourg()	(machine_arch_type == MACH_TYPE_STRASBOURG)
#else
# define machine_is_strasbourg()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_FLUID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_FLUID
# endif
# define machine_is_msm8x60_fluid()	(machine_arch_type == MACH_TYPE_MSM8X60_FLUID)
#else
# define machine_is_msm8x60_fluid()	(0)
#endif

#ifdef CONFIG_MACH_SMARTQV5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMARTQV5
# endif
# define machine_is_smartqv5()	(machine_arch_type == MACH_TYPE_SMARTQV5)
#else
# define machine_is_smartqv5()	(0)
#endif

#ifdef CONFIG_MACH_SMARTQV3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMARTQV3
# endif
# define machine_is_smartqv3()	(machine_arch_type == MACH_TYPE_SMARTQV3)
#else
# define machine_is_smartqv3()	(0)
#endif

#ifdef CONFIG_MACH_SMARTQV7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SMARTQV7
# endif
# define machine_is_smartqv7()	(machine_arch_type == MACH_TYPE_SMARTQV7)
#else
# define machine_is_smartqv7()	(0)
#endif

#ifdef CONFIG_MACH_PAZ00
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PAZ00
# endif
# define machine_is_paz00()	(machine_arch_type == MACH_TYPE_PAZ00)
#else
# define machine_is_paz00()	(0)
#endif

#ifdef CONFIG_MACH_ACMENETUSFOXG20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACMENETUSFOXG20
# endif
# define machine_is_acmenetusfoxg20()	(machine_arch_type == MACH_TYPE_ACMENETUSFOXG20)
#else
# define machine_is_acmenetusfoxg20()	(0)
#endif

#ifdef CONFIG_MACH_FWBD_0404
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FWBD_0404
# endif
# define machine_is_fwbd_0404()	(machine_arch_type == MACH_TYPE_FWBD_0404)
#else
# define machine_is_fwbd_0404()	(0)
#endif

#ifdef CONFIG_MACH_HDGU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HDGU
# endif
# define machine_is_hdgu()	(machine_arch_type == MACH_TYPE_HDGU)
#else
# define machine_is_hdgu()	(0)
#endif

#ifdef CONFIG_MACH_PYRAMID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PYRAMID
# endif
# define machine_is_pyramid()	(machine_arch_type == MACH_TYPE_PYRAMID)
#else
# define machine_is_pyramid()	(0)
#endif

#ifdef CONFIG_MACH_EPIPHAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EPIPHAN
# endif
# define machine_is_epiphan()	(machine_arch_type == MACH_TYPE_EPIPHAN)
#else
# define machine_is_epiphan()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_BENDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_BENDER
# endif
# define machine_is_omap_bender()	(machine_arch_type == MACH_TYPE_OMAP_BENDER)
#else
# define machine_is_omap_bender()	(0)
#endif

#ifdef CONFIG_MACH_GURNARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GURNARD
# endif
# define machine_is_gurnard()	(machine_arch_type == MACH_TYPE_GURNARD)
#else
# define machine_is_gurnard()	(0)
#endif

#ifdef CONFIG_MACH_GTL_IT5100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GTL_IT5100
# endif
# define machine_is_gtl_it5100()	(machine_arch_type == MACH_TYPE_GTL_IT5100)
#else
# define machine_is_gtl_it5100()	(0)
#endif

#ifdef CONFIG_MACH_BCM2708
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCM2708
# endif
# define machine_is_bcm2708()	(machine_arch_type == MACH_TYPE_BCM2708)
#else
# define machine_is_bcm2708()	(0)
#endif

#ifdef CONFIG_MACH_MX51_GGC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_GGC
# endif
# define machine_is_mx51_ggc()	(machine_arch_type == MACH_TYPE_MX51_GGC)
#else
# define machine_is_mx51_ggc()	(0)
#endif

#ifdef CONFIG_MACH_SHARESPACE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHARESPACE
# endif
# define machine_is_sharespace()	(machine_arch_type == MACH_TYPE_SHARESPACE)
#else
# define machine_is_sharespace()	(0)
#endif

#ifdef CONFIG_MACH_HABA_KNX_EXPLORER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HABA_KNX_EXPLORER
# endif
# define machine_is_haba_knx_explorer()	(machine_arch_type == MACH_TYPE_HABA_KNX_EXPLORER)
#else
# define machine_is_haba_knx_explorer()	(0)
#endif

#ifdef CONFIG_MACH_SIMTEC_KIRKMOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SIMTEC_KIRKMOD
# endif
# define machine_is_simtec_kirkmod()	(machine_arch_type == MACH_TYPE_SIMTEC_KIRKMOD)
#else
# define machine_is_simtec_kirkmod()	(0)
#endif

#ifdef CONFIG_MACH_CRUX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CRUX
# endif
# define machine_is_crux()	(machine_arch_type == MACH_TYPE_CRUX)
#else
# define machine_is_crux()	(0)
#endif

#ifdef CONFIG_MACH_MX51_BRAVO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_BRAVO
# endif
# define machine_is_mx51_bravo()	(machine_arch_type == MACH_TYPE_MX51_BRAVO)
#else
# define machine_is_mx51_bravo()	(0)
#endif

#ifdef CONFIG_MACH_CHARON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHARON
# endif
# define machine_is_charon()	(machine_arch_type == MACH_TYPE_CHARON)
#else
# define machine_is_charon()	(0)
#endif

#ifdef CONFIG_MACH_PICOCOM3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOCOM3
# endif
# define machine_is_picocom3()	(machine_arch_type == MACH_TYPE_PICOCOM3)
#else
# define machine_is_picocom3()	(0)
#endif

#ifdef CONFIG_MACH_PICOCOM4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICOCOM4
# endif
# define machine_is_picocom4()	(machine_arch_type == MACH_TYPE_PICOCOM4)
#else
# define machine_is_picocom4()	(0)
#endif

#ifdef CONFIG_MACH_SERRANO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SERRANO
# endif
# define machine_is_serrano()	(machine_arch_type == MACH_TYPE_SERRANO)
#else
# define machine_is_serrano()	(0)
#endif

#ifdef CONFIG_MACH_DOUBLESHOT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DOUBLESHOT
# endif
# define machine_is_doubleshot()	(machine_arch_type == MACH_TYPE_DOUBLESHOT)
#else
# define machine_is_doubleshot()	(0)
#endif

#ifdef CONFIG_MACH_EVSY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EVSY
# endif
# define machine_is_evsy()	(machine_arch_type == MACH_TYPE_EVSY)
#else
# define machine_is_evsy()	(0)
#endif

#ifdef CONFIG_MACH_HUASHAN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HUASHAN
# endif
# define machine_is_huashan()	(machine_arch_type == MACH_TYPE_HUASHAN)
#else
# define machine_is_huashan()	(0)
#endif

#ifdef CONFIG_MACH_LAUSANNE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LAUSANNE
# endif
# define machine_is_lausanne()	(machine_arch_type == MACH_TYPE_LAUSANNE)
#else
# define machine_is_lausanne()	(0)
#endif

#ifdef CONFIG_MACH_EMERALD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EMERALD
# endif
# define machine_is_emerald()	(machine_arch_type == MACH_TYPE_EMERALD)
#else
# define machine_is_emerald()	(0)
#endif

#ifdef CONFIG_MACH_TQMA35
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TQMA35
# endif
# define machine_is_tqma35()	(machine_arch_type == MACH_TYPE_TQMA35)
#else
# define machine_is_tqma35()	(0)
#endif

#ifdef CONFIG_MACH_MARVEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARVEL
# endif
# define machine_is_marvel()	(machine_arch_type == MACH_TYPE_MARVEL)
#else
# define machine_is_marvel()	(0)
#endif

#ifdef CONFIG_MACH_MANUAE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MANUAE
# endif
# define machine_is_manuae()	(machine_arch_type == MACH_TYPE_MANUAE)
#else
# define machine_is_manuae()	(0)
#endif

#ifdef CONFIG_MACH_CHACHA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHACHA
# endif
# define machine_is_chacha()	(machine_arch_type == MACH_TYPE_CHACHA)
#else
# define machine_is_chacha()	(0)
#endif

#ifdef CONFIG_MACH_LEMON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LEMON
# endif
# define machine_is_lemon()	(machine_arch_type == MACH_TYPE_LEMON)
#else
# define machine_is_lemon()	(0)
#endif

#ifdef CONFIG_MACH_CSC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CSC
# endif
# define machine_is_csc()	(machine_arch_type == MACH_TYPE_CSC)
#else
# define machine_is_csc()	(0)
#endif

#ifdef CONFIG_MACH_GIRA_KNXIP_ROUTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GIRA_KNXIP_ROUTER
# endif
# define machine_is_gira_knxip_router()	(machine_arch_type == MACH_TYPE_GIRA_KNXIP_ROUTER)
#else
# define machine_is_gira_knxip_router()	(0)
#endif

#ifdef CONFIG_MACH_T20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T20
# endif
# define machine_is_t20()	(machine_arch_type == MACH_TYPE_T20)
#else
# define machine_is_t20()	(0)
#endif

#ifdef CONFIG_MACH_HDMINI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HDMINI
# endif
# define machine_is_hdmini()	(machine_arch_type == MACH_TYPE_HDMINI)
#else
# define machine_is_hdmini()	(0)
#endif

#ifdef CONFIG_MACH_SCIPHONE_G2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SCIPHONE_G2
# endif
# define machine_is_sciphone_g2()	(machine_arch_type == MACH_TYPE_SCIPHONE_G2)
#else
# define machine_is_sciphone_g2()	(0)
#endif

#ifdef CONFIG_MACH_EXPRESS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPRESS
# endif
# define machine_is_express()	(machine_arch_type == MACH_TYPE_EXPRESS)
#else
# define machine_is_express()	(0)
#endif

#ifdef CONFIG_MACH_EXPRESS_KT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPRESS_KT
# endif
# define machine_is_express_kt()	(machine_arch_type == MACH_TYPE_EXPRESS_KT)
#else
# define machine_is_express_kt()	(0)
#endif

#ifdef CONFIG_MACH_MAXIMASP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MAXIMASP
# endif
# define machine_is_maximasp()	(machine_arch_type == MACH_TYPE_MAXIMASP)
#else
# define machine_is_maximasp()	(0)
#endif

#ifdef CONFIG_MACH_NITROGEN_IMX51
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NITROGEN_IMX51
# endif
# define machine_is_nitrogen_imx51()	(machine_arch_type == MACH_TYPE_NITROGEN_IMX51)
#else
# define machine_is_nitrogen_imx51()	(0)
#endif

#ifdef CONFIG_MACH_NITROGEN_IMX53
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NITROGEN_IMX53
# endif
# define machine_is_nitrogen_imx53()	(machine_arch_type == MACH_TYPE_NITROGEN_IMX53)
#else
# define machine_is_nitrogen_imx53()	(0)
#endif

#ifdef CONFIG_MACH_SUNFIRE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SUNFIRE
# endif
# define machine_is_sunfire()	(machine_arch_type == MACH_TYPE_SUNFIRE)
#else
# define machine_is_sunfire()	(0)
#endif

#ifdef CONFIG_MACH_AROWANA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AROWANA
# endif
# define machine_is_arowana()	(machine_arch_type == MACH_TYPE_AROWANA)
#else
# define machine_is_arowana()	(0)
#endif

#ifdef CONFIG_MACH_TEGRA_DAYTONA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEGRA_DAYTONA
# endif
# define machine_is_tegra_daytona()	(machine_arch_type == MACH_TYPE_TEGRA_DAYTONA)
#else
# define machine_is_tegra_daytona()	(0)
#endif

#ifdef CONFIG_MACH_TEGRA_SWORDFISH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TEGRA_SWORDFISH
# endif
# define machine_is_tegra_swordfish()	(machine_arch_type == MACH_TYPE_TEGRA_SWORDFISH)
#else
# define machine_is_tegra_swordfish()	(0)
#endif

#ifdef CONFIG_MACH_EDISON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EDISON
# endif
# define machine_is_edison()	(machine_arch_type == MACH_TYPE_EDISON)
#else
# define machine_is_edison()	(0)
#endif

#ifdef CONFIG_MACH_SVP8500V1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVP8500V1
# endif
# define machine_is_svp8500v1()	(machine_arch_type == MACH_TYPE_SVP8500V1)
#else
# define machine_is_svp8500v1()	(0)
#endif

#ifdef CONFIG_MACH_SVP8500V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVP8500V2
# endif
# define machine_is_svp8500v2()	(machine_arch_type == MACH_TYPE_SVP8500V2)
#else
# define machine_is_svp8500v2()	(0)
#endif

#ifdef CONFIG_MACH_SVP5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVP5500
# endif
# define machine_is_svp5500()	(machine_arch_type == MACH_TYPE_SVP5500)
#else
# define machine_is_svp5500()	(0)
#endif

#ifdef CONFIG_MACH_B5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_B5500
# endif
# define machine_is_b5500()	(machine_arch_type == MACH_TYPE_B5500)
#else
# define machine_is_b5500()	(0)
#endif

#ifdef CONFIG_MACH_S5500
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_S5500
# endif
# define machine_is_s5500()	(machine_arch_type == MACH_TYPE_S5500)
#else
# define machine_is_s5500()	(0)
#endif

#ifdef CONFIG_MACH_ICON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICON
# endif
# define machine_is_icon()	(machine_arch_type == MACH_TYPE_ICON)
#else
# define machine_is_icon()	(0)
#endif

#ifdef CONFIG_MACH_ELEPHANT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELEPHANT
# endif
# define machine_is_elephant()	(machine_arch_type == MACH_TYPE_ELEPHANT)
#else
# define machine_is_elephant()	(0)
#endif

#ifdef CONFIG_MACH_SHOOTER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHOOTER
# endif
# define machine_is_shooter()	(machine_arch_type == MACH_TYPE_SHOOTER)
#else
# define machine_is_shooter()	(0)
#endif

#ifdef CONFIG_MACH_SPADE_LTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPADE_LTE
# endif
# define machine_is_spade_lte()	(machine_arch_type == MACH_TYPE_SPADE_LTE)
#else
# define machine_is_spade_lte()	(0)
#endif

#ifdef CONFIG_MACH_PHILHWANI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PHILHWANI
# endif
# define machine_is_philhwani()	(machine_arch_type == MACH_TYPE_PHILHWANI)
#else
# define machine_is_philhwani()	(0)
#endif

#ifdef CONFIG_MACH_GSNCOMM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GSNCOMM
# endif
# define machine_is_gsncomm()	(machine_arch_type == MACH_TYPE_GSNCOMM)
#else
# define machine_is_gsncomm()	(0)
#endif

#ifdef CONFIG_MACH_STRASBOURG_A2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STRASBOURG_A2
# endif
# define machine_is_strasbourg_a2()	(machine_arch_type == MACH_TYPE_STRASBOURG_A2)
#else
# define machine_is_strasbourg_a2()	(0)
#endif

#ifdef CONFIG_MACH_MMM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MMM
# endif
# define machine_is_mmm()	(machine_arch_type == MACH_TYPE_MMM)
#else
# define machine_is_mmm()	(0)
#endif

#ifdef CONFIG_MACH_DAVINCI_DM365_BV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAVINCI_DM365_BV
# endif
# define machine_is_davinci_dm365_bv()	(machine_arch_type == MACH_TYPE_DAVINCI_DM365_BV)
#else
# define machine_is_davinci_dm365_bv()	(0)
#endif

#ifdef CONFIG_MACH_AG5EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AG5EVM
# endif
# define machine_is_ag5evm()	(machine_arch_type == MACH_TYPE_AG5EVM)
#else
# define machine_is_ag5evm()	(0)
#endif

#ifdef CONFIG_MACH_SC575PLC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SC575PLC
# endif
# define machine_is_sc575plc()	(machine_arch_type == MACH_TYPE_SC575PLC)
#else
# define machine_is_sc575plc()	(0)
#endif

#ifdef CONFIG_MACH_SC575IPC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SC575IPC
# endif
# define machine_is_sc575hmi()	(machine_arch_type == MACH_TYPE_SC575IPC)
#else
# define machine_is_sc575hmi()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_TDM3730
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_TDM3730
# endif
# define machine_is_omap3_tdm3730()	(machine_arch_type == MACH_TYPE_OMAP3_TDM3730)
#else
# define machine_is_omap3_tdm3730()	(0)
#endif

#ifdef CONFIG_MACH_TOP9000_EVAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOP9000_EVAL
# endif
# define machine_is_top9000_eval()	(machine_arch_type == MACH_TYPE_TOP9000_EVAL)
#else
# define machine_is_top9000_eval()	(0)
#endif

#ifdef CONFIG_MACH_TOP9000_SU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOP9000_SU
# endif
# define machine_is_top9000_su()	(machine_arch_type == MACH_TYPE_TOP9000_SU)
#else
# define machine_is_top9000_su()	(0)
#endif

#ifdef CONFIG_MACH_UTM300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UTM300
# endif
# define machine_is_utm300()	(machine_arch_type == MACH_TYPE_UTM300)
#else
# define machine_is_utm300()	(0)
#endif

#ifdef CONFIG_MACH_TSUNAGI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TSUNAGI
# endif
# define machine_is_tsunagi()	(machine_arch_type == MACH_TYPE_TSUNAGI)
#else
# define machine_is_tsunagi()	(0)
#endif

#ifdef CONFIG_MACH_TS75XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS75XX
# endif
# define machine_is_ts75xx()	(machine_arch_type == MACH_TYPE_TS75XX)
#else
# define machine_is_ts75xx()	(0)
#endif

#ifdef CONFIG_MACH_TS47XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS47XX
# endif
# define machine_is_ts47xx()	(machine_arch_type == MACH_TYPE_TS47XX)
#else
# define machine_is_ts47xx()	(0)
#endif

#ifdef CONFIG_MACH_DA850_K5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DA850_K5
# endif
# define machine_is_da850_k5()	(machine_arch_type == MACH_TYPE_DA850_K5)
#else
# define machine_is_da850_k5()	(0)
#endif

#ifdef CONFIG_MACH_AX502
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AX502
# endif
# define machine_is_ax502()	(machine_arch_type == MACH_TYPE_AX502)
#else
# define machine_is_ax502()	(0)
#endif

#ifdef CONFIG_MACH_IGEP0032
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IGEP0032
# endif
# define machine_is_igep0032()	(machine_arch_type == MACH_TYPE_IGEP0032)
#else
# define machine_is_igep0032()	(0)
#endif

#ifdef CONFIG_MACH_ANTERO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANTERO
# endif
# define machine_is_antero()	(machine_arch_type == MACH_TYPE_ANTERO)
#else
# define machine_is_antero()	(0)
#endif

#ifdef CONFIG_MACH_SYNERGY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SYNERGY
# endif
# define machine_is_synergy()	(machine_arch_type == MACH_TYPE_SYNERGY)
#else
# define machine_is_synergy()	(0)
#endif

#ifdef CONFIG_MACH_ICS_IF_VOIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICS_IF_VOIP
# endif
# define machine_is_ics_if_voip()	(machine_arch_type == MACH_TYPE_ICS_IF_VOIP)
#else
# define machine_is_ics_if_voip()	(0)
#endif

#ifdef CONFIG_MACH_WLF_CRAGG_6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WLF_CRAGG_6410
# endif
# define machine_is_wlf_cragg_6410()	(machine_arch_type == MACH_TYPE_WLF_CRAGG_6410)
#else
# define machine_is_wlf_cragg_6410()	(0)
#endif

#ifdef CONFIG_MACH_PUNICA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PUNICA
# endif
# define machine_is_punica()	(machine_arch_type == MACH_TYPE_PUNICA)
#else
# define machine_is_punica()	(0)
#endif

#ifdef CONFIG_MACH_TRIMSLICE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIMSLICE
# endif
# define machine_is_trimslice()	(machine_arch_type == MACH_TYPE_TRIMSLICE)
#else
# define machine_is_trimslice()	(0)
#endif

#ifdef CONFIG_MACH_MX27_WMULTRA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX27_WMULTRA
# endif
# define machine_is_mx27_wmultra()	(machine_arch_type == MACH_TYPE_MX27_WMULTRA)
#else
# define machine_is_mx27_wmultra()	(0)
#endif

#ifdef CONFIG_MACH_MACKEREL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MACKEREL
# endif
# define machine_is_mackerel()	(machine_arch_type == MACH_TYPE_MACKEREL)
#else
# define machine_is_mackerel()	(0)
#endif

#ifdef CONFIG_MACH_FA9X27
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FA9X27
# endif
# define machine_is_fa9x27()	(machine_arch_type == MACH_TYPE_FA9X27)
#else
# define machine_is_fa9x27()	(0)
#endif

#ifdef CONFIG_MACH_NS2816TB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS2816TB
# endif
# define machine_is_ns2816tb()	(machine_arch_type == MACH_TYPE_NS2816TB)
#else
# define machine_is_ns2816tb()	(0)
#endif

#ifdef CONFIG_MACH_NS2816_NTPAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS2816_NTPAD
# endif
# define machine_is_ns2816_ntpad()	(machine_arch_type == MACH_TYPE_NS2816_NTPAD)
#else
# define machine_is_ns2816_ntpad()	(0)
#endif

#ifdef CONFIG_MACH_NS2816_NTNB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NS2816_NTNB
# endif
# define machine_is_ns2816_ntnb()	(machine_arch_type == MACH_TYPE_NS2816_NTNB)
#else
# define machine_is_ns2816_ntnb()	(0)
#endif

#ifdef CONFIG_MACH_KAEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KAEN
# endif
# define machine_is_kaen()	(machine_arch_type == MACH_TYPE_KAEN)
#else
# define machine_is_kaen()	(0)
#endif

#ifdef CONFIG_MACH_NV1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NV1000
# endif
# define machine_is_nv1000()	(machine_arch_type == MACH_TYPE_NV1000)
#else
# define machine_is_nv1000()	(0)
#endif

#ifdef CONFIG_MACH_NUC950TS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC950TS
# endif
# define machine_is_nuc950ts()	(machine_arch_type == MACH_TYPE_NUC950TS)
#else
# define machine_is_nuc950ts()	(0)
#endif

#ifdef CONFIG_MACH_NOKIA_RM680
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOKIA_RM680
# endif
# define machine_is_nokia_rm680()	(machine_arch_type == MACH_TYPE_NOKIA_RM680)
#else
# define machine_is_nokia_rm680()	(0)
#endif

#ifdef CONFIG_MACH_AST2200
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AST2200
# endif
# define machine_is_ast2200()	(machine_arch_type == MACH_TYPE_AST2200)
#else
# define machine_is_ast2200()	(0)
#endif

#ifdef CONFIG_MACH_LEAD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LEAD
# endif
# define machine_is_lead()	(machine_arch_type == MACH_TYPE_LEAD)
#else
# define machine_is_lead()	(0)
#endif

#ifdef CONFIG_MACH_UNINO1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UNINO1
# endif
# define machine_is_unino1()	(machine_arch_type == MACH_TYPE_UNINO1)
#else
# define machine_is_unino1()	(0)
#endif

#ifdef CONFIG_MACH_GREECO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GREECO
# endif
# define machine_is_greeco()	(machine_arch_type == MACH_TYPE_GREECO)
#else
# define machine_is_greeco()	(0)
#endif

#ifdef CONFIG_MACH_VERDI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERDI
# endif
# define machine_is_verdi()	(machine_arch_type == MACH_TYPE_VERDI)
#else
# define machine_is_verdi()	(0)
#endif

#ifdef CONFIG_MACH_DM6446_ADBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM6446_ADBOX
# endif
# define machine_is_dm6446_adbox()	(machine_arch_type == MACH_TYPE_DM6446_ADBOX)
#else
# define machine_is_dm6446_adbox()	(0)
#endif

#ifdef CONFIG_MACH_QUAD_SALSA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_QUAD_SALSA
# endif
# define machine_is_quad_salsa()	(machine_arch_type == MACH_TYPE_QUAD_SALSA)
#else
# define machine_is_quad_salsa()	(0)
#endif

#ifdef CONFIG_MACH_ABB_GMA_1_1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ABB_GMA_1_1
# endif
# define machine_is_abb_gma_1_1()	(machine_arch_type == MACH_TYPE_ABB_GMA_1_1)
#else
# define machine_is_abb_gma_1_1()	(0)
#endif

#ifdef CONFIG_MACH_SVCID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SVCID
# endif
# define machine_is_svcid()	(machine_arch_type == MACH_TYPE_SVCID)
#else
# define machine_is_svcid()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_SIM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_SIM
# endif
# define machine_is_msm8960_sim()	(machine_arch_type == MACH_TYPE_MSM8960_SIM)
#else
# define machine_is_msm8960_sim()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_RUMI3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_RUMI3
# endif
# define machine_is_msm8960_rumi3()	(machine_arch_type == MACH_TYPE_MSM8960_RUMI3)
#else
# define machine_is_msm8960_rumi3()	(0)
#endif

#ifdef CONFIG_MACH_ICON_G
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICON_G
# endif
# define machine_is_icon_g()	(machine_arch_type == MACH_TYPE_ICON_G)
#else
# define machine_is_icon_g()	(0)
#endif

#ifdef CONFIG_MACH_MB3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MB3
# endif
# define machine_is_mb3()	(machine_arch_type == MACH_TYPE_MB3)
#else
# define machine_is_mb3()	(0)
#endif

#ifdef CONFIG_MACH_GSIA18S
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GSIA18S
# endif
# define machine_is_gsia18s()	(machine_arch_type == MACH_TYPE_GSIA18S)
#else
# define machine_is_gsia18s()	(0)
#endif

#ifdef CONFIG_MACH_PIVICC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PIVICC
# endif
# define machine_is_pivicc()	(machine_arch_type == MACH_TYPE_PIVICC)
#else
# define machine_is_pivicc()	(0)
#endif

#ifdef CONFIG_MACH_PCM048
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM048
# endif
# define machine_is_pcm048()	(machine_arch_type == MACH_TYPE_PCM048)
#else
# define machine_is_pcm048()	(0)
#endif

#ifdef CONFIG_MACH_DDS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DDS
# endif
# define machine_is_dds()	(machine_arch_type == MACH_TYPE_DDS)
#else
# define machine_is_dds()	(0)
#endif

#ifdef CONFIG_MACH_CHALTEN_XA1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CHALTEN_XA1
# endif
# define machine_is_chalten_xa1()	(machine_arch_type == MACH_TYPE_CHALTEN_XA1)
#else
# define machine_is_chalten_xa1()	(0)
#endif

#ifdef CONFIG_MACH_TS48XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS48XX
# endif
# define machine_is_ts48xx()	(machine_arch_type == MACH_TYPE_TS48XX)
#else
# define machine_is_ts48xx()	(0)
#endif

#ifdef CONFIG_MACH_TONGA2_TFTTIMER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TONGA2_TFTTIMER
# endif
# define machine_is_tonga2_tfttimer()	(machine_arch_type == MACH_TYPE_TONGA2_TFTTIMER)
#else
# define machine_is_tonga2_tfttimer()	(0)
#endif

#ifdef CONFIG_MACH_WHISTLER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WHISTLER
# endif
# define machine_is_whistler()	(machine_arch_type == MACH_TYPE_WHISTLER)
#else
# define machine_is_whistler()	(0)
#endif

#ifdef CONFIG_MACH_ASL_PHOENIX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ASL_PHOENIX
# endif
# define machine_is_asl_phoenix()	(machine_arch_type == MACH_TYPE_ASL_PHOENIX)
#else
# define machine_is_asl_phoenix()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9263OTLITE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9263OTLITE
# endif
# define machine_is_at91sam9263otlite()	(machine_arch_type == MACH_TYPE_AT91SAM9263OTLITE)
#else
# define machine_is_at91sam9263otlite()	(0)
#endif

#ifdef CONFIG_MACH_DDPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DDPLUG
# endif
# define machine_is_ddplug()	(machine_arch_type == MACH_TYPE_DDPLUG)
#else
# define machine_is_ddplug()	(0)
#endif

#ifdef CONFIG_MACH_D2PLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_D2PLUG
# endif
# define machine_is_d2plug()	(machine_arch_type == MACH_TYPE_D2PLUG)
#else
# define machine_is_d2plug()	(0)
#endif

#ifdef CONFIG_MACH_KZM9D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KZM9D
# endif
# define machine_is_kzm9d()	(machine_arch_type == MACH_TYPE_KZM9D)
#else
# define machine_is_kzm9d()	(0)
#endif

#ifdef CONFIG_MACH_VERDI_LTE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERDI_LTE
# endif
# define machine_is_verdi_lte()	(machine_arch_type == MACH_TYPE_VERDI_LTE)
#else
# define machine_is_verdi_lte()	(0)
#endif

#ifdef CONFIG_MACH_NANOZOOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NANOZOOM
# endif
# define machine_is_nanozoom()	(machine_arch_type == MACH_TYPE_NANOZOOM)
#else
# define machine_is_nanozoom()	(0)
#endif

#ifdef CONFIG_MACH_DM3730_SOM_LV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM3730_SOM_LV
# endif
# define machine_is_dm3730_som_lv()	(machine_arch_type == MACH_TYPE_DM3730_SOM_LV)
#else
# define machine_is_dm3730_som_lv()	(0)
#endif

#ifdef CONFIG_MACH_DM3730_TORPEDO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM3730_TORPEDO
# endif
# define machine_is_dm3730_torpedo()	(machine_arch_type == MACH_TYPE_DM3730_TORPEDO)
#else
# define machine_is_dm3730_torpedo()	(0)
#endif

#ifdef CONFIG_MACH_ANCHOVY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ANCHOVY
# endif
# define machine_is_anchovy()	(machine_arch_type == MACH_TYPE_ANCHOVY)
#else
# define machine_is_anchovy()	(0)
#endif

#ifdef CONFIG_MACH_RE2REV20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RE2REV20
# endif
# define machine_is_re2rev20()	(machine_arch_type == MACH_TYPE_RE2REV20)
#else
# define machine_is_re2rev20()	(0)
#endif

#ifdef CONFIG_MACH_RE2REV21
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RE2REV21
# endif
# define machine_is_re2rev21()	(machine_arch_type == MACH_TYPE_RE2REV21)
#else
# define machine_is_re2rev21()	(0)
#endif

#ifdef CONFIG_MACH_CNS21XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CNS21XX
# endif
# define machine_is_cns21xx()	(machine_arch_type == MACH_TYPE_CNS21XX)
#else
# define machine_is_cns21xx()	(0)
#endif

#ifdef CONFIG_MACH_RIDER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RIDER
# endif
# define machine_is_rider()	(machine_arch_type == MACH_TYPE_RIDER)
#else
# define machine_is_rider()	(0)
#endif

#ifdef CONFIG_MACH_NSK330
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSK330
# endif
# define machine_is_nsk330()	(machine_arch_type == MACH_TYPE_NSK330)
#else
# define machine_is_nsk330()	(0)
#endif

#ifdef CONFIG_MACH_CNS2133EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CNS2133EVB
# endif
# define machine_is_cns2133evb()	(machine_arch_type == MACH_TYPE_CNS2133EVB)
#else
# define machine_is_cns2133evb()	(0)
#endif

#ifdef CONFIG_MACH_Z3_816X_MOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_Z3_816X_MOD
# endif
# define machine_is_z3_816x_mod()	(machine_arch_type == MACH_TYPE_Z3_816X_MOD)
#else
# define machine_is_z3_816x_mod()	(0)
#endif

#ifdef CONFIG_MACH_Z3_814X_MOD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_Z3_814X_MOD
# endif
# define machine_is_z3_814x_mod()	(machine_arch_type == MACH_TYPE_Z3_814X_MOD)
#else
# define machine_is_z3_814x_mod()	(0)
#endif

#ifdef CONFIG_MACH_BEECT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BEECT
# endif
# define machine_is_beect()	(machine_arch_type == MACH_TYPE_BEECT)
#else
# define machine_is_beect()	(0)
#endif

#ifdef CONFIG_MACH_DMA_THUNDERBUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DMA_THUNDERBUG
# endif
# define machine_is_dma_thunderbug()	(machine_arch_type == MACH_TYPE_DMA_THUNDERBUG)
#else
# define machine_is_dma_thunderbug()	(0)
#endif

#ifdef CONFIG_MACH_OMN_AT91SAM9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMN_AT91SAM9G20
# endif
# define machine_is_omn_at91sam9g20()	(machine_arch_type == MACH_TYPE_OMN_AT91SAM9G20)
#else
# define machine_is_omn_at91sam9g20()	(0)
#endif

#ifdef CONFIG_MACH_MX25_E2S_UC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX25_E2S_UC
# endif
# define machine_is_mx25_e2s_uc()	(machine_arch_type == MACH_TYPE_MX25_E2S_UC)
#else
# define machine_is_mx25_e2s_uc()	(0)
#endif

#ifdef CONFIG_MACH_MIONE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIONE
# endif
# define machine_is_mione()	(machine_arch_type == MACH_TYPE_MIONE)
#else
# define machine_is_mione()	(0)
#endif

#ifdef CONFIG_MACH_TOP9000_TCU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOP9000_TCU
# endif
# define machine_is_top9000_tcu()	(machine_arch_type == MACH_TYPE_TOP9000_TCU)
#else
# define machine_is_top9000_tcu()	(0)
#endif

#ifdef CONFIG_MACH_TOP9000_BSL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TOP9000_BSL
# endif
# define machine_is_top9000_bsl()	(machine_arch_type == MACH_TYPE_TOP9000_BSL)
#else
# define machine_is_top9000_bsl()	(0)
#endif

#ifdef CONFIG_MACH_KINGDOM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KINGDOM
# endif
# define machine_is_kingdom()	(machine_arch_type == MACH_TYPE_KINGDOM)
#else
# define machine_is_kingdom()	(0)
#endif

#ifdef CONFIG_MACH_ARMADILLO460
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMADILLO460
# endif
# define machine_is_armadillo460()	(machine_arch_type == MACH_TYPE_ARMADILLO460)
#else
# define machine_is_armadillo460()	(0)
#endif

#ifdef CONFIG_MACH_LQ2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LQ2
# endif
# define machine_is_lq2()	(machine_arch_type == MACH_TYPE_LQ2)
#else
# define machine_is_lq2()	(0)
#endif

#ifdef CONFIG_MACH_SWEDA_TMS2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWEDA_TMS2
# endif
# define machine_is_sweda_tms2()	(machine_arch_type == MACH_TYPE_SWEDA_TMS2)
#else
# define machine_is_sweda_tms2()	(0)
#endif

#ifdef CONFIG_MACH_MX53_LOCO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX53_LOCO
# endif
# define machine_is_mx53_loco()	(machine_arch_type == MACH_TYPE_MX53_LOCO)
#else
# define machine_is_mx53_loco()	(0)
#endif

#ifdef CONFIG_MACH_ACER_A8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACER_A8
# endif
# define machine_is_acer_a8()	(machine_arch_type == MACH_TYPE_ACER_A8)
#else
# define machine_is_acer_a8()	(0)
#endif

#ifdef CONFIG_MACH_ACER_GAUGUIN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACER_GAUGUIN
# endif
# define machine_is_acer_gauguin()	(machine_arch_type == MACH_TYPE_ACER_GAUGUIN)
#else
# define machine_is_acer_gauguin()	(0)
#endif

#ifdef CONFIG_MACH_GUPPY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GUPPY
# endif
# define machine_is_guppy()	(machine_arch_type == MACH_TYPE_GUPPY)
#else
# define machine_is_guppy()	(0)
#endif

#ifdef CONFIG_MACH_MX61_ARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX61_ARD
# endif
# define machine_is_mx61_ard()	(machine_arch_type == MACH_TYPE_MX61_ARD)
#else
# define machine_is_mx61_ard()	(0)
#endif

#ifdef CONFIG_MACH_TX53
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TX53
# endif
# define machine_is_tx53()	(machine_arch_type == MACH_TYPE_TX53)
#else
# define machine_is_tx53()	(0)
#endif

#ifdef CONFIG_MACH_OMAPL138_CASE_A3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAPL138_CASE_A3
# endif
# define machine_is_omapl138_case_a3()	(machine_arch_type == MACH_TYPE_OMAPL138_CASE_A3)
#else
# define machine_is_omapl138_case_a3()	(0)
#endif

#ifdef CONFIG_MACH_UEMD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UEMD
# endif
# define machine_is_uemd()	(machine_arch_type == MACH_TYPE_UEMD)
#else
# define machine_is_uemd()	(0)
#endif

#ifdef CONFIG_MACH_CCWMX51MUT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCWMX51MUT
# endif
# define machine_is_ccwmx51mut()	(machine_arch_type == MACH_TYPE_CCWMX51MUT)
#else
# define machine_is_ccwmx51mut()	(0)
#endif

#ifdef CONFIG_MACH_ROCKHOPPER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROCKHOPPER
# endif
# define machine_is_rockhopper()	(machine_arch_type == MACH_TYPE_ROCKHOPPER)
#else
# define machine_is_rockhopper()	(0)
#endif

#ifdef CONFIG_MACH_ENCORE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENCORE
# endif
# define machine_is_encore()	(machine_arch_type == MACH_TYPE_ENCORE)
#else
# define machine_is_encore()	(0)
#endif

#ifdef CONFIG_MACH_HKDKC100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HKDKC100
# endif
# define machine_is_hkdkc100()	(machine_arch_type == MACH_TYPE_HKDKC100)
#else
# define machine_is_hkdkc100()	(0)
#endif

#ifdef CONFIG_MACH_TS42XX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS42XX
# endif
# define machine_is_ts42xx()	(machine_arch_type == MACH_TYPE_TS42XX)
#else
# define machine_is_ts42xx()	(0)
#endif

#ifdef CONFIG_MACH_AEBL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AEBL
# endif
# define machine_is_aebl()	(machine_arch_type == MACH_TYPE_AEBL)
#else
# define machine_is_aebl()	(0)
#endif

#ifdef CONFIG_MACH_WARIO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WARIO
# endif
# define machine_is_wario()	(machine_arch_type == MACH_TYPE_WARIO)
#else
# define machine_is_wario()	(0)
#endif

#ifdef CONFIG_MACH_GFS_SPM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GFS_SPM
# endif
# define machine_is_gfs_spm()	(machine_arch_type == MACH_TYPE_GFS_SPM)
#else
# define machine_is_gfs_spm()	(0)
#endif

#ifdef CONFIG_MACH_CM_T3730
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CM_T3730
# endif
# define machine_is_cm_t3730()	(machine_arch_type == MACH_TYPE_CM_T3730)
#else
# define machine_is_cm_t3730()	(0)
#endif

#ifdef CONFIG_MACH_ISC3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ISC3
# endif
# define machine_is_isc3()	(machine_arch_type == MACH_TYPE_ISC3)
#else
# define machine_is_isc3()	(0)
#endif

#ifdef CONFIG_MACH_RASCAL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RASCAL
# endif
# define machine_is_rascal()	(machine_arch_type == MACH_TYPE_RASCAL)
#else
# define machine_is_rascal()	(0)
#endif

#ifdef CONFIG_MACH_HREFV60
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HREFV60
# endif
# define machine_is_hrefv60()	(machine_arch_type == MACH_TYPE_HREFV60)
#else
# define machine_is_hrefv60()	(0)
#endif

#ifdef CONFIG_MACH_TPT_2_0
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TPT_2_0
# endif
# define machine_is_tpt_2_0()	(machine_arch_type == MACH_TYPE_TPT_2_0)
#else
# define machine_is_tpt_2_0()	(0)
#endif

#ifdef CONFIG_MACH_SPLENDOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPLENDOR
# endif
# define machine_is_splendor()	(machine_arch_type == MACH_TYPE_SPLENDOR)
#else
# define machine_is_splendor()	(0)
#endif

#ifdef CONFIG_MACH_MSM8X60_QT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8X60_QT
# endif
# define machine_is_msm8x60_qt()	(machine_arch_type == MACH_TYPE_MSM8X60_QT)
#else
# define machine_is_msm8x60_qt()	(0)
#endif

#ifdef CONFIG_MACH_HTC_HD_MINI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HTC_HD_MINI
# endif
# define machine_is_htc_hd_mini()	(machine_arch_type == MACH_TYPE_HTC_HD_MINI)
#else
# define machine_is_htc_hd_mini()	(0)
#endif

#ifdef CONFIG_MACH_ATHENE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATHENE
# endif
# define machine_is_athene()	(machine_arch_type == MACH_TYPE_ATHENE)
#else
# define machine_is_athene()	(0)
#endif

#ifdef CONFIG_MACH_DEEP_R_EK_1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DEEP_R_EK_1
# endif
# define machine_is_deep_r_ek_1()	(machine_arch_type == MACH_TYPE_DEEP_R_EK_1)
#else
# define machine_is_deep_r_ek_1()	(0)
#endif

#ifdef CONFIG_MACH_VIVOW_CT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIVOW_CT
# endif
# define machine_is_vivow_ct()	(machine_arch_type == MACH_TYPE_VIVOW_CT)
#else
# define machine_is_vivow_ct()	(0)
#endif

#ifdef CONFIG_MACH_NERY_1000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NERY_1000
# endif
# define machine_is_nery_1000()	(machine_arch_type == MACH_TYPE_NERY_1000)
#else
# define machine_is_nery_1000()	(0)
#endif

#ifdef CONFIG_MACH_RFL109145_SSRV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RFL109145_SSRV
# endif
# define machine_is_rfl109145_ssrv()	(machine_arch_type == MACH_TYPE_RFL109145_SSRV)
#else
# define machine_is_rfl109145_ssrv()	(0)
#endif

#ifdef CONFIG_MACH_NMH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NMH
# endif
# define machine_is_nmh()	(machine_arch_type == MACH_TYPE_NMH)
#else
# define machine_is_nmh()	(0)
#endif

#ifdef CONFIG_MACH_WN802T
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WN802T
# endif
# define machine_is_wn802t()	(machine_arch_type == MACH_TYPE_WN802T)
#else
# define machine_is_wn802t()	(0)
#endif

#ifdef CONFIG_MACH_DRAGONET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DRAGONET
# endif
# define machine_is_dragonet()	(machine_arch_type == MACH_TYPE_DRAGONET)
#else
# define machine_is_dragonet()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9263DESK16L
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9263DESK16L
# endif
# define machine_is_at91sam9263desk16l()	(machine_arch_type == MACH_TYPE_AT91SAM9263DESK16L)
#else
# define machine_is_at91sam9263desk16l()	(0)
#endif

#ifdef CONFIG_MACH_BCMHANA_SV
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCMHANA_SV
# endif
# define machine_is_bcmhana_sv()	(machine_arch_type == MACH_TYPE_BCMHANA_SV)
#else
# define machine_is_bcmhana_sv()	(0)
#endif

#ifdef CONFIG_MACH_BCMHANA_TABLET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BCMHANA_TABLET
# endif
# define machine_is_bcmhana_tablet()	(machine_arch_type == MACH_TYPE_BCMHANA_TABLET)
#else
# define machine_is_bcmhana_tablet()	(0)
#endif

#ifdef CONFIG_MACH_KOI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KOI
# endif
# define machine_is_koi()	(machine_arch_type == MACH_TYPE_KOI)
#else
# define machine_is_koi()	(0)
#endif

#ifdef CONFIG_MACH_TS4800
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TS4800
# endif
# define machine_is_ts4800()	(machine_arch_type == MACH_TYPE_TS4800)
#else
# define machine_is_ts4800()	(0)
#endif

#ifdef CONFIG_MACH_TQMA9263
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TQMA9263
# endif
# define machine_is_tqma9263()	(machine_arch_type == MACH_TYPE_TQMA9263)
#else
# define machine_is_tqma9263()	(0)
#endif

#ifdef CONFIG_MACH_HOLIDAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HOLIDAY
# endif
# define machine_is_holiday()	(machine_arch_type == MACH_TYPE_HOLIDAY)
#else
# define machine_is_holiday()	(0)
#endif

#ifdef CONFIG_MACH_DMA6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DMA6410
# endif
# define machine_is_dma_6410()	(machine_arch_type == MACH_TYPE_DMA6410)
#else
# define machine_is_dma_6410()	(0)
#endif

#ifdef CONFIG_MACH_PCATS_OVERLAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCATS_OVERLAY
# endif
# define machine_is_pcats_overlay()	(machine_arch_type == MACH_TYPE_PCATS_OVERLAY)
#else
# define machine_is_pcats_overlay()	(0)
#endif

#ifdef CONFIG_MACH_HWGW6410
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HWGW6410
# endif
# define machine_is_hwgw6410()	(machine_arch_type == MACH_TYPE_HWGW6410)
#else
# define machine_is_hwgw6410()	(0)
#endif

#ifdef CONFIG_MACH_SHENZHOU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHENZHOU
# endif
# define machine_is_shenzhou()	(machine_arch_type == MACH_TYPE_SHENZHOU)
#else
# define machine_is_shenzhou()	(0)
#endif

#ifdef CONFIG_MACH_CWME9210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWME9210
# endif
# define machine_is_cwme9210()	(machine_arch_type == MACH_TYPE_CWME9210)
#else
# define machine_is_cwme9210()	(0)
#endif

#ifdef CONFIG_MACH_CWME9210JS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWME9210JS
# endif
# define machine_is_cwme9210js()	(machine_arch_type == MACH_TYPE_CWME9210JS)
#else
# define machine_is_cwme9210js()	(0)
#endif

#ifdef CONFIG_MACH_PGS_SITARA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PGS_SITARA
# endif
# define machine_is_pgs_v1()	(machine_arch_type == MACH_TYPE_PGS_SITARA)
#else
# define machine_is_pgs_v1()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI_T20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI_T20
# endif
# define machine_is_colibri_t20()	(machine_arch_type == MACH_TYPE_COLIBRI_T20)
#else
# define machine_is_colibri_t20()	(0)
#endif

#ifdef CONFIG_MACH_W21
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_W21
# endif
# define machine_is_w21()	(machine_arch_type == MACH_TYPE_W21)
#else
# define machine_is_w21()	(0)
#endif

#ifdef CONFIG_MACH_POLYSAT1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_POLYSAT1
# endif
# define machine_is_polysat1()	(machine_arch_type == MACH_TYPE_POLYSAT1)
#else
# define machine_is_polysat1()	(0)
#endif

#ifdef CONFIG_MACH_DATAWAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DATAWAY
# endif
# define machine_is_dataway()	(machine_arch_type == MACH_TYPE_DATAWAY)
#else
# define machine_is_dataway()	(0)
#endif

#ifdef CONFIG_MACH_COBRAL138
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COBRAL138
# endif
# define machine_is_cobral138()	(machine_arch_type == MACH_TYPE_COBRAL138)
#else
# define machine_is_cobral138()	(0)
#endif

#ifdef CONFIG_MACH_ROVERPCS8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERPCS8
# endif
# define machine_is_roverpcs8()	(machine_arch_type == MACH_TYPE_ROVERPCS8)
#else
# define machine_is_roverpcs8()	(0)
#endif

#ifdef CONFIG_MACH_MARVELC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARVELC
# endif
# define machine_is_marvelc()	(machine_arch_type == MACH_TYPE_MARVELC)
#else
# define machine_is_marvelc()	(0)
#endif

#ifdef CONFIG_MACH_NAVEFIHID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NAVEFIHID
# endif
# define machine_is_navefihid()	(machine_arch_type == MACH_TYPE_NAVEFIHID)
#else
# define machine_is_navefihid()	(0)
#endif

#ifdef CONFIG_MACH_DM365_CV100
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM365_CV100
# endif
# define machine_is_dm365_cv100()	(machine_arch_type == MACH_TYPE_DM365_CV100)
#else
# define machine_is_dm365_cv100()	(0)
#endif

#ifdef CONFIG_MACH_ABLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ABLE
# endif
# define machine_is_able()	(machine_arch_type == MACH_TYPE_ABLE)
#else
# define machine_is_able()	(0)
#endif

#ifdef CONFIG_MACH_LEGACY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LEGACY
# endif
# define machine_is_legacy()	(machine_arch_type == MACH_TYPE_LEGACY)
#else
# define machine_is_legacy()	(0)
#endif

#ifdef CONFIG_MACH_ICONG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ICONG
# endif
# define machine_is_icong()	(machine_arch_type == MACH_TYPE_ICONG)
#else
# define machine_is_icong()	(0)
#endif

#ifdef CONFIG_MACH_ROVER_G8
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVER_G8
# endif
# define machine_is_rover_g8()	(machine_arch_type == MACH_TYPE_ROVER_G8)
#else
# define machine_is_rover_g8()	(0)
#endif

#ifdef CONFIG_MACH_T5388P
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T5388P
# endif
# define machine_is_t5388p()	(machine_arch_type == MACH_TYPE_T5388P)
#else
# define machine_is_t5388p()	(0)
#endif

#ifdef CONFIG_MACH_DINGO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DINGO
# endif
# define machine_is_dingo()	(machine_arch_type == MACH_TYPE_DINGO)
#else
# define machine_is_dingo()	(0)
#endif

#ifdef CONFIG_MACH_GOFLEXHOME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GOFLEXHOME
# endif
# define machine_is_goflexhome()	(machine_arch_type == MACH_TYPE_GOFLEXHOME)
#else
# define machine_is_goflexhome()	(0)
#endif

#ifdef CONFIG_MACH_LANREADYFN511
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LANREADYFN511
# endif
# define machine_is_lanreadyfn511()	(machine_arch_type == MACH_TYPE_LANREADYFN511)
#else
# define machine_is_lanreadyfn511()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3_BAIA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3_BAIA
# endif
# define machine_is_omap3_baia()	(machine_arch_type == MACH_TYPE_OMAP3_BAIA)
#else
# define machine_is_omap3_baia()	(0)
#endif

#ifdef CONFIG_MACH_OMAP3SMARTDISPLAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP3SMARTDISPLAY
# endif
# define machine_is_omap3smartdisplay()	(machine_arch_type == MACH_TYPE_OMAP3SMARTDISPLAY)
#else
# define machine_is_omap3smartdisplay()	(0)
#endif

#ifdef CONFIG_MACH_XILINX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XILINX
# endif
# define machine_is_xilinx()	(machine_arch_type == MACH_TYPE_XILINX)
#else
# define machine_is_xilinx()	(0)
#endif

#ifdef CONFIG_MACH_A2F
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_A2F
# endif
# define machine_is_a2f()	(machine_arch_type == MACH_TYPE_A2F)
#else
# define machine_is_a2f()	(0)
#endif

#ifdef CONFIG_MACH_SKY25
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SKY25
# endif
# define machine_is_sky25()	(machine_arch_type == MACH_TYPE_SKY25)
#else
# define machine_is_sky25()	(0)
#endif

#ifdef CONFIG_MACH_CCMX53
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCMX53
# endif
# define machine_is_ccmx53()	(machine_arch_type == MACH_TYPE_CCMX53)
#else
# define machine_is_ccmx53()	(0)
#endif

#ifdef CONFIG_MACH_CCMX53JS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCMX53JS
# endif
# define machine_is_ccmx53js()	(machine_arch_type == MACH_TYPE_CCMX53JS)
#else
# define machine_is_ccmx53js()	(0)
#endif

#ifdef CONFIG_MACH_CCWMX53
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCWMX53
# endif
# define machine_is_ccwmx53()	(machine_arch_type == MACH_TYPE_CCWMX53)
#else
# define machine_is_ccwmx53()	(0)
#endif

#ifdef CONFIG_MACH_CCWMX53JS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CCWMX53JS
# endif
# define machine_is_ccwmx53js()	(machine_arch_type == MACH_TYPE_CCWMX53JS)
#else
# define machine_is_ccwmx53js()	(0)
#endif

#ifdef CONFIG_MACH_FRISMS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_FRISMS
# endif
# define machine_is_frisms()	(machine_arch_type == MACH_TYPE_FRISMS)
#else
# define machine_is_frisms()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X27A_FFA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X27A_FFA
# endif
# define machine_is_msm7x27a_ffa()	(machine_arch_type == MACH_TYPE_MSM7X27A_FFA)
#else
# define machine_is_msm7x27a_ffa()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X27A_SURF
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X27A_SURF
# endif
# define machine_is_msm7x27a_surf()	(machine_arch_type == MACH_TYPE_MSM7X27A_SURF)
#else
# define machine_is_msm7x27a_surf()	(0)
#endif

#ifdef CONFIG_MACH_MSM7X27A_RUMI3
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM7X27A_RUMI3
# endif
# define machine_is_msm7x27a_rumi3()	(machine_arch_type == MACH_TYPE_MSM7X27A_RUMI3)
#else
# define machine_is_msm7x27a_rumi3()	(0)
#endif

#ifdef CONFIG_MACH_DIMMSAM9G20
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DIMMSAM9G20
# endif
# define machine_is_dimmsam9g20()	(machine_arch_type == MACH_TYPE_DIMMSAM9G20)
#else
# define machine_is_dimmsam9g20()	(0)
#endif

#ifdef CONFIG_MACH_DIMM_IMX28
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DIMM_IMX28
# endif
# define machine_is_dimm_imx28()	(machine_arch_type == MACH_TYPE_DIMM_IMX28)
#else
# define machine_is_dimm_imx28()	(0)
#endif

#ifdef CONFIG_MACH_AMK_A4
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AMK_A4
# endif
# define machine_is_amk_a4()	(machine_arch_type == MACH_TYPE_AMK_A4)
#else
# define machine_is_amk_a4()	(0)
#endif

#ifdef CONFIG_MACH_GNET_SGME
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GNET_SGME
# endif
# define machine_is_gnet_sgme()	(machine_arch_type == MACH_TYPE_GNET_SGME)
#else
# define machine_is_gnet_sgme()	(0)
#endif

#ifdef CONFIG_MACH_SHOOTER_U
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHOOTER_U
# endif
# define machine_is_shooter_u()	(machine_arch_type == MACH_TYPE_SHOOTER_U)
#else
# define machine_is_shooter_u()	(0)
#endif

#ifdef CONFIG_MACH_VMX53
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VMX53
# endif
# define machine_is_vmx53()	(machine_arch_type == MACH_TYPE_VMX53)
#else
# define machine_is_vmx53()	(0)
#endif

#ifdef CONFIG_MACH_RHINO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RHINO
# endif
# define machine_is_rhino()	(machine_arch_type == MACH_TYPE_RHINO)
#else
# define machine_is_rhino()	(0)
#endif

#ifdef CONFIG_MACH_ARMLEX4210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARMLEX4210
# endif
# define machine_is_armlex4210()	(machine_arch_type == MACH_TYPE_ARMLEX4210)
#else
# define machine_is_armlex4210()	(0)
#endif

#ifdef CONFIG_MACH_SWARCOEXTMODEM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SWARCOEXTMODEM
# endif
# define machine_is_swarcoextmodem()	(machine_arch_type == MACH_TYPE_SWARCOEXTMODEM)
#else
# define machine_is_swarcoextmodem()	(0)
#endif

#ifdef CONFIG_MACH_PCM049
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PCM049
# endif
# define machine_is_pcm049()	(machine_arch_type == MACH_TYPE_PCM049)
#else
# define machine_is_pcm049()	(0)
#endif

#ifdef CONFIG_MACH_VIGOR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIGOR
# endif
# define machine_is_vigor()	(machine_arch_type == MACH_TYPE_VIGOR)
#else
# define machine_is_vigor()	(0)
#endif

#ifdef CONFIG_MACH_OSLO_AMUNDSEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OSLO_AMUNDSEN
# endif
# define machine_is_oslo_amundsen()	(machine_arch_type == MACH_TYPE_OSLO_AMUNDSEN)
#else
# define machine_is_oslo_amundsen()	(0)
#endif

#ifdef CONFIG_MACH_GSL_DIAMOND
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GSL_DIAMOND
# endif
# define machine_is_gsl_diamond()	(machine_arch_type == MACH_TYPE_GSL_DIAMOND)
#else
# define machine_is_gsl_diamond()	(0)
#endif

#ifdef CONFIG_MACH_CV2201
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CV2201
# endif
# define machine_is_cv2201()	(machine_arch_type == MACH_TYPE_CV2201)
#else
# define machine_is_cv2201()	(0)
#endif

#ifdef CONFIG_MACH_CV2202
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CV2202
# endif
# define machine_is_cv2202()	(machine_arch_type == MACH_TYPE_CV2202)
#else
# define machine_is_cv2202()	(0)
#endif

#ifdef CONFIG_MACH_CV2203
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CV2203
# endif
# define machine_is_cv2203()	(machine_arch_type == MACH_TYPE_CV2203)
#else
# define machine_is_cv2203()	(0)
#endif

#ifdef CONFIG_MACH_VIT_IBOX
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIT_IBOX
# endif
# define machine_is_vit_ibox()	(machine_arch_type == MACH_TYPE_VIT_IBOX)
#else
# define machine_is_vit_ibox()	(0)
#endif

#ifdef CONFIG_MACH_DM6441_ESP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM6441_ESP
# endif
# define machine_is_dm6441_esp()	(machine_arch_type == MACH_TYPE_DM6441_ESP)
#else
# define machine_is_dm6441_esp()	(0)
#endif

#ifdef CONFIG_MACH_AT91SAM9X5EK
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AT91SAM9X5EK
# endif
# define machine_is_at91sam9x5ek()	(machine_arch_type == MACH_TYPE_AT91SAM9X5EK)
#else
# define machine_is_at91sam9x5ek()	(0)
#endif

#ifdef CONFIG_MACH_LIBRA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_LIBRA
# endif
# define machine_is_libra()	(machine_arch_type == MACH_TYPE_LIBRA)
#else
# define machine_is_libra()	(0)
#endif

#ifdef CONFIG_MACH_EASYCRRH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EASYCRRH
# endif
# define machine_is_easycrrh()	(machine_arch_type == MACH_TYPE_EASYCRRH)
#else
# define machine_is_easycrrh()	(0)
#endif

#ifdef CONFIG_MACH_TRIPEL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRIPEL
# endif
# define machine_is_tripel()	(machine_arch_type == MACH_TYPE_TRIPEL)
#else
# define machine_is_tripel()	(0)
#endif

#ifdef CONFIG_MACH_ENDIAN_MINI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ENDIAN_MINI
# endif
# define machine_is_endian_mini()	(machine_arch_type == MACH_TYPE_ENDIAN_MINI)
#else
# define machine_is_endian_mini()	(0)
#endif

#ifdef CONFIG_MACH_XILINX_EP107
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XILINX_EP107
# endif
# define machine_is_xilinx_ep107()	(machine_arch_type == MACH_TYPE_XILINX_EP107)
#else
# define machine_is_xilinx_ep107()	(0)
#endif

#ifdef CONFIG_MACH_NURI
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NURI
# endif
# define machine_is_nuri()	(machine_arch_type == MACH_TYPE_NURI)
#else
# define machine_is_nuri()	(0)
#endif

#ifdef CONFIG_MACH_JANUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_JANUS
# endif
# define machine_is_janus()	(machine_arch_type == MACH_TYPE_JANUS)
#else
# define machine_is_janus()	(0)
#endif

#ifdef CONFIG_MACH_DDNAS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DDNAS
# endif
# define machine_is_ddnas()	(machine_arch_type == MACH_TYPE_DDNAS)
#else
# define machine_is_ddnas()	(0)
#endif

#ifdef CONFIG_MACH_TAG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAG
# endif
# define machine_is_tag()	(machine_arch_type == MACH_TYPE_TAG)
#else
# define machine_is_tag()	(0)
#endif

#ifdef CONFIG_MACH_TAGW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TAGW
# endif
# define machine_is_tagw()	(machine_arch_type == MACH_TYPE_TAGW)
#else
# define machine_is_tagw()	(0)
#endif

#ifdef CONFIG_MACH_NITROGEN_VM_IMX51
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NITROGEN_VM_IMX51
# endif
# define machine_is_nitrogen_vm_imx51()	(machine_arch_type == MACH_TYPE_NITROGEN_VM_IMX51)
#else
# define machine_is_nitrogen_vm_imx51()	(0)
#endif

#ifdef CONFIG_MACH_VIPRINET
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VIPRINET
# endif
# define machine_is_viprinet()	(machine_arch_type == MACH_TYPE_VIPRINET)
#else
# define machine_is_viprinet()	(0)
#endif

#ifdef CONFIG_MACH_BOCKW
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BOCKW
# endif
# define machine_is_bockw()	(machine_arch_type == MACH_TYPE_BOCKW)
#else
# define machine_is_bockw()	(0)
#endif

#ifdef CONFIG_MACH_EVA2000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EVA2000
# endif
# define machine_is_eva2000()	(machine_arch_type == MACH_TYPE_EVA2000)
#else
# define machine_is_eva2000()	(0)
#endif

#ifdef CONFIG_MACH_STEELYARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_STEELYARD
# endif
# define machine_is_steelyard()	(machine_arch_type == MACH_TYPE_STEELYARD)
#else
# define machine_is_steelyard()	(0)
#endif

#ifdef CONFIG_MACH_MACH_SDH001
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MACH_SDH001
# endif
# define machine_is_sdh001()	(machine_arch_type == MACH_TYPE_MACH_SDH001)
#else
# define machine_is_sdh001()	(0)
#endif

#ifdef CONFIG_MACH_NSSLSBOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NSSLSBOARD
# endif
# define machine_is_nsslsboard()	(machine_arch_type == MACH_TYPE_NSSLSBOARD)
#else
# define machine_is_nsslsboard()	(0)
#endif

#ifdef CONFIG_MACH_GENEVA_B5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GENEVA_B5
# endif
# define machine_is_geneva_b5()	(machine_arch_type == MACH_TYPE_GENEVA_B5)
#else
# define machine_is_geneva_b5()	(0)
#endif

#ifdef CONFIG_MACH_SPEAR1340
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SPEAR1340
# endif
# define machine_is_spear1340()	(machine_arch_type == MACH_TYPE_SPEAR1340)
#else
# define machine_is_spear1340()	(0)
#endif

#ifdef CONFIG_MACH_REXMAS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_REXMAS
# endif
# define machine_is_rexmas()	(machine_arch_type == MACH_TYPE_REXMAS)
#else
# define machine_is_rexmas()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_CDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_CDP
# endif
# define machine_is_msm8960_cdp()	(machine_arch_type == MACH_TYPE_MSM8960_CDP)
#else
# define machine_is_msm8960_cdp()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_MDP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_MDP
# endif
# define machine_is_msm8960_mdp()	(machine_arch_type == MACH_TYPE_MSM8960_MDP)
#else
# define machine_is_msm8960_mdp()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_FLUID
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_FLUID
# endif
# define machine_is_msm8960_fluid()	(machine_arch_type == MACH_TYPE_MSM8960_FLUID)
#else
# define machine_is_msm8960_fluid()	(0)
#endif

#ifdef CONFIG_MACH_MSM8960_APQ
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MSM8960_APQ
# endif
# define machine_is_msm8960_apq()	(machine_arch_type == MACH_TYPE_MSM8960_APQ)
#else
# define machine_is_msm8960_apq()	(0)
#endif

#ifdef CONFIG_MACH_HELIOS_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HELIOS_V2
# endif
# define machine_is_helios_v2()	(machine_arch_type == MACH_TYPE_HELIOS_V2)
#else
# define machine_is_helios_v2()	(0)
#endif

#ifdef CONFIG_MACH_MIF10P
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MIF10P
# endif
# define machine_is_mif10p()	(machine_arch_type == MACH_TYPE_MIF10P)
#else
# define machine_is_mif10p()	(0)
#endif

#ifdef CONFIG_MACH_IAM28
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_IAM28
# endif
# define machine_is_iam28()	(machine_arch_type == MACH_TYPE_IAM28)
#else
# define machine_is_iam28()	(0)
#endif

#ifdef CONFIG_MACH_PICASSO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICASSO
# endif
# define machine_is_picasso()	(machine_arch_type == MACH_TYPE_PICASSO)
#else
# define machine_is_picasso()	(0)
#endif

#ifdef CONFIG_MACH_MR301A
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MR301A
# endif
# define machine_is_mr301a()	(machine_arch_type == MACH_TYPE_MR301A)
#else
# define machine_is_mr301a()	(0)
#endif

#ifdef CONFIG_MACH_NOTLE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NOTLE
# endif
# define machine_is_notle()	(machine_arch_type == MACH_TYPE_NOTLE)
#else
# define machine_is_notle()	(0)
#endif

#ifdef CONFIG_MACH_EELX2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EELX2
# endif
# define machine_is_eelx2()	(machine_arch_type == MACH_TYPE_EELX2)
#else
# define machine_is_eelx2()	(0)
#endif

#ifdef CONFIG_MACH_MOON
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MOON
# endif
# define machine_is_moon()	(machine_arch_type == MACH_TYPE_MOON)
#else
# define machine_is_moon()	(0)
#endif

#ifdef CONFIG_MACH_RUBY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_RUBY
# endif
# define machine_is_ruby()	(machine_arch_type == MACH_TYPE_RUBY)
#else
# define machine_is_ruby()	(0)
#endif

#ifdef CONFIG_MACH_GOLDENGATE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GOLDENGATE
# endif
# define machine_is_goldengate()	(machine_arch_type == MACH_TYPE_GOLDENGATE)
#else
# define machine_is_goldengate()	(0)
#endif

#ifdef CONFIG_MACH_CTBU_GEN2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CTBU_GEN2
# endif
# define machine_is_ctbu_gen2()	(machine_arch_type == MACH_TYPE_CTBU_GEN2)
#else
# define machine_is_ctbu_gen2()	(0)
#endif

#ifdef CONFIG_MACH_KMP_AM17_01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KMP_AM17_01
# endif
# define machine_is_kmp_am17_01()	(machine_arch_type == MACH_TYPE_KMP_AM17_01)
#else
# define machine_is_kmp_am17_01()	(0)
#endif

#ifdef CONFIG_MACH_WTPLUG
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WTPLUG
# endif
# define machine_is_wtplug()	(machine_arch_type == MACH_TYPE_WTPLUG)
#else
# define machine_is_wtplug()	(0)
#endif

#ifdef CONFIG_MACH_MX27SU2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX27SU2
# endif
# define machine_is_mx27su2()	(machine_arch_type == MACH_TYPE_MX27SU2)
#else
# define machine_is_mx27su2()	(0)
#endif

#ifdef CONFIG_MACH_NB31
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NB31
# endif
# define machine_is_nb31()	(machine_arch_type == MACH_TYPE_NB31)
#else
# define machine_is_nb31()	(0)
#endif

#ifdef CONFIG_MACH_HJSDU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HJSDU
# endif
# define machine_is_hjsdu()	(machine_arch_type == MACH_TYPE_HJSDU)
#else
# define machine_is_hjsdu()	(0)
#endif

#ifdef CONFIG_MACH_TD3_REV1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TD3_REV1
# endif
# define machine_is_td3_rev1()	(machine_arch_type == MACH_TYPE_TD3_REV1)
#else
# define machine_is_td3_rev1()	(0)
#endif

#ifdef CONFIG_MACH_EAG_CI4000
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EAG_CI4000
# endif
# define machine_is_eag_ci4000()	(machine_arch_type == MACH_TYPE_EAG_CI4000)
#else
# define machine_is_eag_ci4000()	(0)
#endif

#ifdef CONFIG_MACH_NET5BIG_NAND_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NET5BIG_NAND_V2
# endif
# define machine_is_net5big_nand_v2()	(machine_arch_type == MACH_TYPE_NET5BIG_NAND_V2)
#else
# define machine_is_net5big_nand_v2()	(0)
#endif

#ifdef CONFIG_MACH_CPX2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CPX2
# endif
# define machine_is_cpx2()	(machine_arch_type == MACH_TYPE_CPX2)
#else
# define machine_is_cpx2()	(0)
#endif

#ifdef CONFIG_MACH_NET2BIG_NAND_V2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NET2BIG_NAND_V2
# endif
# define machine_is_net2big_nand_v2()	(machine_arch_type == MACH_TYPE_NET2BIG_NAND_V2)
#else
# define machine_is_net2big_nand_v2()	(0)
#endif

#ifdef CONFIG_MACH_ECUV5
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ECUV5
# endif
# define machine_is_ecuv5()	(machine_arch_type == MACH_TYPE_ECUV5)
#else
# define machine_is_ecuv5()	(0)
#endif

#ifdef CONFIG_MACH_HSGX6D
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_HSGX6D
# endif
# define machine_is_hsgx6d()	(machine_arch_type == MACH_TYPE_HSGX6D)
#else
# define machine_is_hsgx6d()	(0)
#endif

#ifdef CONFIG_MACH_DAWAD7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DAWAD7
# endif
# define machine_is_dawad7()	(machine_arch_type == MACH_TYPE_DAWAD7)
#else
# define machine_is_dawad7()	(0)
#endif

#ifdef CONFIG_MACH_SAM9REPEATER
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SAM9REPEATER
# endif
# define machine_is_sam9repeater()	(machine_arch_type == MACH_TYPE_SAM9REPEATER)
#else
# define machine_is_sam9repeater()	(0)
#endif

#ifdef CONFIG_MACH_GT_I5700
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_GT_I5700
# endif
# define machine_is_gt_i5700()	(machine_arch_type == MACH_TYPE_GT_I5700)
#else
# define machine_is_gt_i5700()	(0)
#endif

#ifdef CONFIG_MACH_CTERA_PLUG_C2
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CTERA_PLUG_C2
# endif
# define machine_is_ctera_plug_c2()	(machine_arch_type == MACH_TYPE_CTERA_PLUG_C2)
#else
# define machine_is_ctera_plug_c2()	(0)
#endif

#ifdef CONFIG_MACH_MARVELCT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MARVELCT
# endif
# define machine_is_marvelct()	(machine_arch_type == MACH_TYPE_MARVELCT)
#else
# define machine_is_marvelct()	(0)
#endif

#ifdef CONFIG_MACH_AG11005
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_AG11005
# endif
# define machine_is_ag11005()	(machine_arch_type == MACH_TYPE_AG11005)
#else
# define machine_is_ag11005()	(0)
#endif

#ifdef CONFIG_MACH_VANGOGH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VANGOGH
# endif
# define machine_is_vangogh()	(machine_arch_type == MACH_TYPE_VANGOGH)
#else
# define machine_is_vangogh()	(0)
#endif

#ifdef CONFIG_MACH_MATRIX505
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MATRIX505
# endif
# define machine_is_matrix505()	(machine_arch_type == MACH_TYPE_MATRIX505)
#else
# define machine_is_matrix505()	(0)
#endif

#ifdef CONFIG_MACH_OCE_NIGMA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OCE_NIGMA
# endif
# define machine_is_oce_nigma()	(machine_arch_type == MACH_TYPE_OCE_NIGMA)
#else
# define machine_is_oce_nigma()	(0)
#endif

#ifdef CONFIG_MACH_T55
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_T55
# endif
# define machine_is_t55()	(machine_arch_type == MACH_TYPE_T55)
#else
# define machine_is_t55()	(0)
#endif

#ifdef CONFIG_MACH_BIO3K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BIO3K
# endif
# define machine_is_bio3k()	(machine_arch_type == MACH_TYPE_BIO3K)
#else
# define machine_is_bio3k()	(0)
#endif

#ifdef CONFIG_MACH_EXPRESSCT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPRESSCT
# endif
# define machine_is_expressct()	(machine_arch_type == MACH_TYPE_EXPRESSCT)
#else
# define machine_is_expressct()	(0)
#endif

#ifdef CONFIG_MACH_CARDHU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CARDHU
# endif
# define machine_is_cardhu()	(machine_arch_type == MACH_TYPE_CARDHU)
#else
# define machine_is_cardhu()	(0)
#endif

#ifdef CONFIG_MACH_ARUBA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARUBA
# endif
# define machine_is_aruba()	(machine_arch_type == MACH_TYPE_ARUBA)
#else
# define machine_is_aruba()	(0)
#endif

#ifdef CONFIG_MACH_BONAIRE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BONAIRE
# endif
# define machine_is_bonaire()	(machine_arch_type == MACH_TYPE_BONAIRE)
#else
# define machine_is_bonaire()	(0)
#endif

#ifdef CONFIG_MACH_NUC700EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC700EVB
# endif
# define machine_is_nuc700evb()	(machine_arch_type == MACH_TYPE_NUC700EVB)
#else
# define machine_is_nuc700evb()	(0)
#endif

#ifdef CONFIG_MACH_NUC710EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC710EVB
# endif
# define machine_is_nuc710evb()	(machine_arch_type == MACH_TYPE_NUC710EVB)
#else
# define machine_is_nuc710evb()	(0)
#endif

#ifdef CONFIG_MACH_NUC740EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC740EVB
# endif
# define machine_is_nuc740evb()	(machine_arch_type == MACH_TYPE_NUC740EVB)
#else
# define machine_is_nuc740evb()	(0)
#endif

#ifdef CONFIG_MACH_NUC745EVB
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NUC745EVB
# endif
# define machine_is_nuc745evb()	(machine_arch_type == MACH_TYPE_NUC745EVB)
#else
# define machine_is_nuc745evb()	(0)
#endif

#ifdef CONFIG_MACH_TRANSCEDE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRANSCEDE
# endif
# define machine_is_transcede()	(machine_arch_type == MACH_TYPE_TRANSCEDE)
#else
# define machine_is_transcede()	(0)
#endif

#ifdef CONFIG_MACH_MORA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MORA
# endif
# define machine_is_mora()	(machine_arch_type == MACH_TYPE_MORA)
#else
# define machine_is_mora()	(0)
#endif

#ifdef CONFIG_MACH_NDA_EVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_NDA_EVM
# endif
# define machine_is_nda_evm()	(machine_arch_type == MACH_TYPE_NDA_EVM)
#else
# define machine_is_nda_evm()	(0)
#endif

#ifdef CONFIG_MACH_TIMU
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TIMU
# endif
# define machine_is_timu()	(machine_arch_type == MACH_TYPE_TIMU)
#else
# define machine_is_timu()	(0)
#endif

#ifdef CONFIG_MACH_EXPRESSH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EXPRESSH
# endif
# define machine_is_expressh()	(machine_arch_type == MACH_TYPE_EXPRESSH)
#else
# define machine_is_expressh()	(0)
#endif

#ifdef CONFIG_MACH_VERIDIS_A300
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_VERIDIS_A300
# endif
# define machine_is_veridis_a300()	(machine_arch_type == MACH_TYPE_VERIDIS_A300)
#else
# define machine_is_veridis_a300()	(0)
#endif

#ifdef CONFIG_MACH_DM368_LEOPARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DM368_LEOPARD
# endif
# define machine_is_dm368_leopard()	(machine_arch_type == MACH_TYPE_DM368_LEOPARD)
#else
# define machine_is_dm368_leopard()	(0)
#endif

#ifdef CONFIG_MACH_OMAP_MCOP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_OMAP_MCOP
# endif
# define machine_is_omap_mcop()	(machine_arch_type == MACH_TYPE_OMAP_MCOP)
#else
# define machine_is_omap_mcop()	(0)
#endif

#ifdef CONFIG_MACH_TRITIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TRITIP
# endif
# define machine_is_tritip()	(machine_arch_type == MACH_TYPE_TRITIP)
#else
# define machine_is_tritip()	(0)
#endif

#ifdef CONFIG_MACH_SM1K
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SM1K
# endif
# define machine_is_sm1k()	(machine_arch_type == MACH_TYPE_SM1K)
#else
# define machine_is_sm1k()	(0)
#endif

#ifdef CONFIG_MACH_MONCH
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MONCH
# endif
# define machine_is_monch()	(machine_arch_type == MACH_TYPE_MONCH)
#else
# define machine_is_monch()	(0)
#endif

#ifdef CONFIG_MACH_CURACAO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CURACAO
# endif
# define machine_is_curacao()	(machine_arch_type == MACH_TYPE_CURACAO)
#else
# define machine_is_curacao()	(0)
#endif

#ifdef CONFIG_MACH_ORIGEN
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ORIGEN
# endif
# define machine_is_origen()	(machine_arch_type == MACH_TYPE_ORIGEN)
#else
# define machine_is_origen()	(0)
#endif

#ifdef CONFIG_MACH_EPC10
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_EPC10
# endif
# define machine_is_epc10()	(machine_arch_type == MACH_TYPE_EPC10)
#else
# define machine_is_epc10()	(0)
#endif

#ifdef CONFIG_MACH_SGH_I740
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SGH_I740
# endif
# define machine_is_sgh_i740()	(machine_arch_type == MACH_TYPE_SGH_I740)
#else
# define machine_is_sgh_i740()	(0)
#endif

#ifdef CONFIG_MACH_TUNA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TUNA
# endif
# define machine_is_tuna()	(machine_arch_type == MACH_TYPE_TUNA)
#else
# define machine_is_tuna()	(0)
#endif

#ifdef CONFIG_MACH_MX51_TULIP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_TULIP
# endif
# define machine_is_mx51_tulip()	(machine_arch_type == MACH_TYPE_MX51_TULIP)
#else
# define machine_is_mx51_tulip()	(0)
#endif

#ifdef CONFIG_MACH_MX51_ASTER7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_ASTER7
# endif
# define machine_is_mx51_aster7()	(machine_arch_type == MACH_TYPE_MX51_ASTER7)
#else
# define machine_is_mx51_aster7()	(0)
#endif

#ifdef CONFIG_MACH_ACRO37XBRD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACRO37XBRD
# endif
# define machine_is_acro37xbrd()	(machine_arch_type == MACH_TYPE_ACRO37XBRD)
#else
# define machine_is_acro37xbrd()	(0)
#endif

#ifdef CONFIG_MACH_ELKE
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ELKE
# endif
# define machine_is_elke()	(machine_arch_type == MACH_TYPE_ELKE)
#else
# define machine_is_elke()	(0)
#endif

#ifdef CONFIG_MACH_SBC6000X
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SBC6000X
# endif
# define machine_is_sbc6000x()	(machine_arch_type == MACH_TYPE_SBC6000X)
#else
# define machine_is_sbc6000x()	(0)
#endif

#ifdef CONFIG_MACH_R1801E
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_R1801E
# endif
# define machine_is_r1801e()	(machine_arch_type == MACH_TYPE_R1801E)
#else
# define machine_is_r1801e()	(0)
#endif

#ifdef CONFIG_MACH_H1600
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_H1600
# endif
# define machine_is_h1600()	(machine_arch_type == MACH_TYPE_H1600)
#else
# define machine_is_h1600()	(0)
#endif

#ifdef CONFIG_MACH_MINI210
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MINI210
# endif
# define machine_is_mini210()	(machine_arch_type == MACH_TYPE_MINI210)
#else
# define machine_is_mini210()	(0)
#endif

#ifdef CONFIG_MACH_MINI8168
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MINI8168
# endif
# define machine_is_mini8168()	(machine_arch_type == MACH_TYPE_MINI8168)
#else
# define machine_is_mini8168()	(0)
#endif

#ifdef CONFIG_MACH_PC7308
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PC7308
# endif
# define machine_is_pc7308()	(machine_arch_type == MACH_TYPE_PC7308)
#else
# define machine_is_pc7308()	(0)
#endif

#ifdef CONFIG_MACH_KMM2M01
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_KMM2M01
# endif
# define machine_is_kmm2m01()	(machine_arch_type == MACH_TYPE_KMM2M01)
#else
# define machine_is_kmm2m01()	(0)
#endif

#ifdef CONFIG_MACH_MX51EREBUS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51EREBUS
# endif
# define machine_is_mx51erebus()	(machine_arch_type == MACH_TYPE_MX51EREBUS)
#else
# define machine_is_mx51erebus()	(0)
#endif

#ifdef CONFIG_MACH_WM8650REFBOARD
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_WM8650REFBOARD
# endif
# define machine_is_wm8650refboard()	(machine_arch_type == MACH_TYPE_WM8650REFBOARD)
#else
# define machine_is_wm8650refboard()	(0)
#endif

#ifdef CONFIG_MACH_TUXRAIL
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_TUXRAIL
# endif
# define machine_is_tuxrail()	(machine_arch_type == MACH_TYPE_TUXRAIL)
#else
# define machine_is_tuxrail()	(0)
#endif

#ifdef CONFIG_MACH_ARTHUR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ARTHUR
# endif
# define machine_is_arthur()	(machine_arch_type == MACH_TYPE_ARTHUR)
#else
# define machine_is_arthur()	(0)
#endif

#ifdef CONFIG_MACH_DOORBOY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DOORBOY
# endif
# define machine_is_doorboy()	(machine_arch_type == MACH_TYPE_DOORBOY)
#else
# define machine_is_doorboy()	(0)
#endif

#ifdef CONFIG_MACH_XARINA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_XARINA
# endif
# define machine_is_xarina()	(machine_arch_type == MACH_TYPE_XARINA)
#else
# define machine_is_xarina()	(0)
#endif

#ifdef CONFIG_MACH_ROVERX7
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ROVERX7
# endif
# define machine_is_roverx7()	(machine_arch_type == MACH_TYPE_ROVERX7)
#else
# define machine_is_roverx7()	(0)
#endif

#ifdef CONFIG_MACH_SDVR
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SDVR
# endif
# define machine_is_sdvr()	(machine_arch_type == MACH_TYPE_SDVR)
#else
# define machine_is_sdvr()	(0)
#endif

#ifdef CONFIG_MACH_ACER_MAYA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACER_MAYA
# endif
# define machine_is_acer_maya()	(machine_arch_type == MACH_TYPE_ACER_MAYA)
#else
# define machine_is_acer_maya()	(0)
#endif

#ifdef CONFIG_MACH_PICO
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_PICO
# endif
# define machine_is_pico()	(machine_arch_type == MACH_TYPE_PICO)
#else
# define machine_is_pico()	(0)
#endif

#ifdef CONFIG_MACH_CWMX233
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWMX233
# endif
# define machine_is_cwmx233()	(machine_arch_type == MACH_TYPE_CWMX233)
#else
# define machine_is_cwmx233()	(0)
#endif

#ifdef CONFIG_MACH_CWAM1808
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWAM1808
# endif
# define machine_is_cwam1808()	(machine_arch_type == MACH_TYPE_CWAM1808)
#else
# define machine_is_cwam1808()	(0)
#endif

#ifdef CONFIG_MACH_CWDM365
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_CWDM365
# endif
# define machine_is_cwdm365()	(machine_arch_type == MACH_TYPE_CWDM365)
#else
# define machine_is_cwdm365()	(0)
#endif

#ifdef CONFIG_MACH_MX51_MORAY
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_MX51_MORAY
# endif
# define machine_is_mx51_moray()	(machine_arch_type == MACH_TYPE_MX51_MORAY)
#else
# define machine_is_mx51_moray()	(0)
#endif

#ifdef CONFIG_MACH_THALES_CBC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_THALES_CBC
# endif
# define machine_is_thales_cbc()	(machine_arch_type == MACH_TYPE_THALES_CBC)
#else
# define machine_is_thales_cbc()	(0)
#endif

#ifdef CONFIG_MACH_BLUEPOINT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLUEPOINT
# endif
# define machine_is_bluepoint()	(machine_arch_type == MACH_TYPE_BLUEPOINT)
#else
# define machine_is_bluepoint()	(0)
#endif

#ifdef CONFIG_MACH_DIR665
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_DIR665
# endif
# define machine_is_dir665()	(machine_arch_type == MACH_TYPE_DIR665)
#else
# define machine_is_dir665()	(0)
#endif

#ifdef CONFIG_MACH_ACMEROVER1
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ACMEROVER1
# endif
# define machine_is_acmerover1()	(machine_arch_type == MACH_TYPE_ACMEROVER1)
#else
# define machine_is_acmerover1()	(0)
#endif

#ifdef CONFIG_MACH_SHOOTER_CT
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_SHOOTER_CT
# endif
# define machine_is_shooter_ct()	(machine_arch_type == MACH_TYPE_SHOOTER_CT)
#else
# define machine_is_shooter_ct()	(0)
#endif

#ifdef CONFIG_MACH_BLISS
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLISS
# endif
# define machine_is_bliss()	(machine_arch_type == MACH_TYPE_BLISS)
#else
# define machine_is_bliss()	(0)
#endif

#ifdef CONFIG_MACH_BLISSC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_BLISSC
# endif
# define machine_is_blissc()	(machine_arch_type == MACH_TYPE_BLISSC)
#else
# define machine_is_blissc()	(0)
#endif

#ifdef CONFIG_MACH_THALES_ADC
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_THALES_ADC
# endif
# define machine_is_thales_adc()	(machine_arch_type == MACH_TYPE_THALES_ADC)
#else
# define machine_is_thales_adc()	(0)
#endif

#ifdef CONFIG_MACH_UBISYS_P9D_EVP
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_UBISYS_P9D_EVP
# endif
# define machine_is_ubisys_p9d_evp()	(machine_arch_type == MACH_TYPE_UBISYS_P9D_EVP)
#else
# define machine_is_ubisys_p9d_evp()	(0)
#endif

#ifdef CONFIG_MACH_ATDGP318
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_ATDGP318
# endif
# define machine_is_atdgp318()	(machine_arch_type == MACH_TYPE_ATDGP318)
#else
# define machine_is_atdgp318()	(0)
#endif

#ifdef CONFIG_MACH_OMAP5_SEVM
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type     __machine_arch_type
# else
#  define machine_arch_type     MACH_TYPE_OMAP5_SEVM
# endif
# define machine_is_omap5_sevm()      (machine_arch_type == MACH_TYPE_OMAP5_SEVM)
#else
# define machine_is_omap5_sevm()      (0)
#endif

#ifdef CONFIG_MACH_ARMADILLO800EVA
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type __machine_arch_type
# else
#  define machine_arch_type MACH_TYPE_ARMADILLO800EVA
# endif
# define machine_is_armadillo800eva()	(machine_arch_type == MACH_TYPE_ARMADILLO800EVA)
#else
# define machine_is_armadillo800eva()	(0)
#endif

#ifdef CONFIG_MACH_KZM9G
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type __machine_arch_type
# else
#  define machine_arch_type MACH_TYPE_KZM9G
# endif
# define machine_is_kzm9g()	(machine_arch_type == MACH_TYPE_KZM9G)
#else
# define machine_is_kzm9g()	(0)
#endif

#ifdef CONFIG_MACH_COLIBRI_T30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_COLIBRI_T30
# endif
# define machine_is_colibri_t30()	(machine_arch_type == MACH_TYPE_COLIBRI_T30)
#else
# define machine_is_colibri_t30()	(0)
#endif

#ifdef CONFIG_MACH_APALIS_T30
# ifdef machine_arch_type
#  undef machine_arch_type
#  define machine_arch_type	__machine_arch_type
# else
#  define machine_arch_type	MACH_TYPE_APALIS_T30
# endif
# define machine_is_apalis_t30()	(machine_arch_type == MACH_TYPE_APALIS_T30)
#else
# define machine_is_apalis_t30()	(0)
#endif

/*
 * These have not yet been registered
 */

#ifndef machine_arch_type
#define machine_arch_type	__machine_arch_type
#endif

#endif
